import { __commonJS, __require, __toESM, versions_default, parseList, getEnvConfig, JsPackageManagerFactory, useNpmWarning, getStorybookVersion, getStorybookVersionSpecifier, commandLog, installableProjectTypes, paddedLog, HandledError, codeLog, isNxProject, getPackageDetails, supportedTemplates, unsupportedTemplate, getCliDir, copyTemplate, getBabelDependencies, copyTemplateFiles, externalFrameworks } from './chunk-OEVQ57SW.mjs';
import program from 'commander';
import path9, { join, resolve, basename } from 'path';
import chalk12 from 'chalk';
import envinfo from 'envinfo';
import leven from 'leven';
import { sync } from 'read-pkg-up';
import { logger, instance } from '@storybook/node-logger';
import { addToGlobalContext, telemetry, getStorybookCoreVersion } from '@storybook/telemetry';
import prompts5 from 'prompts';
import { withTelemetry, buildDevStandalone, buildStaticStandalone } from '@storybook/core-server';
import dedent17, { dedent } from 'ts-dedent';
import fs from 'fs';
import findUp from 'find-up';
import semver from 'semver';
import fse, { pathExists, move, remove, writeFile, existsSync, readdir, createWriteStream, stat, readFile, readJson, writeJson } from 'fs-extra';
import ora from 'ora';
import detectIndent from 'detect-indent';
import { readConfig, writeConfig } from '@storybook/csf-tools';
import { sync as sync$1, spawn } from 'cross-spawn';
import { getStorybookInfo, loadMainConfig, cache, rendererPackages, frameworkPackages, commonGlobOptions, builderPackages } from '@storybook/core-common';
import { listCodemods, runCodemod } from '@storybook/codemod';
import tempy from 'tempy';
import glob from 'globby';
import { loadPartialConfigAsync } from '@babel/core';
import * as t from '@babel/types';
import { EOL } from 'os';
import puppeteerCore from 'puppeteer-core';
import express from 'express';
import getPort from 'get-port';
import { downloadTemplate } from 'giget';

// ../../node_modules/ansi-regex/index.js
var require_ansi_regex = __commonJS({
  "../../node_modules/ansi-regex/index.js"(exports, module) {
    module.exports = ({ onlyFirst = false } = {}) => {
      const pattern = [
        "[\\u001B\\u009B][[\\]()#;?]*(?:(?:(?:(?:;[-a-zA-Z\\d\\/#&.:=?%@~_]+)*|[a-zA-Z\\d]+(?:;[-a-zA-Z\\d\\/#&.:=?%@~_]*)*)?\\u0007)",
        "(?:(?:\\d{1,4}(?:;\\d{0,4})*)?[\\dA-PR-TZcf-ntqry=><~]))"
      ].join("|");
      return new RegExp(pattern, onlyFirst ? void 0 : "g");
    };
  }
});

// ../../node_modules/strip-ansi/index.js
var require_strip_ansi = __commonJS({
  "../../node_modules/strip-ansi/index.js"(exports, module) {
    var ansiRegex2 = require_ansi_regex();
    module.exports = (string) => typeof string === "string" ? string.replace(ansiRegex2(), "") : string;
  }
});

// ../../node_modules/is-fullwidth-code-point/index.js
var require_is_fullwidth_code_point = __commonJS({
  "../../node_modules/is-fullwidth-code-point/index.js"(exports, module) {
    var isFullwidthCodePoint = (codePoint) => {
      if (Number.isNaN(codePoint)) {
        return false;
      }
      if (codePoint >= 4352 && (codePoint <= 4447 || // Hangul Jamo
      codePoint === 9001 || // LEFT-POINTING ANGLE BRACKET
      codePoint === 9002 || // RIGHT-POINTING ANGLE BRACKET
      // CJK Radicals Supplement .. Enclosed CJK Letters and Months
      11904 <= codePoint && codePoint <= 12871 && codePoint !== 12351 || // Enclosed CJK Letters and Months .. CJK Unified Ideographs Extension A
      12880 <= codePoint && codePoint <= 19903 || // CJK Unified Ideographs .. Yi Radicals
      19968 <= codePoint && codePoint <= 42182 || // Hangul Jamo Extended-A
      43360 <= codePoint && codePoint <= 43388 || // Hangul Syllables
      44032 <= codePoint && codePoint <= 55203 || // CJK Compatibility Ideographs
      63744 <= codePoint && codePoint <= 64255 || // Vertical Forms
      65040 <= codePoint && codePoint <= 65049 || // CJK Compatibility Forms .. Small Form Variants
      65072 <= codePoint && codePoint <= 65131 || // Halfwidth and Fullwidth Forms
      65281 <= codePoint && codePoint <= 65376 || 65504 <= codePoint && codePoint <= 65510 || // Kana Supplement
      110592 <= codePoint && codePoint <= 110593 || // Enclosed Ideographic Supplement
      127488 <= codePoint && codePoint <= 127569 || // CJK Unified Ideographs Extension B .. Tertiary Ideographic Plane
      131072 <= codePoint && codePoint <= 262141)) {
        return true;
      }
      return false;
    };
    module.exports = isFullwidthCodePoint;
    module.exports.default = isFullwidthCodePoint;
  }
});

// ../../node_modules/string-width/node_modules/emoji-regex/index.js
var require_emoji_regex = __commonJS({
  "../../node_modules/string-width/node_modules/emoji-regex/index.js"(exports, module) {
    module.exports = function() {
      return /\uD83C\uDFF4\uDB40\uDC67\uDB40\uDC62(?:\uDB40\uDC65\uDB40\uDC6E\uDB40\uDC67|\uDB40\uDC73\uDB40\uDC63\uDB40\uDC74|\uDB40\uDC77\uDB40\uDC6C\uDB40\uDC73)\uDB40\uDC7F|\uD83D\uDC68(?:\uD83C\uDFFC\u200D(?:\uD83E\uDD1D\u200D\uD83D\uDC68\uD83C\uDFFB|\uD83C[\uDF3E\uDF73\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|\uD83C\uDFFF\u200D(?:\uD83E\uDD1D\u200D\uD83D\uDC68(?:\uD83C[\uDFFB-\uDFFE])|\uD83C[\uDF3E\uDF73\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|\uD83C\uDFFE\u200D(?:\uD83E\uDD1D\u200D\uD83D\uDC68(?:\uD83C[\uDFFB-\uDFFD])|\uD83C[\uDF3E\uDF73\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|\uD83C\uDFFD\u200D(?:\uD83E\uDD1D\u200D\uD83D\uDC68(?:\uD83C[\uDFFB\uDFFC])|\uD83C[\uDF3E\uDF73\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|\u200D(?:\u2764\uFE0F\u200D(?:\uD83D\uDC8B\u200D)?\uD83D\uDC68|(?:\uD83D[\uDC68\uDC69])\u200D(?:\uD83D\uDC66\u200D\uD83D\uDC66|\uD83D\uDC67\u200D(?:\uD83D[\uDC66\uDC67]))|\uD83D\uDC66\u200D\uD83D\uDC66|\uD83D\uDC67\u200D(?:\uD83D[\uDC66\uDC67])|(?:\uD83D[\uDC68\uDC69])\u200D(?:\uD83D[\uDC66\uDC67])|[\u2695\u2696\u2708]\uFE0F|\uD83D[\uDC66\uDC67]|\uD83C[\uDF3E\uDF73\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|(?:\uD83C\uDFFB\u200D[\u2695\u2696\u2708]|\uD83C\uDFFF\u200D[\u2695\u2696\u2708]|\uD83C\uDFFE\u200D[\u2695\u2696\u2708]|\uD83C\uDFFD\u200D[\u2695\u2696\u2708]|\uD83C\uDFFC\u200D[\u2695\u2696\u2708])\uFE0F|\uD83C\uDFFB\u200D(?:\uD83C[\uDF3E\uDF73\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|\uD83C[\uDFFB-\uDFFF])|(?:\uD83E\uDDD1\uD83C\uDFFB\u200D\uD83E\uDD1D\u200D\uD83E\uDDD1|\uD83D\uDC69\uD83C\uDFFC\u200D\uD83E\uDD1D\u200D\uD83D\uDC69)\uD83C\uDFFB|\uD83E\uDDD1(?:\uD83C\uDFFF\u200D\uD83E\uDD1D\u200D\uD83E\uDDD1(?:\uD83C[\uDFFB-\uDFFF])|\u200D\uD83E\uDD1D\u200D\uD83E\uDDD1)|(?:\uD83E\uDDD1\uD83C\uDFFE\u200D\uD83E\uDD1D\u200D\uD83E\uDDD1|\uD83D\uDC69\uD83C\uDFFF\u200D\uD83E\uDD1D\u200D(?:\uD83D[\uDC68\uDC69]))(?:\uD83C[\uDFFB-\uDFFE])|(?:\uD83E\uDDD1\uD83C\uDFFC\u200D\uD83E\uDD1D\u200D\uD83E\uDDD1|\uD83D\uDC69\uD83C\uDFFD\u200D\uD83E\uDD1D\u200D\uD83D\uDC69)(?:\uD83C[\uDFFB\uDFFC])|\uD83D\uDC69(?:\uD83C\uDFFE\u200D(?:\uD83E\uDD1D\u200D\uD83D\uDC68(?:\uD83C[\uDFFB-\uDFFD\uDFFF])|\uD83C[\uDF3E\uDF73\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|\uD83C\uDFFC\u200D(?:\uD83E\uDD1D\u200D\uD83D\uDC68(?:\uD83C[\uDFFB\uDFFD-\uDFFF])|\uD83C[\uDF3E\uDF73\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|\uD83C\uDFFB\u200D(?:\uD83E\uDD1D\u200D\uD83D\uDC68(?:\uD83C[\uDFFC-\uDFFF])|\uD83C[\uDF3E\uDF73\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|\uD83C\uDFFD\u200D(?:\uD83E\uDD1D\u200D\uD83D\uDC68(?:\uD83C[\uDFFB\uDFFC\uDFFE\uDFFF])|\uD83C[\uDF3E\uDF73\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|\u200D(?:\u2764\uFE0F\u200D(?:\uD83D\uDC8B\u200D(?:\uD83D[\uDC68\uDC69])|\uD83D[\uDC68\uDC69])|\uD83C[\uDF3E\uDF73\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|\uD83C\uDFFF\u200D(?:\uD83C[\uDF3E\uDF73\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD]))|\uD83D\uDC69\u200D\uD83D\uDC69\u200D(?:\uD83D\uDC66\u200D\uD83D\uDC66|\uD83D\uDC67\u200D(?:\uD83D[\uDC66\uDC67]))|(?:\uD83E\uDDD1\uD83C\uDFFD\u200D\uD83E\uDD1D\u200D\uD83E\uDDD1|\uD83D\uDC69\uD83C\uDFFE\u200D\uD83E\uDD1D\u200D\uD83D\uDC69)(?:\uD83C[\uDFFB-\uDFFD])|\uD83D\uDC69\u200D\uD83D\uDC66\u200D\uD83D\uDC66|\uD83D\uDC69\u200D\uD83D\uDC69\u200D(?:\uD83D[\uDC66\uDC67])|(?:\uD83D\uDC41\uFE0F\u200D\uD83D\uDDE8|\uD83D\uDC69(?:\uD83C\uDFFF\u200D[\u2695\u2696\u2708]|\uD83C\uDFFE\u200D[\u2695\u2696\u2708]|\uD83C\uDFFC\u200D[\u2695\u2696\u2708]|\uD83C\uDFFB\u200D[\u2695\u2696\u2708]|\uD83C\uDFFD\u200D[\u2695\u2696\u2708]|\u200D[\u2695\u2696\u2708])|(?:(?:\u26F9|\uD83C[\uDFCB\uDFCC]|\uD83D\uDD75)\uFE0F|\uD83D\uDC6F|\uD83E[\uDD3C\uDDDE\uDDDF])\u200D[\u2640\u2642]|(?:\u26F9|\uD83C[\uDFCB\uDFCC]|\uD83D\uDD75)(?:\uD83C[\uDFFB-\uDFFF])\u200D[\u2640\u2642]|(?:\uD83C[\uDFC3\uDFC4\uDFCA]|\uD83D[\uDC6E\uDC71\uDC73\uDC77\uDC81\uDC82\uDC86\uDC87\uDE45-\uDE47\uDE4B\uDE4D\uDE4E\uDEA3\uDEB4-\uDEB6]|\uD83E[\uDD26\uDD37-\uDD39\uDD3D\uDD3E\uDDB8\uDDB9\uDDCD-\uDDCF\uDDD6-\uDDDD])(?:(?:\uD83C[\uDFFB-\uDFFF])\u200D[\u2640\u2642]|\u200D[\u2640\u2642])|\uD83C\uDFF4\u200D\u2620)\uFE0F|\uD83D\uDC69\u200D\uD83D\uDC67\u200D(?:\uD83D[\uDC66\uDC67])|\uD83C\uDFF3\uFE0F\u200D\uD83C\uDF08|\uD83D\uDC15\u200D\uD83E\uDDBA|\uD83D\uDC69\u200D\uD83D\uDC66|\uD83D\uDC69\u200D\uD83D\uDC67|\uD83C\uDDFD\uD83C\uDDF0|\uD83C\uDDF4\uD83C\uDDF2|\uD83C\uDDF6\uD83C\uDDE6|[#\*0-9]\uFE0F\u20E3|\uD83C\uDDE7(?:\uD83C[\uDDE6\uDDE7\uDDE9-\uDDEF\uDDF1-\uDDF4\uDDF6-\uDDF9\uDDFB\uDDFC\uDDFE\uDDFF])|\uD83C\uDDF9(?:\uD83C[\uDDE6\uDDE8\uDDE9\uDDEB-\uDDED\uDDEF-\uDDF4\uDDF7\uDDF9\uDDFB\uDDFC\uDDFF])|\uD83C\uDDEA(?:\uD83C[\uDDE6\uDDE8\uDDEA\uDDEC\uDDED\uDDF7-\uDDFA])|\uD83E\uDDD1(?:\uD83C[\uDFFB-\uDFFF])|\uD83C\uDDF7(?:\uD83C[\uDDEA\uDDF4\uDDF8\uDDFA\uDDFC])|\uD83D\uDC69(?:\uD83C[\uDFFB-\uDFFF])|\uD83C\uDDF2(?:\uD83C[\uDDE6\uDDE8-\uDDED\uDDF0-\uDDFF])|\uD83C\uDDE6(?:\uD83C[\uDDE8-\uDDEC\uDDEE\uDDF1\uDDF2\uDDF4\uDDF6-\uDDFA\uDDFC\uDDFD\uDDFF])|\uD83C\uDDF0(?:\uD83C[\uDDEA\uDDEC-\uDDEE\uDDF2\uDDF3\uDDF5\uDDF7\uDDFC\uDDFE\uDDFF])|\uD83C\uDDED(?:\uD83C[\uDDF0\uDDF2\uDDF3\uDDF7\uDDF9\uDDFA])|\uD83C\uDDE9(?:\uD83C[\uDDEA\uDDEC\uDDEF\uDDF0\uDDF2\uDDF4\uDDFF])|\uD83C\uDDFE(?:\uD83C[\uDDEA\uDDF9])|\uD83C\uDDEC(?:\uD83C[\uDDE6\uDDE7\uDDE9-\uDDEE\uDDF1-\uDDF3\uDDF5-\uDDFA\uDDFC\uDDFE])|\uD83C\uDDF8(?:\uD83C[\uDDE6-\uDDEA\uDDEC-\uDDF4\uDDF7-\uDDF9\uDDFB\uDDFD-\uDDFF])|\uD83C\uDDEB(?:\uD83C[\uDDEE-\uDDF0\uDDF2\uDDF4\uDDF7])|\uD83C\uDDF5(?:\uD83C[\uDDE6\uDDEA-\uDDED\uDDF0-\uDDF3\uDDF7-\uDDF9\uDDFC\uDDFE])|\uD83C\uDDFB(?:\uD83C[\uDDE6\uDDE8\uDDEA\uDDEC\uDDEE\uDDF3\uDDFA])|\uD83C\uDDF3(?:\uD83C[\uDDE6\uDDE8\uDDEA-\uDDEC\uDDEE\uDDF1\uDDF4\uDDF5\uDDF7\uDDFA\uDDFF])|\uD83C\uDDE8(?:\uD83C[\uDDE6\uDDE8\uDDE9\uDDEB-\uDDEE\uDDF0-\uDDF5\uDDF7\uDDFA-\uDDFF])|\uD83C\uDDF1(?:\uD83C[\uDDE6-\uDDE8\uDDEE\uDDF0\uDDF7-\uDDFB\uDDFE])|\uD83C\uDDFF(?:\uD83C[\uDDE6\uDDF2\uDDFC])|\uD83C\uDDFC(?:\uD83C[\uDDEB\uDDF8])|\uD83C\uDDFA(?:\uD83C[\uDDE6\uDDEC\uDDF2\uDDF3\uDDF8\uDDFE\uDDFF])|\uD83C\uDDEE(?:\uD83C[\uDDE8-\uDDEA\uDDF1-\uDDF4\uDDF6-\uDDF9])|\uD83C\uDDEF(?:\uD83C[\uDDEA\uDDF2\uDDF4\uDDF5])|(?:\uD83C[\uDFC3\uDFC4\uDFCA]|\uD83D[\uDC6E\uDC71\uDC73\uDC77\uDC81\uDC82\uDC86\uDC87\uDE45-\uDE47\uDE4B\uDE4D\uDE4E\uDEA3\uDEB4-\uDEB6]|\uD83E[\uDD26\uDD37-\uDD39\uDD3D\uDD3E\uDDB8\uDDB9\uDDCD-\uDDCF\uDDD6-\uDDDD])(?:\uD83C[\uDFFB-\uDFFF])|(?:\u26F9|\uD83C[\uDFCB\uDFCC]|\uD83D\uDD75)(?:\uD83C[\uDFFB-\uDFFF])|(?:[\u261D\u270A-\u270D]|\uD83C[\uDF85\uDFC2\uDFC7]|\uD83D[\uDC42\uDC43\uDC46-\uDC50\uDC66\uDC67\uDC6B-\uDC6D\uDC70\uDC72\uDC74-\uDC76\uDC78\uDC7C\uDC83\uDC85\uDCAA\uDD74\uDD7A\uDD90\uDD95\uDD96\uDE4C\uDE4F\uDEC0\uDECC]|\uD83E[\uDD0F\uDD18-\uDD1C\uDD1E\uDD1F\uDD30-\uDD36\uDDB5\uDDB6\uDDBB\uDDD2-\uDDD5])(?:\uD83C[\uDFFB-\uDFFF])|(?:[\u231A\u231B\u23E9-\u23EC\u23F0\u23F3\u25FD\u25FE\u2614\u2615\u2648-\u2653\u267F\u2693\u26A1\u26AA\u26AB\u26BD\u26BE\u26C4\u26C5\u26CE\u26D4\u26EA\u26F2\u26F3\u26F5\u26FA\u26FD\u2705\u270A\u270B\u2728\u274C\u274E\u2753-\u2755\u2757\u2795-\u2797\u27B0\u27BF\u2B1B\u2B1C\u2B50\u2B55]|\uD83C[\uDC04\uDCCF\uDD8E\uDD91-\uDD9A\uDDE6-\uDDFF\uDE01\uDE1A\uDE2F\uDE32-\uDE36\uDE38-\uDE3A\uDE50\uDE51\uDF00-\uDF20\uDF2D-\uDF35\uDF37-\uDF7C\uDF7E-\uDF93\uDFA0-\uDFCA\uDFCF-\uDFD3\uDFE0-\uDFF0\uDFF4\uDFF8-\uDFFF]|\uD83D[\uDC00-\uDC3E\uDC40\uDC42-\uDCFC\uDCFF-\uDD3D\uDD4B-\uDD4E\uDD50-\uDD67\uDD7A\uDD95\uDD96\uDDA4\uDDFB-\uDE4F\uDE80-\uDEC5\uDECC\uDED0-\uDED2\uDED5\uDEEB\uDEEC\uDEF4-\uDEFA\uDFE0-\uDFEB]|\uD83E[\uDD0D-\uDD3A\uDD3C-\uDD45\uDD47-\uDD71\uDD73-\uDD76\uDD7A-\uDDA2\uDDA5-\uDDAA\uDDAE-\uDDCA\uDDCD-\uDDFF\uDE70-\uDE73\uDE78-\uDE7A\uDE80-\uDE82\uDE90-\uDE95])|(?:[#\*0-9\xA9\xAE\u203C\u2049\u2122\u2139\u2194-\u2199\u21A9\u21AA\u231A\u231B\u2328\u23CF\u23E9-\u23F3\u23F8-\u23FA\u24C2\u25AA\u25AB\u25B6\u25C0\u25FB-\u25FE\u2600-\u2604\u260E\u2611\u2614\u2615\u2618\u261D\u2620\u2622\u2623\u2626\u262A\u262E\u262F\u2638-\u263A\u2640\u2642\u2648-\u2653\u265F\u2660\u2663\u2665\u2666\u2668\u267B\u267E\u267F\u2692-\u2697\u2699\u269B\u269C\u26A0\u26A1\u26AA\u26AB\u26B0\u26B1\u26BD\u26BE\u26C4\u26C5\u26C8\u26CE\u26CF\u26D1\u26D3\u26D4\u26E9\u26EA\u26F0-\u26F5\u26F7-\u26FA\u26FD\u2702\u2705\u2708-\u270D\u270F\u2712\u2714\u2716\u271D\u2721\u2728\u2733\u2734\u2744\u2747\u274C\u274E\u2753-\u2755\u2757\u2763\u2764\u2795-\u2797\u27A1\u27B0\u27BF\u2934\u2935\u2B05-\u2B07\u2B1B\u2B1C\u2B50\u2B55\u3030\u303D\u3297\u3299]|\uD83C[\uDC04\uDCCF\uDD70\uDD71\uDD7E\uDD7F\uDD8E\uDD91-\uDD9A\uDDE6-\uDDFF\uDE01\uDE02\uDE1A\uDE2F\uDE32-\uDE3A\uDE50\uDE51\uDF00-\uDF21\uDF24-\uDF93\uDF96\uDF97\uDF99-\uDF9B\uDF9E-\uDFF0\uDFF3-\uDFF5\uDFF7-\uDFFF]|\uD83D[\uDC00-\uDCFD\uDCFF-\uDD3D\uDD49-\uDD4E\uDD50-\uDD67\uDD6F\uDD70\uDD73-\uDD7A\uDD87\uDD8A-\uDD8D\uDD90\uDD95\uDD96\uDDA4\uDDA5\uDDA8\uDDB1\uDDB2\uDDBC\uDDC2-\uDDC4\uDDD1-\uDDD3\uDDDC-\uDDDE\uDDE1\uDDE3\uDDE8\uDDEF\uDDF3\uDDFA-\uDE4F\uDE80-\uDEC5\uDECB-\uDED2\uDED5\uDEE0-\uDEE5\uDEE9\uDEEB\uDEEC\uDEF0\uDEF3-\uDEFA\uDFE0-\uDFEB]|\uD83E[\uDD0D-\uDD3A\uDD3C-\uDD45\uDD47-\uDD71\uDD73-\uDD76\uDD7A-\uDDA2\uDDA5-\uDDAA\uDDAE-\uDDCA\uDDCD-\uDDFF\uDE70-\uDE73\uDE78-\uDE7A\uDE80-\uDE82\uDE90-\uDE95])\uFE0F|(?:[\u261D\u26F9\u270A-\u270D]|\uD83C[\uDF85\uDFC2-\uDFC4\uDFC7\uDFCA-\uDFCC]|\uD83D[\uDC42\uDC43\uDC46-\uDC50\uDC66-\uDC78\uDC7C\uDC81-\uDC83\uDC85-\uDC87\uDC8F\uDC91\uDCAA\uDD74\uDD75\uDD7A\uDD90\uDD95\uDD96\uDE45-\uDE47\uDE4B-\uDE4F\uDEA3\uDEB4-\uDEB6\uDEC0\uDECC]|\uD83E[\uDD0F\uDD18-\uDD1F\uDD26\uDD30-\uDD39\uDD3C-\uDD3E\uDDB5\uDDB6\uDDB8\uDDB9\uDDBB\uDDCD-\uDDCF\uDDD1-\uDDDD])/g;
    };
  }
});

// ../../node_modules/string-width/index.js
var require_string_width = __commonJS({
  "../../node_modules/string-width/index.js"(exports, module) {
    var stripAnsi = require_strip_ansi();
    var isFullwidthCodePoint = require_is_fullwidth_code_point();
    var emojiRegex = require_emoji_regex();
    var stringWidth = (string) => {
      if (typeof string !== "string" || string.length === 0) {
        return 0;
      }
      string = stripAnsi(string);
      if (string.length === 0) {
        return 0;
      }
      string = string.replace(emojiRegex(), "  ");
      let width = 0;
      for (let i = 0; i < string.length; i++) {
        const code = string.codePointAt(i);
        if (code <= 31 || code >= 127 && code <= 159) {
          continue;
        }
        if (code >= 768 && code <= 879) {
          continue;
        }
        if (code > 65535) {
          i++;
        }
        width += isFullwidthCodePoint(code) ? 2 : 1;
      }
      return width;
    };
    module.exports = stringWidth;
    module.exports.default = stringWidth;
  }
});

// ../../node_modules/widest-line/index.js
var require_widest_line = __commonJS({
  "../../node_modules/widest-line/index.js"(exports, module) {
    var stringWidth = require_string_width();
    var widestLine = (input) => {
      let max = 0;
      for (const line of input.split("\n")) {
        max = Math.max(max, stringWidth(line));
      }
      return max;
    };
    module.exports = widestLine;
    module.exports.default = widestLine;
  }
});

// ../../node_modules/cli-boxes/boxes.json
var require_boxes = __commonJS({
  "../../node_modules/cli-boxes/boxes.json"(exports, module) {
    module.exports = {
      single: {
        topLeft: "\u250C",
        topRight: "\u2510",
        bottomRight: "\u2518",
        bottomLeft: "\u2514",
        vertical: "\u2502",
        horizontal: "\u2500"
      },
      double: {
        topLeft: "\u2554",
        topRight: "\u2557",
        bottomRight: "\u255D",
        bottomLeft: "\u255A",
        vertical: "\u2551",
        horizontal: "\u2550"
      },
      round: {
        topLeft: "\u256D",
        topRight: "\u256E",
        bottomRight: "\u256F",
        bottomLeft: "\u2570",
        vertical: "\u2502",
        horizontal: "\u2500"
      },
      bold: {
        topLeft: "\u250F",
        topRight: "\u2513",
        bottomRight: "\u251B",
        bottomLeft: "\u2517",
        vertical: "\u2503",
        horizontal: "\u2501"
      },
      singleDouble: {
        topLeft: "\u2553",
        topRight: "\u2556",
        bottomRight: "\u255C",
        bottomLeft: "\u2559",
        vertical: "\u2551",
        horizontal: "\u2500"
      },
      doubleSingle: {
        topLeft: "\u2552",
        topRight: "\u2555",
        bottomRight: "\u255B",
        bottomLeft: "\u2558",
        vertical: "\u2502",
        horizontal: "\u2550"
      },
      classic: {
        topLeft: "+",
        topRight: "+",
        bottomRight: "+",
        bottomLeft: "+",
        vertical: "|",
        horizontal: "-"
      }
    };
  }
});

// ../../node_modules/cli-boxes/index.js
var require_cli_boxes = __commonJS({
  "../../node_modules/cli-boxes/index.js"(exports, module) {
    var cliBoxes = require_boxes();
    module.exports = cliBoxes;
    module.exports.default = cliBoxes;
  }
});

// ../../node_modules/boxen/node_modules/camelcase/index.js
var require_camelcase = __commonJS({
  "../../node_modules/boxen/node_modules/camelcase/index.js"(exports, module) {
    var UPPERCASE = /[\p{Lu}]/u;
    var LOWERCASE = /[\p{Ll}]/u;
    var LEADING_CAPITAL = /^[\p{Lu}](?![\p{Lu}])/gu;
    var IDENTIFIER = /([\p{Alpha}\p{N}_]|$)/u;
    var SEPARATORS = /[_.\- ]+/;
    var LEADING_SEPARATORS = new RegExp("^" + SEPARATORS.source);
    var SEPARATORS_AND_IDENTIFIER = new RegExp(SEPARATORS.source + IDENTIFIER.source, "gu");
    var NUMBERS_AND_IDENTIFIER = new RegExp("\\d+" + IDENTIFIER.source, "gu");
    var preserveCamelCase = (string, toLowerCase, toUpperCase) => {
      let isLastCharLower = false;
      let isLastCharUpper = false;
      let isLastLastCharUpper = false;
      for (let i = 0; i < string.length; i++) {
        const character = string[i];
        if (isLastCharLower && UPPERCASE.test(character)) {
          string = string.slice(0, i) + "-" + string.slice(i);
          isLastCharLower = false;
          isLastLastCharUpper = isLastCharUpper;
          isLastCharUpper = true;
          i++;
        } else if (isLastCharUpper && isLastLastCharUpper && LOWERCASE.test(character)) {
          string = string.slice(0, i - 1) + "-" + string.slice(i - 1);
          isLastLastCharUpper = isLastCharUpper;
          isLastCharUpper = false;
          isLastCharLower = true;
        } else {
          isLastCharLower = toLowerCase(character) === character && toUpperCase(character) !== character;
          isLastLastCharUpper = isLastCharUpper;
          isLastCharUpper = toUpperCase(character) === character && toLowerCase(character) !== character;
        }
      }
      return string;
    };
    var preserveConsecutiveUppercase = (input, toLowerCase) => {
      LEADING_CAPITAL.lastIndex = 0;
      return input.replace(LEADING_CAPITAL, (m1) => toLowerCase(m1));
    };
    var postProcess = (input, toUpperCase) => {
      SEPARATORS_AND_IDENTIFIER.lastIndex = 0;
      NUMBERS_AND_IDENTIFIER.lastIndex = 0;
      return input.replace(SEPARATORS_AND_IDENTIFIER, (_, identifier2) => toUpperCase(identifier2)).replace(NUMBERS_AND_IDENTIFIER, (m) => toUpperCase(m));
    };
    var camelCase = (input, options) => {
      if (!(typeof input === "string" || Array.isArray(input))) {
        throw new TypeError("Expected the input to be `string | string[]`");
      }
      options = {
        pascalCase: false,
        preserveConsecutiveUppercase: false,
        ...options
      };
      if (Array.isArray(input)) {
        input = input.map((x) => x.trim()).filter((x) => x.length).join("-");
      } else {
        input = input.trim();
      }
      if (input.length === 0) {
        return "";
      }
      const toLowerCase = options.locale === false ? (string) => string.toLowerCase() : (string) => string.toLocaleLowerCase(options.locale);
      const toUpperCase = options.locale === false ? (string) => string.toUpperCase() : (string) => string.toLocaleUpperCase(options.locale);
      if (input.length === 1) {
        return options.pascalCase ? toUpperCase(input) : toLowerCase(input);
      }
      const hasUpperCase = input !== toLowerCase(input);
      if (hasUpperCase) {
        input = preserveCamelCase(input, toLowerCase, toUpperCase);
      }
      input = input.replace(LEADING_SEPARATORS, "");
      if (options.preserveConsecutiveUppercase) {
        input = preserveConsecutiveUppercase(input, toLowerCase);
      } else {
        input = toLowerCase(input);
      }
      if (options.pascalCase) {
        input = toUpperCase(input.charAt(0)) + input.slice(1);
      }
      return postProcess(input, toUpperCase);
    };
    module.exports = camelCase;
    module.exports.default = camelCase;
  }
});

// ../../node_modules/ansi-align/index.js
var require_ansi_align = __commonJS({
  "../../node_modules/ansi-align/index.js"(exports, module) {
    var stringWidth = require_string_width();
    function ansiAlign(text, opts) {
      if (!text)
        return text;
      opts = opts || {};
      const align = opts.align || "center";
      if (align === "left")
        return text;
      const split = opts.split || "\n";
      const pad = opts.pad || " ";
      const widthDiffFn = align !== "right" ? halfDiff : fullDiff;
      let returnString = false;
      if (!Array.isArray(text)) {
        returnString = true;
        text = String(text).split(split);
      }
      let width;
      let maxWidth = 0;
      text = text.map(function(str) {
        str = String(str);
        width = stringWidth(str);
        maxWidth = Math.max(width, maxWidth);
        return {
          str,
          width
        };
      }).map(function(obj) {
        return new Array(widthDiffFn(maxWidth, obj.width) + 1).join(pad) + obj.str;
      });
      return returnString ? text.join(split) : text;
    }
    ansiAlign.left = function left(text) {
      return ansiAlign(text, { align: "left" });
    };
    ansiAlign.center = function center(text) {
      return ansiAlign(text, { align: "center" });
    };
    ansiAlign.right = function right(text) {
      return ansiAlign(text, { align: "right" });
    };
    module.exports = ansiAlign;
    function halfDiff(maxWidth, curWidth) {
      return Math.floor((maxWidth - curWidth) / 2);
    }
    function fullDiff(maxWidth, curWidth) {
      return maxWidth - curWidth;
    }
  }
});

// ../../node_modules/color-convert/node_modules/color-name/index.js
var require_color_name = __commonJS({
  "../../node_modules/color-convert/node_modules/color-name/index.js"(exports, module) {
    module.exports = {
      "aliceblue": [240, 248, 255],
      "antiquewhite": [250, 235, 215],
      "aqua": [0, 255, 255],
      "aquamarine": [127, 255, 212],
      "azure": [240, 255, 255],
      "beige": [245, 245, 220],
      "bisque": [255, 228, 196],
      "black": [0, 0, 0],
      "blanchedalmond": [255, 235, 205],
      "blue": [0, 0, 255],
      "blueviolet": [138, 43, 226],
      "brown": [165, 42, 42],
      "burlywood": [222, 184, 135],
      "cadetblue": [95, 158, 160],
      "chartreuse": [127, 255, 0],
      "chocolate": [210, 105, 30],
      "coral": [255, 127, 80],
      "cornflowerblue": [100, 149, 237],
      "cornsilk": [255, 248, 220],
      "crimson": [220, 20, 60],
      "cyan": [0, 255, 255],
      "darkblue": [0, 0, 139],
      "darkcyan": [0, 139, 139],
      "darkgoldenrod": [184, 134, 11],
      "darkgray": [169, 169, 169],
      "darkgreen": [0, 100, 0],
      "darkgrey": [169, 169, 169],
      "darkkhaki": [189, 183, 107],
      "darkmagenta": [139, 0, 139],
      "darkolivegreen": [85, 107, 47],
      "darkorange": [255, 140, 0],
      "darkorchid": [153, 50, 204],
      "darkred": [139, 0, 0],
      "darksalmon": [233, 150, 122],
      "darkseagreen": [143, 188, 143],
      "darkslateblue": [72, 61, 139],
      "darkslategray": [47, 79, 79],
      "darkslategrey": [47, 79, 79],
      "darkturquoise": [0, 206, 209],
      "darkviolet": [148, 0, 211],
      "deeppink": [255, 20, 147],
      "deepskyblue": [0, 191, 255],
      "dimgray": [105, 105, 105],
      "dimgrey": [105, 105, 105],
      "dodgerblue": [30, 144, 255],
      "firebrick": [178, 34, 34],
      "floralwhite": [255, 250, 240],
      "forestgreen": [34, 139, 34],
      "fuchsia": [255, 0, 255],
      "gainsboro": [220, 220, 220],
      "ghostwhite": [248, 248, 255],
      "gold": [255, 215, 0],
      "goldenrod": [218, 165, 32],
      "gray": [128, 128, 128],
      "green": [0, 128, 0],
      "greenyellow": [173, 255, 47],
      "grey": [128, 128, 128],
      "honeydew": [240, 255, 240],
      "hotpink": [255, 105, 180],
      "indianred": [205, 92, 92],
      "indigo": [75, 0, 130],
      "ivory": [255, 255, 240],
      "khaki": [240, 230, 140],
      "lavender": [230, 230, 250],
      "lavenderblush": [255, 240, 245],
      "lawngreen": [124, 252, 0],
      "lemonchiffon": [255, 250, 205],
      "lightblue": [173, 216, 230],
      "lightcoral": [240, 128, 128],
      "lightcyan": [224, 255, 255],
      "lightgoldenrodyellow": [250, 250, 210],
      "lightgray": [211, 211, 211],
      "lightgreen": [144, 238, 144],
      "lightgrey": [211, 211, 211],
      "lightpink": [255, 182, 193],
      "lightsalmon": [255, 160, 122],
      "lightseagreen": [32, 178, 170],
      "lightskyblue": [135, 206, 250],
      "lightslategray": [119, 136, 153],
      "lightslategrey": [119, 136, 153],
      "lightsteelblue": [176, 196, 222],
      "lightyellow": [255, 255, 224],
      "lime": [0, 255, 0],
      "limegreen": [50, 205, 50],
      "linen": [250, 240, 230],
      "magenta": [255, 0, 255],
      "maroon": [128, 0, 0],
      "mediumaquamarine": [102, 205, 170],
      "mediumblue": [0, 0, 205],
      "mediumorchid": [186, 85, 211],
      "mediumpurple": [147, 112, 219],
      "mediumseagreen": [60, 179, 113],
      "mediumslateblue": [123, 104, 238],
      "mediumspringgreen": [0, 250, 154],
      "mediumturquoise": [72, 209, 204],
      "mediumvioletred": [199, 21, 133],
      "midnightblue": [25, 25, 112],
      "mintcream": [245, 255, 250],
      "mistyrose": [255, 228, 225],
      "moccasin": [255, 228, 181],
      "navajowhite": [255, 222, 173],
      "navy": [0, 0, 128],
      "oldlace": [253, 245, 230],
      "olive": [128, 128, 0],
      "olivedrab": [107, 142, 35],
      "orange": [255, 165, 0],
      "orangered": [255, 69, 0],
      "orchid": [218, 112, 214],
      "palegoldenrod": [238, 232, 170],
      "palegreen": [152, 251, 152],
      "paleturquoise": [175, 238, 238],
      "palevioletred": [219, 112, 147],
      "papayawhip": [255, 239, 213],
      "peachpuff": [255, 218, 185],
      "peru": [205, 133, 63],
      "pink": [255, 192, 203],
      "plum": [221, 160, 221],
      "powderblue": [176, 224, 230],
      "purple": [128, 0, 128],
      "rebeccapurple": [102, 51, 153],
      "red": [255, 0, 0],
      "rosybrown": [188, 143, 143],
      "royalblue": [65, 105, 225],
      "saddlebrown": [139, 69, 19],
      "salmon": [250, 128, 114],
      "sandybrown": [244, 164, 96],
      "seagreen": [46, 139, 87],
      "seashell": [255, 245, 238],
      "sienna": [160, 82, 45],
      "silver": [192, 192, 192],
      "skyblue": [135, 206, 235],
      "slateblue": [106, 90, 205],
      "slategray": [112, 128, 144],
      "slategrey": [112, 128, 144],
      "snow": [255, 250, 250],
      "springgreen": [0, 255, 127],
      "steelblue": [70, 130, 180],
      "tan": [210, 180, 140],
      "teal": [0, 128, 128],
      "thistle": [216, 191, 216],
      "tomato": [255, 99, 71],
      "turquoise": [64, 224, 208],
      "violet": [238, 130, 238],
      "wheat": [245, 222, 179],
      "white": [255, 255, 255],
      "whitesmoke": [245, 245, 245],
      "yellow": [255, 255, 0],
      "yellowgreen": [154, 205, 50]
    };
  }
});

// ../../node_modules/color-convert/conversions.js
var require_conversions = __commonJS({
  "../../node_modules/color-convert/conversions.js"(exports, module) {
    var cssKeywords = require_color_name();
    var reverseKeywords = {};
    for (const key of Object.keys(cssKeywords)) {
      reverseKeywords[cssKeywords[key]] = key;
    }
    var convert = {
      rgb: { channels: 3, labels: "rgb" },
      hsl: { channels: 3, labels: "hsl" },
      hsv: { channels: 3, labels: "hsv" },
      hwb: { channels: 3, labels: "hwb" },
      cmyk: { channels: 4, labels: "cmyk" },
      xyz: { channels: 3, labels: "xyz" },
      lab: { channels: 3, labels: "lab" },
      lch: { channels: 3, labels: "lch" },
      hex: { channels: 1, labels: ["hex"] },
      keyword: { channels: 1, labels: ["keyword"] },
      ansi16: { channels: 1, labels: ["ansi16"] },
      ansi256: { channels: 1, labels: ["ansi256"] },
      hcg: { channels: 3, labels: ["h", "c", "g"] },
      apple: { channels: 3, labels: ["r16", "g16", "b16"] },
      gray: { channels: 1, labels: ["gray"] }
    };
    module.exports = convert;
    for (const model of Object.keys(convert)) {
      if (!("channels" in convert[model])) {
        throw new Error("missing channels property: " + model);
      }
      if (!("labels" in convert[model])) {
        throw new Error("missing channel labels property: " + model);
      }
      if (convert[model].labels.length !== convert[model].channels) {
        throw new Error("channel and label counts mismatch: " + model);
      }
      const { channels, labels } = convert[model];
      delete convert[model].channels;
      delete convert[model].labels;
      Object.defineProperty(convert[model], "channels", { value: channels });
      Object.defineProperty(convert[model], "labels", { value: labels });
    }
    convert.rgb.hsl = function(rgb) {
      const r = rgb[0] / 255;
      const g = rgb[1] / 255;
      const b = rgb[2] / 255;
      const min = Math.min(r, g, b);
      const max = Math.max(r, g, b);
      const delta = max - min;
      let h;
      let s;
      if (max === min) {
        h = 0;
      } else if (r === max) {
        h = (g - b) / delta;
      } else if (g === max) {
        h = 2 + (b - r) / delta;
      } else if (b === max) {
        h = 4 + (r - g) / delta;
      }
      h = Math.min(h * 60, 360);
      if (h < 0) {
        h += 360;
      }
      const l = (min + max) / 2;
      if (max === min) {
        s = 0;
      } else if (l <= 0.5) {
        s = delta / (max + min);
      } else {
        s = delta / (2 - max - min);
      }
      return [h, s * 100, l * 100];
    };
    convert.rgb.hsv = function(rgb) {
      let rdif;
      let gdif;
      let bdif;
      let h;
      let s;
      const r = rgb[0] / 255;
      const g = rgb[1] / 255;
      const b = rgb[2] / 255;
      const v = Math.max(r, g, b);
      const diff = v - Math.min(r, g, b);
      const diffc = function(c) {
        return (v - c) / 6 / diff + 1 / 2;
      };
      if (diff === 0) {
        h = 0;
        s = 0;
      } else {
        s = diff / v;
        rdif = diffc(r);
        gdif = diffc(g);
        bdif = diffc(b);
        if (r === v) {
          h = bdif - gdif;
        } else if (g === v) {
          h = 1 / 3 + rdif - bdif;
        } else if (b === v) {
          h = 2 / 3 + gdif - rdif;
        }
        if (h < 0) {
          h += 1;
        } else if (h > 1) {
          h -= 1;
        }
      }
      return [
        h * 360,
        s * 100,
        v * 100
      ];
    };
    convert.rgb.hwb = function(rgb) {
      const r = rgb[0];
      const g = rgb[1];
      let b = rgb[2];
      const h = convert.rgb.hsl(rgb)[0];
      const w = 1 / 255 * Math.min(r, Math.min(g, b));
      b = 1 - 1 / 255 * Math.max(r, Math.max(g, b));
      return [h, w * 100, b * 100];
    };
    convert.rgb.cmyk = function(rgb) {
      const r = rgb[0] / 255;
      const g = rgb[1] / 255;
      const b = rgb[2] / 255;
      const k = Math.min(1 - r, 1 - g, 1 - b);
      const c = (1 - r - k) / (1 - k) || 0;
      const m = (1 - g - k) / (1 - k) || 0;
      const y = (1 - b - k) / (1 - k) || 0;
      return [c * 100, m * 100, y * 100, k * 100];
    };
    function comparativeDistance(x, y) {
      return (x[0] - y[0]) ** 2 + (x[1] - y[1]) ** 2 + (x[2] - y[2]) ** 2;
    }
    convert.rgb.keyword = function(rgb) {
      const reversed = reverseKeywords[rgb];
      if (reversed) {
        return reversed;
      }
      let currentClosestDistance = Infinity;
      let currentClosestKeyword;
      for (const keyword of Object.keys(cssKeywords)) {
        const value = cssKeywords[keyword];
        const distance = comparativeDistance(rgb, value);
        if (distance < currentClosestDistance) {
          currentClosestDistance = distance;
          currentClosestKeyword = keyword;
        }
      }
      return currentClosestKeyword;
    };
    convert.keyword.rgb = function(keyword) {
      return cssKeywords[keyword];
    };
    convert.rgb.xyz = function(rgb) {
      let r = rgb[0] / 255;
      let g = rgb[1] / 255;
      let b = rgb[2] / 255;
      r = r > 0.04045 ? ((r + 0.055) / 1.055) ** 2.4 : r / 12.92;
      g = g > 0.04045 ? ((g + 0.055) / 1.055) ** 2.4 : g / 12.92;
      b = b > 0.04045 ? ((b + 0.055) / 1.055) ** 2.4 : b / 12.92;
      const x = r * 0.4124 + g * 0.3576 + b * 0.1805;
      const y = r * 0.2126 + g * 0.7152 + b * 0.0722;
      const z = r * 0.0193 + g * 0.1192 + b * 0.9505;
      return [x * 100, y * 100, z * 100];
    };
    convert.rgb.lab = function(rgb) {
      const xyz = convert.rgb.xyz(rgb);
      let x = xyz[0];
      let y = xyz[1];
      let z = xyz[2];
      x /= 95.047;
      y /= 100;
      z /= 108.883;
      x = x > 8856e-6 ? x ** (1 / 3) : 7.787 * x + 16 / 116;
      y = y > 8856e-6 ? y ** (1 / 3) : 7.787 * y + 16 / 116;
      z = z > 8856e-6 ? z ** (1 / 3) : 7.787 * z + 16 / 116;
      const l = 116 * y - 16;
      const a = 500 * (x - y);
      const b = 200 * (y - z);
      return [l, a, b];
    };
    convert.hsl.rgb = function(hsl) {
      const h = hsl[0] / 360;
      const s = hsl[1] / 100;
      const l = hsl[2] / 100;
      let t2;
      let t3;
      let val;
      if (s === 0) {
        val = l * 255;
        return [val, val, val];
      }
      if (l < 0.5) {
        t2 = l * (1 + s);
      } else {
        t2 = l + s - l * s;
      }
      const t1 = 2 * l - t2;
      const rgb = [0, 0, 0];
      for (let i = 0; i < 3; i++) {
        t3 = h + 1 / 3 * -(i - 1);
        if (t3 < 0) {
          t3++;
        }
        if (t3 > 1) {
          t3--;
        }
        if (6 * t3 < 1) {
          val = t1 + (t2 - t1) * 6 * t3;
        } else if (2 * t3 < 1) {
          val = t2;
        } else if (3 * t3 < 2) {
          val = t1 + (t2 - t1) * (2 / 3 - t3) * 6;
        } else {
          val = t1;
        }
        rgb[i] = val * 255;
      }
      return rgb;
    };
    convert.hsl.hsv = function(hsl) {
      const h = hsl[0];
      let s = hsl[1] / 100;
      let l = hsl[2] / 100;
      let smin = s;
      const lmin = Math.max(l, 0.01);
      l *= 2;
      s *= l <= 1 ? l : 2 - l;
      smin *= lmin <= 1 ? lmin : 2 - lmin;
      const v = (l + s) / 2;
      const sv = l === 0 ? 2 * smin / (lmin + smin) : 2 * s / (l + s);
      return [h, sv * 100, v * 100];
    };
    convert.hsv.rgb = function(hsv) {
      const h = hsv[0] / 60;
      const s = hsv[1] / 100;
      let v = hsv[2] / 100;
      const hi = Math.floor(h) % 6;
      const f = h - Math.floor(h);
      const p = 255 * v * (1 - s);
      const q = 255 * v * (1 - s * f);
      const t2 = 255 * v * (1 - s * (1 - f));
      v *= 255;
      switch (hi) {
        case 0:
          return [v, t2, p];
        case 1:
          return [q, v, p];
        case 2:
          return [p, v, t2];
        case 3:
          return [p, q, v];
        case 4:
          return [t2, p, v];
        case 5:
          return [v, p, q];
      }
    };
    convert.hsv.hsl = function(hsv) {
      const h = hsv[0];
      const s = hsv[1] / 100;
      const v = hsv[2] / 100;
      const vmin = Math.max(v, 0.01);
      let sl;
      let l;
      l = (2 - s) * v;
      const lmin = (2 - s) * vmin;
      sl = s * vmin;
      sl /= lmin <= 1 ? lmin : 2 - lmin;
      sl = sl || 0;
      l /= 2;
      return [h, sl * 100, l * 100];
    };
    convert.hwb.rgb = function(hwb) {
      const h = hwb[0] / 360;
      let wh = hwb[1] / 100;
      let bl = hwb[2] / 100;
      const ratio = wh + bl;
      let f;
      if (ratio > 1) {
        wh /= ratio;
        bl /= ratio;
      }
      const i = Math.floor(6 * h);
      const v = 1 - bl;
      f = 6 * h - i;
      if ((i & 1) !== 0) {
        f = 1 - f;
      }
      const n = wh + f * (v - wh);
      let r;
      let g;
      let b;
      switch (i) {
        default:
        case 6:
        case 0:
          r = v;
          g = n;
          b = wh;
          break;
        case 1:
          r = n;
          g = v;
          b = wh;
          break;
        case 2:
          r = wh;
          g = v;
          b = n;
          break;
        case 3:
          r = wh;
          g = n;
          b = v;
          break;
        case 4:
          r = n;
          g = wh;
          b = v;
          break;
        case 5:
          r = v;
          g = wh;
          b = n;
          break;
      }
      return [r * 255, g * 255, b * 255];
    };
    convert.cmyk.rgb = function(cmyk) {
      const c = cmyk[0] / 100;
      const m = cmyk[1] / 100;
      const y = cmyk[2] / 100;
      const k = cmyk[3] / 100;
      const r = 1 - Math.min(1, c * (1 - k) + k);
      const g = 1 - Math.min(1, m * (1 - k) + k);
      const b = 1 - Math.min(1, y * (1 - k) + k);
      return [r * 255, g * 255, b * 255];
    };
    convert.xyz.rgb = function(xyz) {
      const x = xyz[0] / 100;
      const y = xyz[1] / 100;
      const z = xyz[2] / 100;
      let r;
      let g;
      let b;
      r = x * 3.2406 + y * -1.5372 + z * -0.4986;
      g = x * -0.9689 + y * 1.8758 + z * 0.0415;
      b = x * 0.0557 + y * -0.204 + z * 1.057;
      r = r > 31308e-7 ? 1.055 * r ** (1 / 2.4) - 0.055 : r * 12.92;
      g = g > 31308e-7 ? 1.055 * g ** (1 / 2.4) - 0.055 : g * 12.92;
      b = b > 31308e-7 ? 1.055 * b ** (1 / 2.4) - 0.055 : b * 12.92;
      r = Math.min(Math.max(0, r), 1);
      g = Math.min(Math.max(0, g), 1);
      b = Math.min(Math.max(0, b), 1);
      return [r * 255, g * 255, b * 255];
    };
    convert.xyz.lab = function(xyz) {
      let x = xyz[0];
      let y = xyz[1];
      let z = xyz[2];
      x /= 95.047;
      y /= 100;
      z /= 108.883;
      x = x > 8856e-6 ? x ** (1 / 3) : 7.787 * x + 16 / 116;
      y = y > 8856e-6 ? y ** (1 / 3) : 7.787 * y + 16 / 116;
      z = z > 8856e-6 ? z ** (1 / 3) : 7.787 * z + 16 / 116;
      const l = 116 * y - 16;
      const a = 500 * (x - y);
      const b = 200 * (y - z);
      return [l, a, b];
    };
    convert.lab.xyz = function(lab) {
      const l = lab[0];
      const a = lab[1];
      const b = lab[2];
      let x;
      let y;
      let z;
      y = (l + 16) / 116;
      x = a / 500 + y;
      z = y - b / 200;
      const y2 = y ** 3;
      const x2 = x ** 3;
      const z2 = z ** 3;
      y = y2 > 8856e-6 ? y2 : (y - 16 / 116) / 7.787;
      x = x2 > 8856e-6 ? x2 : (x - 16 / 116) / 7.787;
      z = z2 > 8856e-6 ? z2 : (z - 16 / 116) / 7.787;
      x *= 95.047;
      y *= 100;
      z *= 108.883;
      return [x, y, z];
    };
    convert.lab.lch = function(lab) {
      const l = lab[0];
      const a = lab[1];
      const b = lab[2];
      let h;
      const hr = Math.atan2(b, a);
      h = hr * 360 / 2 / Math.PI;
      if (h < 0) {
        h += 360;
      }
      const c = Math.sqrt(a * a + b * b);
      return [l, c, h];
    };
    convert.lch.lab = function(lch) {
      const l = lch[0];
      const c = lch[1];
      const h = lch[2];
      const hr = h / 360 * 2 * Math.PI;
      const a = c * Math.cos(hr);
      const b = c * Math.sin(hr);
      return [l, a, b];
    };
    convert.rgb.ansi16 = function(args, saturation = null) {
      const [r, g, b] = args;
      let value = saturation === null ? convert.rgb.hsv(args)[2] : saturation;
      value = Math.round(value / 50);
      if (value === 0) {
        return 30;
      }
      let ansi = 30 + (Math.round(b / 255) << 2 | Math.round(g / 255) << 1 | Math.round(r / 255));
      if (value === 2) {
        ansi += 60;
      }
      return ansi;
    };
    convert.hsv.ansi16 = function(args) {
      return convert.rgb.ansi16(convert.hsv.rgb(args), args[2]);
    };
    convert.rgb.ansi256 = function(args) {
      const r = args[0];
      const g = args[1];
      const b = args[2];
      if (r === g && g === b) {
        if (r < 8) {
          return 16;
        }
        if (r > 248) {
          return 231;
        }
        return Math.round((r - 8) / 247 * 24) + 232;
      }
      const ansi = 16 + 36 * Math.round(r / 255 * 5) + 6 * Math.round(g / 255 * 5) + Math.round(b / 255 * 5);
      return ansi;
    };
    convert.ansi16.rgb = function(args) {
      let color = args % 10;
      if (color === 0 || color === 7) {
        if (args > 50) {
          color += 3.5;
        }
        color = color / 10.5 * 255;
        return [color, color, color];
      }
      const mult = (~~(args > 50) + 1) * 0.5;
      const r = (color & 1) * mult * 255;
      const g = (color >> 1 & 1) * mult * 255;
      const b = (color >> 2 & 1) * mult * 255;
      return [r, g, b];
    };
    convert.ansi256.rgb = function(args) {
      if (args >= 232) {
        const c = (args - 232) * 10 + 8;
        return [c, c, c];
      }
      args -= 16;
      let rem;
      const r = Math.floor(args / 36) / 5 * 255;
      const g = Math.floor((rem = args % 36) / 6) / 5 * 255;
      const b = rem % 6 / 5 * 255;
      return [r, g, b];
    };
    convert.rgb.hex = function(args) {
      const integer = ((Math.round(args[0]) & 255) << 16) + ((Math.round(args[1]) & 255) << 8) + (Math.round(args[2]) & 255);
      const string = integer.toString(16).toUpperCase();
      return "000000".substring(string.length) + string;
    };
    convert.hex.rgb = function(args) {
      const match = args.toString(16).match(/[a-f0-9]{6}|[a-f0-9]{3}/i);
      if (!match) {
        return [0, 0, 0];
      }
      let colorString = match[0];
      if (match[0].length === 3) {
        colorString = colorString.split("").map((char) => {
          return char + char;
        }).join("");
      }
      const integer = parseInt(colorString, 16);
      const r = integer >> 16 & 255;
      const g = integer >> 8 & 255;
      const b = integer & 255;
      return [r, g, b];
    };
    convert.rgb.hcg = function(rgb) {
      const r = rgb[0] / 255;
      const g = rgb[1] / 255;
      const b = rgb[2] / 255;
      const max = Math.max(Math.max(r, g), b);
      const min = Math.min(Math.min(r, g), b);
      const chroma = max - min;
      let grayscale;
      let hue;
      if (chroma < 1) {
        grayscale = min / (1 - chroma);
      } else {
        grayscale = 0;
      }
      if (chroma <= 0) {
        hue = 0;
      } else if (max === r) {
        hue = (g - b) / chroma % 6;
      } else if (max === g) {
        hue = 2 + (b - r) / chroma;
      } else {
        hue = 4 + (r - g) / chroma;
      }
      hue /= 6;
      hue %= 1;
      return [hue * 360, chroma * 100, grayscale * 100];
    };
    convert.hsl.hcg = function(hsl) {
      const s = hsl[1] / 100;
      const l = hsl[2] / 100;
      const c = l < 0.5 ? 2 * s * l : 2 * s * (1 - l);
      let f = 0;
      if (c < 1) {
        f = (l - 0.5 * c) / (1 - c);
      }
      return [hsl[0], c * 100, f * 100];
    };
    convert.hsv.hcg = function(hsv) {
      const s = hsv[1] / 100;
      const v = hsv[2] / 100;
      const c = s * v;
      let f = 0;
      if (c < 1) {
        f = (v - c) / (1 - c);
      }
      return [hsv[0], c * 100, f * 100];
    };
    convert.hcg.rgb = function(hcg) {
      const h = hcg[0] / 360;
      const c = hcg[1] / 100;
      const g = hcg[2] / 100;
      if (c === 0) {
        return [g * 255, g * 255, g * 255];
      }
      const pure = [0, 0, 0];
      const hi = h % 1 * 6;
      const v = hi % 1;
      const w = 1 - v;
      let mg = 0;
      switch (Math.floor(hi)) {
        case 0:
          pure[0] = 1;
          pure[1] = v;
          pure[2] = 0;
          break;
        case 1:
          pure[0] = w;
          pure[1] = 1;
          pure[2] = 0;
          break;
        case 2:
          pure[0] = 0;
          pure[1] = 1;
          pure[2] = v;
          break;
        case 3:
          pure[0] = 0;
          pure[1] = w;
          pure[2] = 1;
          break;
        case 4:
          pure[0] = v;
          pure[1] = 0;
          pure[2] = 1;
          break;
        default:
          pure[0] = 1;
          pure[1] = 0;
          pure[2] = w;
      }
      mg = (1 - c) * g;
      return [
        (c * pure[0] + mg) * 255,
        (c * pure[1] + mg) * 255,
        (c * pure[2] + mg) * 255
      ];
    };
    convert.hcg.hsv = function(hcg) {
      const c = hcg[1] / 100;
      const g = hcg[2] / 100;
      const v = c + g * (1 - c);
      let f = 0;
      if (v > 0) {
        f = c / v;
      }
      return [hcg[0], f * 100, v * 100];
    };
    convert.hcg.hsl = function(hcg) {
      const c = hcg[1] / 100;
      const g = hcg[2] / 100;
      const l = g * (1 - c) + 0.5 * c;
      let s = 0;
      if (l > 0 && l < 0.5) {
        s = c / (2 * l);
      } else if (l >= 0.5 && l < 1) {
        s = c / (2 * (1 - l));
      }
      return [hcg[0], s * 100, l * 100];
    };
    convert.hcg.hwb = function(hcg) {
      const c = hcg[1] / 100;
      const g = hcg[2] / 100;
      const v = c + g * (1 - c);
      return [hcg[0], (v - c) * 100, (1 - v) * 100];
    };
    convert.hwb.hcg = function(hwb) {
      const w = hwb[1] / 100;
      const b = hwb[2] / 100;
      const v = 1 - b;
      const c = v - w;
      let g = 0;
      if (c < 1) {
        g = (v - c) / (1 - c);
      }
      return [hwb[0], c * 100, g * 100];
    };
    convert.apple.rgb = function(apple) {
      return [apple[0] / 65535 * 255, apple[1] / 65535 * 255, apple[2] / 65535 * 255];
    };
    convert.rgb.apple = function(rgb) {
      return [rgb[0] / 255 * 65535, rgb[1] / 255 * 65535, rgb[2] / 255 * 65535];
    };
    convert.gray.rgb = function(args) {
      return [args[0] / 100 * 255, args[0] / 100 * 255, args[0] / 100 * 255];
    };
    convert.gray.hsl = function(args) {
      return [0, 0, args[0]];
    };
    convert.gray.hsv = convert.gray.hsl;
    convert.gray.hwb = function(gray) {
      return [0, 100, gray[0]];
    };
    convert.gray.cmyk = function(gray) {
      return [0, 0, 0, gray[0]];
    };
    convert.gray.lab = function(gray) {
      return [gray[0], 0, 0];
    };
    convert.gray.hex = function(gray) {
      const val = Math.round(gray[0] / 100 * 255) & 255;
      const integer = (val << 16) + (val << 8) + val;
      const string = integer.toString(16).toUpperCase();
      return "000000".substring(string.length) + string;
    };
    convert.rgb.gray = function(rgb) {
      const val = (rgb[0] + rgb[1] + rgb[2]) / 3;
      return [val / 255 * 100];
    };
  }
});

// ../../node_modules/color-convert/route.js
var require_route = __commonJS({
  "../../node_modules/color-convert/route.js"(exports, module) {
    var conversions = require_conversions();
    function buildGraph() {
      const graph = {};
      const models = Object.keys(conversions);
      for (let len = models.length, i = 0; i < len; i++) {
        graph[models[i]] = {
          // http://jsperf.com/1-vs-infinity
          // micro-opt, but this is simple.
          distance: -1,
          parent: null
        };
      }
      return graph;
    }
    function deriveBFS(fromModel) {
      const graph = buildGraph();
      const queue = [fromModel];
      graph[fromModel].distance = 0;
      while (queue.length) {
        const current = queue.pop();
        const adjacents = Object.keys(conversions[current]);
        for (let len = adjacents.length, i = 0; i < len; i++) {
          const adjacent = adjacents[i];
          const node = graph[adjacent];
          if (node.distance === -1) {
            node.distance = graph[current].distance + 1;
            node.parent = current;
            queue.unshift(adjacent);
          }
        }
      }
      return graph;
    }
    function link2(from, to) {
      return function(args) {
        return to(from(args));
      };
    }
    function wrapConversion(toModel, graph) {
      const path11 = [graph[toModel].parent, toModel];
      let fn = conversions[graph[toModel].parent][toModel];
      let cur = graph[toModel].parent;
      while (graph[cur].parent) {
        path11.unshift(graph[cur].parent);
        fn = link2(conversions[graph[cur].parent][cur], fn);
        cur = graph[cur].parent;
      }
      fn.conversion = path11;
      return fn;
    }
    module.exports = function(fromModel) {
      const graph = deriveBFS(fromModel);
      const conversion = {};
      const models = Object.keys(graph);
      for (let len = models.length, i = 0; i < len; i++) {
        const toModel = models[i];
        const node = graph[toModel];
        if (node.parent === null) {
          continue;
        }
        conversion[toModel] = wrapConversion(toModel, graph);
      }
      return conversion;
    };
  }
});

// ../../node_modules/color-convert/index.js
var require_color_convert = __commonJS({
  "../../node_modules/color-convert/index.js"(exports, module) {
    var conversions = require_conversions();
    var route = require_route();
    var convert = {};
    var models = Object.keys(conversions);
    function wrapRaw(fn) {
      const wrappedFn = function(...args) {
        const arg0 = args[0];
        if (arg0 === void 0 || arg0 === null) {
          return arg0;
        }
        if (arg0.length > 1) {
          args = arg0;
        }
        return fn(args);
      };
      if ("conversion" in fn) {
        wrappedFn.conversion = fn.conversion;
      }
      return wrappedFn;
    }
    function wrapRounded(fn) {
      const wrappedFn = function(...args) {
        const arg0 = args[0];
        if (arg0 === void 0 || arg0 === null) {
          return arg0;
        }
        if (arg0.length > 1) {
          args = arg0;
        }
        const result = fn(args);
        if (typeof result === "object") {
          for (let len = result.length, i = 0; i < len; i++) {
            result[i] = Math.round(result[i]);
          }
        }
        return result;
      };
      if ("conversion" in fn) {
        wrappedFn.conversion = fn.conversion;
      }
      return wrappedFn;
    }
    models.forEach((fromModel) => {
      convert[fromModel] = {};
      Object.defineProperty(convert[fromModel], "channels", { value: conversions[fromModel].channels });
      Object.defineProperty(convert[fromModel], "labels", { value: conversions[fromModel].labels });
      const routes = route(fromModel);
      const routeModels = Object.keys(routes);
      routeModels.forEach((toModel) => {
        const fn = routes[toModel];
        convert[fromModel][toModel] = wrapRounded(fn);
        convert[fromModel][toModel].raw = wrapRaw(fn);
      });
    });
    module.exports = convert;
  }
});

// ../../node_modules/ansi-styles/index.js
var require_ansi_styles = __commonJS({
  "../../node_modules/ansi-styles/index.js"(exports, module) {
    var wrapAnsi16 = (fn, offset) => (...args) => {
      const code = fn(...args);
      return `\x1B[${code + offset}m`;
    };
    var wrapAnsi256 = (fn, offset) => (...args) => {
      const code = fn(...args);
      return `\x1B[${38 + offset};5;${code}m`;
    };
    var wrapAnsi16m = (fn, offset) => (...args) => {
      const rgb = fn(...args);
      return `\x1B[${38 + offset};2;${rgb[0]};${rgb[1]};${rgb[2]}m`;
    };
    var ansi2ansi = (n) => n;
    var rgb2rgb = (r, g, b) => [r, g, b];
    var setLazyProperty = (object, property, get) => {
      Object.defineProperty(object, property, {
        get: () => {
          const value = get();
          Object.defineProperty(object, property, {
            value,
            enumerable: true,
            configurable: true
          });
          return value;
        },
        enumerable: true,
        configurable: true
      });
    };
    var colorConvert;
    var makeDynamicStyles = (wrap, targetSpace, identity, isBackground) => {
      if (colorConvert === void 0) {
        colorConvert = require_color_convert();
      }
      const offset = isBackground ? 10 : 0;
      const styles = {};
      for (const [sourceSpace, suite] of Object.entries(colorConvert)) {
        const name = sourceSpace === "ansi16" ? "ansi" : sourceSpace;
        if (sourceSpace === targetSpace) {
          styles[name] = wrap(identity, offset);
        } else if (typeof suite === "object") {
          styles[name] = wrap(suite[targetSpace], offset);
        }
      }
      return styles;
    };
    function assembleStyles() {
      const codes = /* @__PURE__ */ new Map();
      const styles = {
        modifier: {
          reset: [0, 0],
          // 21 isn't widely supported and 22 does the same thing
          bold: [1, 22],
          dim: [2, 22],
          italic: [3, 23],
          underline: [4, 24],
          inverse: [7, 27],
          hidden: [8, 28],
          strikethrough: [9, 29]
        },
        color: {
          black: [30, 39],
          red: [31, 39],
          green: [32, 39],
          yellow: [33, 39],
          blue: [34, 39],
          magenta: [35, 39],
          cyan: [36, 39],
          white: [37, 39],
          // Bright color
          blackBright: [90, 39],
          redBright: [91, 39],
          greenBright: [92, 39],
          yellowBright: [93, 39],
          blueBright: [94, 39],
          magentaBright: [95, 39],
          cyanBright: [96, 39],
          whiteBright: [97, 39]
        },
        bgColor: {
          bgBlack: [40, 49],
          bgRed: [41, 49],
          bgGreen: [42, 49],
          bgYellow: [43, 49],
          bgBlue: [44, 49],
          bgMagenta: [45, 49],
          bgCyan: [46, 49],
          bgWhite: [47, 49],
          // Bright color
          bgBlackBright: [100, 49],
          bgRedBright: [101, 49],
          bgGreenBright: [102, 49],
          bgYellowBright: [103, 49],
          bgBlueBright: [104, 49],
          bgMagentaBright: [105, 49],
          bgCyanBright: [106, 49],
          bgWhiteBright: [107, 49]
        }
      };
      styles.color.gray = styles.color.blackBright;
      styles.bgColor.bgGray = styles.bgColor.bgBlackBright;
      styles.color.grey = styles.color.blackBright;
      styles.bgColor.bgGrey = styles.bgColor.bgBlackBright;
      for (const [groupName, group] of Object.entries(styles)) {
        for (const [styleName, style] of Object.entries(group)) {
          styles[styleName] = {
            open: `\x1B[${style[0]}m`,
            close: `\x1B[${style[1]}m`
          };
          group[styleName] = styles[styleName];
          codes.set(style[0], style[1]);
        }
        Object.defineProperty(styles, groupName, {
          value: group,
          enumerable: false
        });
      }
      Object.defineProperty(styles, "codes", {
        value: codes,
        enumerable: false
      });
      styles.color.close = "\x1B[39m";
      styles.bgColor.close = "\x1B[49m";
      setLazyProperty(styles.color, "ansi", () => makeDynamicStyles(wrapAnsi16, "ansi16", ansi2ansi, false));
      setLazyProperty(styles.color, "ansi256", () => makeDynamicStyles(wrapAnsi256, "ansi256", ansi2ansi, false));
      setLazyProperty(styles.color, "ansi16m", () => makeDynamicStyles(wrapAnsi16m, "rgb", rgb2rgb, false));
      setLazyProperty(styles.bgColor, "ansi", () => makeDynamicStyles(wrapAnsi16, "ansi16", ansi2ansi, true));
      setLazyProperty(styles.bgColor, "ansi256", () => makeDynamicStyles(wrapAnsi256, "ansi256", ansi2ansi, true));
      setLazyProperty(styles.bgColor, "ansi16m", () => makeDynamicStyles(wrapAnsi16m, "rgb", rgb2rgb, true));
      return styles;
    }
    Object.defineProperty(module, "exports", {
      enumerable: true,
      get: assembleStyles
    });
  }
});

// ../../node_modules/wrap-ansi/index.js
var require_wrap_ansi = __commonJS({
  "../../node_modules/wrap-ansi/index.js"(exports, module) {
    var stringWidth = require_string_width();
    var stripAnsi = require_strip_ansi();
    var ansiStyles = require_ansi_styles();
    var ESCAPES = /* @__PURE__ */ new Set([
      "\x1B",
      "\x9B"
    ]);
    var END_CODE = 39;
    var ANSI_ESCAPE_BELL = "\x07";
    var ANSI_CSI = "[";
    var ANSI_OSC = "]";
    var ANSI_SGR_TERMINATOR = "m";
    var ANSI_ESCAPE_LINK = `${ANSI_OSC}8;;`;
    var wrapAnsi = (code) => `${ESCAPES.values().next().value}${ANSI_CSI}${code}${ANSI_SGR_TERMINATOR}`;
    var wrapAnsiHyperlink = (uri) => `${ESCAPES.values().next().value}${ANSI_ESCAPE_LINK}${uri}${ANSI_ESCAPE_BELL}`;
    var wordLengths = (string) => string.split(" ").map((character) => stringWidth(character));
    var wrapWord = (rows, word, columns) => {
      const characters = [...word];
      let isInsideEscape = false;
      let isInsideLinkEscape = false;
      let visible = stringWidth(stripAnsi(rows[rows.length - 1]));
      for (const [index, character] of characters.entries()) {
        const characterLength = stringWidth(character);
        if (visible + characterLength <= columns) {
          rows[rows.length - 1] += character;
        } else {
          rows.push(character);
          visible = 0;
        }
        if (ESCAPES.has(character)) {
          isInsideEscape = true;
          isInsideLinkEscape = characters.slice(index + 1).join("").startsWith(ANSI_ESCAPE_LINK);
        }
        if (isInsideEscape) {
          if (isInsideLinkEscape) {
            if (character === ANSI_ESCAPE_BELL) {
              isInsideEscape = false;
              isInsideLinkEscape = false;
            }
          } else if (character === ANSI_SGR_TERMINATOR) {
            isInsideEscape = false;
          }
          continue;
        }
        visible += characterLength;
        if (visible === columns && index < characters.length - 1) {
          rows.push("");
          visible = 0;
        }
      }
      if (!visible && rows[rows.length - 1].length > 0 && rows.length > 1) {
        rows[rows.length - 2] += rows.pop();
      }
    };
    var stringVisibleTrimSpacesRight = (string) => {
      const words = string.split(" ");
      let last = words.length;
      while (last > 0) {
        if (stringWidth(words[last - 1]) > 0) {
          break;
        }
        last--;
      }
      if (last === words.length) {
        return string;
      }
      return words.slice(0, last).join(" ") + words.slice(last).join("");
    };
    var exec2 = (string, columns, options = {}) => {
      if (options.trim !== false && string.trim() === "") {
        return "";
      }
      let returnValue = "";
      let escapeCode;
      let escapeUrl;
      const lengths = wordLengths(string);
      let rows = [""];
      for (const [index, word] of string.split(" ").entries()) {
        if (options.trim !== false) {
          rows[rows.length - 1] = rows[rows.length - 1].trimStart();
        }
        let rowLength = stringWidth(rows[rows.length - 1]);
        if (index !== 0) {
          if (rowLength >= columns && (options.wordWrap === false || options.trim === false)) {
            rows.push("");
            rowLength = 0;
          }
          if (rowLength > 0 || options.trim === false) {
            rows[rows.length - 1] += " ";
            rowLength++;
          }
        }
        if (options.hard && lengths[index] > columns) {
          const remainingColumns = columns - rowLength;
          const breaksStartingThisLine = 1 + Math.floor((lengths[index] - remainingColumns - 1) / columns);
          const breaksStartingNextLine = Math.floor((lengths[index] - 1) / columns);
          if (breaksStartingNextLine < breaksStartingThisLine) {
            rows.push("");
          }
          wrapWord(rows, word, columns);
          continue;
        }
        if (rowLength + lengths[index] > columns && rowLength > 0 && lengths[index] > 0) {
          if (options.wordWrap === false && rowLength < columns) {
            wrapWord(rows, word, columns);
            continue;
          }
          rows.push("");
        }
        if (rowLength + lengths[index] > columns && options.wordWrap === false) {
          wrapWord(rows, word, columns);
          continue;
        }
        rows[rows.length - 1] += word;
      }
      if (options.trim !== false) {
        rows = rows.map(stringVisibleTrimSpacesRight);
      }
      const pre = [...rows.join("\n")];
      for (const [index, character] of pre.entries()) {
        returnValue += character;
        if (ESCAPES.has(character)) {
          const { groups } = new RegExp(`(?:\\${ANSI_CSI}(?<code>\\d+)m|\\${ANSI_ESCAPE_LINK}(?<uri>.*)${ANSI_ESCAPE_BELL})`).exec(pre.slice(index).join("")) || { groups: {} };
          if (groups.code !== void 0) {
            const code2 = Number.parseFloat(groups.code);
            escapeCode = code2 === END_CODE ? void 0 : code2;
          } else if (groups.uri !== void 0) {
            escapeUrl = groups.uri.length === 0 ? void 0 : groups.uri;
          }
        }
        const code = ansiStyles.codes.get(Number(escapeCode));
        if (pre[index + 1] === "\n") {
          if (escapeUrl) {
            returnValue += wrapAnsiHyperlink("");
          }
          if (escapeCode && code) {
            returnValue += wrapAnsi(code);
          }
        } else if (character === "\n") {
          if (escapeCode && code) {
            returnValue += wrapAnsi(escapeCode);
          }
          if (escapeUrl) {
            returnValue += wrapAnsiHyperlink(escapeUrl);
          }
        }
      }
      return returnValue;
    };
    module.exports = (string, columns, options) => {
      return String(string).normalize().replace(/\r\n/g, "\n").split("\n").map((line) => exec2(line, columns, options)).join("\n");
    };
  }
});

// ../../node_modules/boxen/index.js
var require_boxen = __commonJS({
  "../../node_modules/boxen/index.js"(exports, module) {
    var stringWidth = require_string_width();
    var chalk29 = __require("chalk");
    var widestLine = require_widest_line();
    var cliBoxes = require_cli_boxes();
    var camelCase = require_camelcase();
    var ansiAlign = require_ansi_align();
    var wrapAnsi = require_wrap_ansi();
    var NL = "\n";
    var PAD = " ";
    var terminalColumns = () => {
      const { env, stdout, stderr } = process;
      if (stdout && stdout.columns) {
        return stdout.columns;
      }
      if (stderr && stderr.columns) {
        return stderr.columns;
      }
      if (env.COLUMNS) {
        return Number.parseInt(env.COLUMNS, 10);
      }
      return 80;
    };
    var getObject = (detail) => {
      return typeof detail === "number" ? {
        top: detail,
        right: detail * 3,
        bottom: detail,
        left: detail * 3
      } : {
        top: 0,
        right: 0,
        bottom: 0,
        left: 0,
        ...detail
      };
    };
    var getBorderChars = (borderStyle) => {
      const sides = [
        "topLeft",
        "topRight",
        "bottomRight",
        "bottomLeft",
        "vertical",
        "horizontal"
      ];
      let chararacters;
      if (typeof borderStyle === "string") {
        chararacters = cliBoxes[borderStyle];
        if (!chararacters) {
          throw new TypeError(`Invalid border style: ${borderStyle}`);
        }
      } else {
        for (const side of sides) {
          if (!borderStyle[side] || typeof borderStyle[side] !== "string") {
            throw new TypeError(`Invalid border style: ${side}`);
          }
        }
        chararacters = borderStyle;
      }
      return chararacters;
    };
    var makeTitle = (text, horizontal, alignement) => {
      let title = "";
      const textWidth = stringWidth(text);
      switch (alignement) {
        case "left":
          title = text + horizontal.slice(textWidth);
          break;
        case "right":
          title = horizontal.slice(textWidth) + text;
          break;
        default:
          horizontal = horizontal.slice(textWidth);
          if (horizontal.length % 2 === 1) {
            horizontal = horizontal.slice(Math.floor(horizontal.length / 2));
            title = horizontal.slice(1) + text + horizontal;
          } else {
            horizontal = horizontal.slice(horizontal.length / 2);
            title = horizontal + text + horizontal;
          }
          break;
      }
      return title;
    };
    var makeContentText = (text, padding, columns, align) => {
      text = ansiAlign(text, { align });
      let lines = text.split(NL);
      const textWidth = widestLine(text);
      const max = columns - padding.left - padding.right;
      if (textWidth > max) {
        const newLines = [];
        for (const line of lines) {
          const createdLines = wrapAnsi(line, max, { hard: true });
          const alignedLines = ansiAlign(createdLines, { align });
          const alignedLinesArray = alignedLines.split("\n");
          const longestLength = Math.max(...alignedLinesArray.map((s) => stringWidth(s)));
          for (const alignedLine of alignedLinesArray) {
            let paddedLine;
            switch (align) {
              case "center":
                paddedLine = PAD.repeat((max - longestLength) / 2) + alignedLine;
                break;
              case "right":
                paddedLine = PAD.repeat(max - longestLength) + alignedLine;
                break;
              default:
                paddedLine = alignedLine;
                break;
            }
            newLines.push(paddedLine);
          }
        }
        lines = newLines;
      }
      if (align === "center" && textWidth < max) {
        lines = lines.map((line) => PAD.repeat((max - textWidth) / 2) + line);
      } else if (align === "right" && textWidth < max) {
        lines = lines.map((line) => PAD.repeat(max - textWidth) + line);
      }
      const paddingLeft = PAD.repeat(padding.left);
      const paddingRight = PAD.repeat(padding.right);
      lines = lines.map((line) => paddingLeft + line + paddingRight);
      lines = lines.map((line) => {
        if (columns - stringWidth(line) > 0) {
          switch (align) {
            case "center":
              return line + PAD.repeat(columns - stringWidth(line));
            case "right":
              return line + PAD.repeat(columns - stringWidth(line));
            default:
              return line + PAD.repeat(columns - stringWidth(line));
          }
        }
        return line;
      });
      if (padding.top > 0) {
        lines = new Array(padding.top).fill(PAD.repeat(columns)).concat(lines);
      }
      if (padding.bottom > 0) {
        lines = lines.concat(new Array(padding.bottom).fill(PAD.repeat(columns)));
      }
      return lines.join(NL);
    };
    var isHex = (color) => color.match(/^#(?:[0-f]{3}){1,2}$/i);
    var isColorValid = (color) => typeof color === "string" && (chalk29[color] || isHex(color));
    var getColorFn = (color) => isHex(color) ? chalk29.hex(color) : chalk29[color];
    var getBGColorFn = (color) => isHex(color) ? chalk29.bgHex(color) : chalk29[camelCase(["bg", color])];
    module.exports = (text, options) => {
      options = {
        padding: 0,
        borderStyle: "single",
        dimBorder: false,
        textAlignment: "left",
        float: "left",
        titleAlignment: "left",
        ...options
      };
      if (options.align) {
        options.textAlignment = options.align;
      }
      const BORDERS_WIDTH = 2;
      if (options.borderColor && !isColorValid(options.borderColor)) {
        throw new Error(`${options.borderColor} is not a valid borderColor`);
      }
      if (options.backgroundColor && !isColorValid(options.backgroundColor)) {
        throw new Error(`${options.backgroundColor} is not a valid backgroundColor`);
      }
      const chars = getBorderChars(options.borderStyle);
      const padding = getObject(options.padding);
      const margin = getObject(options.margin);
      const colorizeBorder = (border) => {
        const newBorder = options.borderColor ? getColorFn(options.borderColor)(border) : border;
        return options.dimBorder ? chalk29.dim(newBorder) : newBorder;
      };
      const colorizeContent = (content) => options.backgroundColor ? getBGColorFn(options.backgroundColor)(content) : content;
      const columns = terminalColumns();
      let contentWidth = widestLine(wrapAnsi(text, columns - BORDERS_WIDTH, { hard: true, trim: false })) + padding.left + padding.right;
      let title = options.title && options.title.slice(0, columns - 4 - margin.left - margin.right);
      if (title) {
        title = ` ${title} `;
        if (stringWidth(title) > contentWidth) {
          contentWidth = stringWidth(title);
        }
      }
      if (margin.left && margin.right && contentWidth + BORDERS_WIDTH + margin.left + margin.right > columns) {
        const spaceForMargins = columns - contentWidth - BORDERS_WIDTH;
        const multiplier = spaceForMargins / (margin.left + margin.right);
        margin.left = Math.max(0, Math.floor(margin.left * multiplier));
        margin.right = Math.max(0, Math.floor(margin.right * multiplier));
      }
      contentWidth = Math.min(contentWidth, columns - BORDERS_WIDTH - margin.left - margin.right);
      text = makeContentText(text, padding, contentWidth, options.textAlignment);
      let marginLeft = PAD.repeat(margin.left);
      if (options.float === "center") {
        const marginWidth = Math.max((columns - contentWidth - BORDERS_WIDTH) / 2, 0);
        marginLeft = PAD.repeat(marginWidth);
      } else if (options.float === "right") {
        const marginWidth = Math.max(columns - contentWidth - margin.right - BORDERS_WIDTH, 0);
        marginLeft = PAD.repeat(marginWidth);
      }
      const horizontal = chars.horizontal.repeat(contentWidth);
      const top = colorizeBorder(NL.repeat(margin.top) + marginLeft + chars.topLeft + (title ? makeTitle(title, horizontal, options.titleAlignment) : horizontal) + chars.topRight);
      const bottom = colorizeBorder(marginLeft + chars.bottomLeft + horizontal + chars.bottomRight + NL.repeat(margin.bottom));
      const side = colorizeBorder(chars.vertical);
      const LINE_SEPARATOR = contentWidth + BORDERS_WIDTH + margin.left >= columns ? "" : NL;
      const lines = text.split(NL);
      const middle = lines.map((line) => {
        return marginLeft + side + colorizeContent(line) + side;
      }).join(LINE_SEPARATOR);
      return top + LINE_SEPARATOR + middle + LINE_SEPARATOR + bottom;
    };
    module.exports._borderStyles = cliBoxes;
  }
});

// src/initiate.ts
var import_boxen = __toESM(require_boxen());
var viteConfigFiles = ["vite.config.ts", "vite.config.js", "vite.config.mjs"];
var webpackConfigFiles = ["webpack.config.js"];
var hasDependency = (packageJson, name, matcher) => {
  const version = packageJson.dependencies?.[name] || packageJson.devDependencies?.[name];
  if (version && typeof matcher === "function") {
    return matcher(version);
  }
  return !!version;
};
var hasPeerDependency = (packageJson, name, matcher) => {
  const version = packageJson.peerDependencies?.[name];
  if (version && typeof matcher === "function") {
    return matcher(version);
  }
  return !!version;
};
var getFrameworkPreset = (packageJson, framework) => {
  const matcher = {
    dependencies: [false],
    peerDependencies: [false],
    files: [false]
  };
  const { preset, files, dependencies, peerDependencies, matcherFunction } = framework;
  let dependencySearches = [];
  if (Array.isArray(dependencies)) {
    dependencySearches = dependencies.map((name) => [name, void 0]);
  } else if (typeof dependencies === "object") {
    dependencySearches = Object.entries(dependencies);
  }
  if (dependencySearches.length > 0) {
    matcher.dependencies = dependencySearches.map(
      ([name, matchFn]) => hasDependency(packageJson, name, matchFn)
    );
  }
  let peerDependencySearches = [];
  if (Array.isArray(peerDependencies)) {
    peerDependencySearches = peerDependencies.map((name) => [name, void 0]);
  } else if (typeof peerDependencies === "object") {
    peerDependencySearches = Object.entries(peerDependencies);
  }
  if (peerDependencySearches.length > 0) {
    matcher.peerDependencies = peerDependencySearches.map(
      ([name, matchFn]) => hasPeerDependency(packageJson, name, matchFn)
    );
  }
  if (Array.isArray(files) && files.length > 0) {
    matcher.files = files.map((name) => fs.existsSync(name));
  }
  return matcherFunction(matcher) ? preset : null;
};
function detectFrameworkPreset(packageJson = {}) {
  const result = [...supportedTemplates, unsupportedTemplate].find((framework) => {
    return getFrameworkPreset(packageJson, framework) !== null;
  });
  return result ? result.preset : "UNDETECTED" /* UNDETECTED */;
}
async function detectBuilder(packageManager, projectType) {
  const viteConfig = findUp.sync(viteConfigFiles);
  const webpackConfig = findUp.sync(webpackConfigFiles);
  const dependencies = await packageManager.getAllDependencies();
  if (viteConfig || dependencies["vite"] && dependencies["webpack"] === void 0) {
    commandLog("Detected Vite project. Setting builder to Vite")();
    return "vite" /* Vite */;
  }
  if (webpackConfig || dependencies["webpack"] && dependencies["vite"] !== void 0) {
    commandLog("Detected webpack project. Setting builder to webpack")();
    return "webpack5" /* Webpack5 */;
  }
  switch (projectType) {
    case "SFC_VUE" /* SFC_VUE */:
    case "NUXT" /* NUXT */:
      return "vite" /* Vite */;
    case "REACT_SCRIPTS" /* REACT_SCRIPTS */:
    case "ANGULAR" /* ANGULAR */:
    case "REACT_NATIVE" /* REACT_NATIVE */:
    case "NEXTJS" /* NEXTJS */:
      return "webpack5" /* Webpack5 */;
    default:
      const { builder } = await prompts5({
        type: "select",
        name: "builder",
        message: "We were not able to detect the right builder for your project. Please select one:",
        choices: [
          { title: "Vite", value: "vite" /* Vite */ },
          { title: "Webpack 5", value: "webpack5" /* Webpack5 */ }
        ]
      });
      return builder;
  }
}
function isStorybookInstantiated(configDir = resolve(process.cwd(), ".storybook")) {
  return fs.existsSync(configDir);
}
async function detectPnp() {
  return !!findUp.sync([".pnp.js", ".pnp.cjs"]);
}
async function detectLanguage(packageManager) {
  let language = "javascript" /* JAVASCRIPT */;
  if (fs.existsSync("jsconfig.json")) {
    return language;
  }
  const isTypescriptDirectDependency = await packageManager.getAllDependencies().then((deps) => Boolean(deps["typescript"]));
  const typescriptVersion = await packageManager.getPackageVersion("typescript");
  const prettierVersion = await packageManager.getPackageVersion("prettier");
  const babelPluginTransformTypescriptVersion = await packageManager.getPackageVersion(
    "@babel/plugin-transform-typescript"
  );
  const typescriptEslintParserVersion = await packageManager.getPackageVersion(
    "@typescript-eslint/parser"
  );
  const eslintPluginStorybookVersion = await packageManager.getPackageVersion(
    "eslint-plugin-storybook"
  );
  if (isTypescriptDirectDependency && typescriptVersion) {
    if (semver.gte(typescriptVersion, "4.9.0") && (!prettierVersion || semver.gte(prettierVersion, "2.8.0")) && (!babelPluginTransformTypescriptVersion || semver.gte(babelPluginTransformTypescriptVersion, "7.20.0")) && (!typescriptEslintParserVersion || semver.gte(typescriptEslintParserVersion, "5.44.0")) && (!eslintPluginStorybookVersion || semver.gte(eslintPluginStorybookVersion, "0.6.8"))) {
      language = "typescript-4-9" /* TYPESCRIPT_4_9 */;
    } else if (semver.gte(typescriptVersion, "3.8.0")) {
      language = "typescript-3-8" /* TYPESCRIPT_3_8 */;
    } else if (semver.lt(typescriptVersion, "3.8.0")) {
      logger.warn("Detected TypeScript < 3.8, populating with JavaScript examples");
    }
  }
  if (fs.existsSync("tsconfig.json")) {
    language = "typescript-4-9" /* TYPESCRIPT_4_9 */;
  }
  return language;
}
async function detect(packageManager, options = {}) {
  const packageJson = await packageManager.retrievePackageJson();
  if (!packageJson) {
    return "UNDETECTED" /* UNDETECTED */;
  }
  if (await isNxProject()) {
    return "NX" /* NX */;
  }
  if (options.html) {
    return "HTML" /* HTML */;
  }
  return detectFrameworkPreset(packageJson);
}
var logger2 = console;
var sanitizeFramework = (framework) => {
  const matches = framework.match(/(@storybook\/\w+(?:-\w+)*)|(storybook-(\w+(?:-\w+)*))/g);
  if (!matches) {
    return void 0;
  }
  return matches[0];
};
async function configureMain({
  addons,
  extensions = ["js", "jsx", "mjs", "ts", "tsx"],
  storybookConfigFolder,
  language,
  prefixes = [],
  ...custom
}) {
  const srcPath = path9.resolve(storybookConfigFolder, "../src");
  const prefix = await fse.pathExists(srcPath) ? "../src" : "../stories";
  const config = {
    stories: [`${prefix}/**/*.mdx`, `${prefix}/**/*.stories.@(${extensions.join("|")})`],
    addons,
    ...custom
  };
  const isTypescript = language === "typescript-4-9" /* TYPESCRIPT_4_9 */ || language === "typescript-3-8" /* TYPESCRIPT_3_8 */;
  let mainConfigTemplate = dedent`<<import>><<prefix>>const config<<type>> = <<mainContents>>;
    export default config;`;
  const frameworkPackage = sanitizeFramework(custom.framework?.name);
  if (!frameworkPackage) {
    mainConfigTemplate = mainConfigTemplate.replace("<<import>>", "").replace("<<type>>", "");
    logger2.warn("Could not find framework package name");
  }
  const mainContents = JSON.stringify(config, null, 2).replace(/['"]%%/g, "").replace(/%%['"]/g, "");
  const imports = [];
  const finalPrefixes = [...prefixes];
  if (custom.framework?.name.includes("path.dirname(")) {
    imports.push(`import path from 'path';`);
  }
  if (isTypescript) {
    imports.push(`import type { StorybookConfig } from '${frameworkPackage}';`);
  } else {
    finalPrefixes.push(`/** @type { import('${frameworkPackage}').StorybookConfig } */`);
  }
  const mainJsContents = mainConfigTemplate.replace("<<import>>", `${imports.join("\n\n")}

`).replace("<<prefix>>", finalPrefixes.length > 0 ? `${finalPrefixes.join("\n\n")}
` : "").replace("<<type>>", isTypescript ? ": StorybookConfig" : "").replace("<<mainContents>>", mainContents);
  const prettier = (await import('prettier')).default;
  const mainPath = `./${storybookConfigFolder}/main.${isTypescript ? "ts" : "js"}`;
  const prettyMain = prettier.format(dedent(mainJsContents), {
    ...prettier.resolveConfig.sync(process.cwd()),
    filepath: mainPath
  });
  await fse.writeFile(mainPath, prettyMain, { encoding: "utf8" });
}
async function configurePreview(options) {
  const { prefix: frameworkPrefix = "" } = options.frameworkPreviewParts || {};
  const isTypescript = options.language === "typescript-4-9" /* TYPESCRIPT_4_9 */ || options.language === "typescript-3-8" /* TYPESCRIPT_3_8 */;
  const rendererPackage = options.rendererId && !externalFrameworks.map(({ name }) => name).includes(options.rendererId) ? `@storybook/${options.rendererId}` : null;
  const previewPath = `./${options.storybookConfigFolder}/preview.${isTypescript ? "ts" : "js"}`;
  if (await fse.pathExists(previewPath)) {
    return;
  }
  const prefix = [
    isTypescript && rendererPackage ? `import type { Preview } from '${rendererPackage}'` : "",
    frameworkPrefix
  ].filter(Boolean).join("\n");
  const preview = dedent`
    ${prefix}${prefix.length > 0 ? "\n" : ""}
    ${!isTypescript && rendererPackage ? `/** @type { import('${rendererPackage}').Preview } */
` : ""}const preview${isTypescript ? ": Preview" : ""} = {
      parameters: {
        actions: { argTypesRegex: '^on[A-Z].*' },
        controls: {
          matchers: {
           color: /(background|color)$/i,
           date: /Date$/,
          },
        },
      },
    };
    
    export default preview;
    `.replace("  \n", "").trim();
  const prettier = (await import('prettier')).default;
  const prettyPreview = prettier.format(preview, {
    ...prettier.resolveConfig.sync(process.cwd()),
    filepath: previewPath
  });
  await fse.writeFile(previewPath, prettyPreview, { encoding: "utf8" });
}
var generateStorybookBabelConfigInCWD = async () => {
  const target = process.cwd();
  return generateStorybookBabelConfig({ target });
};
var getBabelPresets = ({ typescript, jsx }) => {
  const dependencies = ["@babel/preset-env"];
  if (typescript) {
    dependencies.push("@babel/preset-typescript");
  }
  if (jsx) {
    dependencies.push("@babel/preset-react");
  }
  return dependencies;
};
var writeBabelConfigFile = async ({
  location,
  typescript,
  jsx
}) => {
  const fileLocation = location || path9.join(process.cwd(), ".babelrc.json");
  const presets = [["@babel/preset-env", { targets: { chrome: 100 } }]];
  if (typescript) {
    presets.push("@babel/preset-typescript");
  }
  if (jsx) {
    presets.push("@babel/preset-react");
  }
  const contents = JSON.stringify(
    {
      sourceType: "unambiguous",
      presets,
      plugins: []
    },
    null,
    2
  );
  await writeFile(fileLocation, contents);
};
var generateStorybookBabelConfig = async ({ target }) => {
  logger.info(`Generating the Storybook default babel config at ${target}`);
  const fileName = ".babelrc.json";
  const location = path9.join(target, fileName);
  const exists = await pathExists(location);
  if (exists) {
    const { overwrite } = await prompts5({
      type: "confirm",
      initial: false,
      name: "overwrite",
      message: `${fileName} already exists. Would you like overwrite it?`
    });
    if (overwrite === false) {
      logger.warn(`Cancelled, babel config file was NOT written to file-system.`);
      return;
    }
  }
  const { typescript, jsx } = await prompts5([
    {
      type: "confirm",
      initial: false,
      name: "typescript",
      message: `Do you want to add the TypeScript preset?`
    },
    {
      type: "confirm",
      initial: false,
      name: "jsx",
      message: `Do you want to add the React preset?`
    }
  ]);
  const dependencies = getBabelPresets({ typescript, jsx });
  logger.info(`Writing file to ${location}`);
  await writeBabelConfigFile({ location, typescript, jsx });
  const packageManager = JsPackageManagerFactory.getPackageManager();
  logger.info(`Installing dependencies (${dependencies.join(", ")})`);
  await packageManager.addDependencies({ installAsDevDependencies: true }, dependencies);
};
var SUPPORTED_ESLINT_EXTENSIONS = ["js", "cjs", "json"];
var UNSUPPORTED_ESLINT_EXTENSIONS = ["yaml", "yml"];
var findEslintFile = () => {
  const filePrefix = ".eslintrc";
  const unsupportedExtension = UNSUPPORTED_ESLINT_EXTENSIONS.find(
    (ext) => fse.existsSync(`${filePrefix}.${ext}`)
  );
  if (unsupportedExtension) {
    throw new Error(unsupportedExtension);
  }
  const extension = SUPPORTED_ESLINT_EXTENSIONS.find(
    (ext) => fse.existsSync(`${filePrefix}.${ext}`)
  );
  return extension ? `${filePrefix}.${extension}` : null;
};
async function extractEslintInfo(packageManager) {
  const allDependencies = await packageManager.getAllDependencies();
  const packageJson = await packageManager.retrievePackageJson();
  let eslintConfigFile = null;
  try {
    eslintConfigFile = findEslintFile();
  } catch (err) {
  }
  const isStorybookPluginInstalled = !!allDependencies["eslint-plugin-storybook"];
  const hasEslint = allDependencies.eslint || eslintConfigFile || packageJson.eslintConfig;
  return { hasEslint, isStorybookPluginInstalled, eslintConfigFile };
}
async function configureEslintPlugin(eslintFile, packageManager) {
  if (eslintFile) {
    paddedLog(`Configuring Storybook ESLint plugin at ${eslintFile}`);
    if (eslintFile.endsWith("json")) {
      const eslintConfig = await readJson(eslintFile);
      const existingConfigValue = Array.isArray(eslintConfig.extends) ? eslintConfig.extends : [eslintConfig.extends].filter(Boolean);
      eslintConfig.extends = [...existingConfigValue || [], "plugin:storybook/recommended"];
      const eslintFileContents = await readFile(eslintFile, "utf8");
      const spaces = detectIndent(eslintFileContents).amount || 2;
      await writeJson(eslintFile, eslintConfig, { spaces });
    } else {
      const eslint = await readConfig(eslintFile);
      const extendsConfig = eslint.getFieldValue(["extends"]) || [];
      const existingConfigValue = Array.isArray(extendsConfig) ? extendsConfig : [extendsConfig].filter(Boolean);
      eslint.setFieldValue(
        ["extends"],
        [...existingConfigValue || [], "plugin:storybook/recommended"]
      );
      await writeConfig(eslint);
    }
  } else {
    paddedLog(`Configuring eslint-plugin-storybook in your package.json`);
    const packageJson = await packageManager.retrievePackageJson();
    await packageManager.writePackageJson({
      ...packageJson,
      eslintConfig: {
        ...packageJson.eslintConfig,
        extends: [...packageJson.eslintConfig?.extends || [], "plugin:storybook/recommended"]
      }
    });
  }
}
var suggestESLintPlugin = async () => {
  const { shouldInstall } = await prompts5({
    type: "confirm",
    name: "shouldInstall",
    message: dedent`
        We have detected that you're using ESLint. Storybook provides a plugin that gives the best experience with Storybook and helps follow best practices: ${chalk12.yellow(
      "https://github.com/storybookjs/eslint-plugin-storybook#readme"
    )}

        Would you like to install it?
      `,
    initial: true
  });
  return shouldInstall;
};

// src/generators/baseGenerator.ts
var logger4 = console;
var defaultOptions = {
  extraPackages: [],
  extraAddons: [],
  staticDir: void 0,
  addScripts: true,
  addMainFile: true,
  addComponents: true,
  skipBabel: false,
  extraMain: void 0,
  framework: void 0,
  extensions: void 0,
  componentsDestinationPath: void 0,
  storybookConfigFolder: ".storybook"
};
var getBuilderDetails = (builder) => {
  const map = versions_default;
  if (map[builder]) {
    return builder;
  }
  const builderPackage = `@storybook/${builder}`;
  if (map[builderPackage]) {
    return builderPackage;
  }
  return builder;
};
var getExternalFramework = (framework) => externalFrameworks.find(
  (exFramework) => framework !== void 0 && (exFramework.name === framework || exFramework.packageName === framework || exFramework?.frameworks?.some?.((item) => item === framework))
);
var getFrameworkPackage = (framework, renderer, builder) => {
  const externalFramework = getExternalFramework(framework);
  const storybookBuilder = builder?.replace(/^@storybook\/builder-/, "");
  const storybookFramework = framework?.replace(/^@storybook\//, "");
  if (externalFramework === void 0) {
    const frameworkPackage = framework ? `@storybook/${storybookFramework}` : `@storybook/${renderer}-${storybookBuilder}`;
    if (versions_default[frameworkPackage]) {
      return frameworkPackage;
    }
    throw new Error(
      dedent`
        Could not find framework package: ${frameworkPackage}.
        Make sure this package exists, and if it does, please file an issue as this might be a bug in Storybook.
      `
    );
  }
  if (externalFramework.frameworks !== void 0) {
    return externalFramework.frameworks.find(
      (item) => item.match(new RegExp(`-${storybookBuilder}`))
    );
  }
  return externalFramework.packageName;
};
var getRendererPackage = (framework, renderer) => {
  const externalFramework = getExternalFramework(framework);
  if (externalFramework !== void 0)
    return externalFramework.renderer || externalFramework.packageName;
  return `@storybook/${renderer}`;
};
var applyRequireWrapper = (packageName) => `%%getAbsolutePath('${packageName}')%%`;
var getFrameworkDetails = (renderer, builder, pnp, language, framework, shouldApplyRequireWrapperOnPackageNames) => {
  const frameworkPackage = getFrameworkPackage(framework, renderer, builder);
  const frameworkPackagePath = shouldApplyRequireWrapperOnPackageNames ? applyRequireWrapper(frameworkPackage) : frameworkPackage;
  const rendererPackage = getRendererPackage(framework, renderer);
  const rendererPackagePath = shouldApplyRequireWrapperOnPackageNames ? applyRequireWrapper(rendererPackage) : rendererPackage;
  const builderPackage = getBuilderDetails(builder);
  const builderPackagePath = shouldApplyRequireWrapperOnPackageNames ? applyRequireWrapper(builderPackage) : builderPackage;
  const isExternalFramework = !!getExternalFramework(frameworkPackage);
  const isKnownFramework = isExternalFramework || !!versions_default[frameworkPackage];
  const isKnownRenderer = !!versions_default[rendererPackage];
  if (isKnownFramework) {
    return {
      packages: [rendererPackage, frameworkPackage],
      framework: frameworkPackagePath,
      rendererId: renderer,
      type: "framework"
    };
  }
  if (isKnownRenderer) {
    return {
      packages: [rendererPackage, builderPackage],
      builder: builderPackagePath,
      renderer: rendererPackagePath,
      rendererId: renderer,
      type: "renderer"
    };
  }
  throw new Error(
    `Could not find the framework (${frameworkPackage}) or renderer (${rendererPackage}) package`
  );
};
var stripVersions = (addons) => addons.map((addon) => getPackageDetails(addon)[0]);
var hasInteractiveStories = (rendererId) => ["react", "angular", "preact", "svelte", "vue", "vue3", "html", "solid", "qwik"].includes(
  rendererId
);
var hasFrameworkTemplates = (framework) => ["angular", "nextjs", "nuxt"].includes(framework);
async function baseGenerator(packageManager, npmOptions, {
  language,
  builder = "webpack5" /* Webpack5 */,
  pnp,
  frameworkPreviewParts,
  yes: skipPrompts
}, renderer, options = defaultOptions, framework) {
  const isStorybookInMonorepository = packageManager.isStorybookInMonorepo();
  const shouldApplyRequireWrapperOnPackageNames = isStorybookInMonorepository || pnp;
  const {
    extraAddons: extraAddonPackages,
    extraPackages,
    staticDir,
    addScripts,
    addMainFile,
    addComponents,
    skipBabel,
    extraMain,
    extensions,
    storybookConfigFolder,
    componentsDestinationPath
  } = {
    ...defaultOptions,
    ...options
  };
  const {
    packages: frameworkPackages5,
    type,
    rendererId,
    framework: frameworkInclude,
    builder: builderInclude
  } = getFrameworkDetails(
    renderer,
    builder,
    pnp,
    language,
    framework,
    shouldApplyRequireWrapperOnPackageNames
  );
  const addons = [
    "@storybook/addon-links",
    "@storybook/addon-essentials",
    ...stripVersions(extraAddonPackages)
  ];
  const addonPackages = [
    "@storybook/addon-links",
    "@storybook/addon-essentials",
    "@storybook/blocks",
    ...extraAddonPackages
  ];
  if (hasInteractiveStories(rendererId)) {
    addons.push("@storybook/addon-interactions");
    addonPackages.push("@storybook/addon-interactions", "@storybook/testing-library@^0.2.0-next.0");
  }
  const files = await fse.readdir(process.cwd());
  const packageJson = await packageManager.retrievePackageJson();
  const installedDependencies = new Set(
    Object.keys({ ...packageJson.dependencies, ...packageJson.devDependencies })
  );
  if (!installedDependencies.has("react")) {
    addonPackages.push("react");
  }
  if (!installedDependencies.has("react-dom")) {
    addonPackages.push("react-dom");
  }
  if (type === "renderer") {
    throw new Error(
      dedent`
        Sorry, for now, you can not do this, please use a framework such as @storybook/react-webpack5

        https://github.com/storybookjs/storybook/issues/18360
      `
    );
  }
  const allPackages = [
    "storybook",
    getExternalFramework(rendererId) ? void 0 : `@storybook/${rendererId}`,
    ...frameworkPackages5,
    ...addonPackages,
    ...extraPackages
  ].filter(Boolean);
  const packages = [...new Set(allPackages)].filter(
    (packageToInstall) => !installedDependencies.has(getPackageDetails(packageToInstall)[0])
  );
  logger4.log();
  const versionedPackagesSpinner = ora({
    indent: 2,
    text: `Getting the correct version of ${packages.length} packages`
  }).start();
  const versionedPackages = await packageManager.getVersionedPackages(packages);
  versionedPackagesSpinner.succeed();
  const depsToInstall = [...versionedPackages];
  if (builder !== "vite" /* Vite */ && !skipBabel) {
    const frameworksThatNeedBabelConfig2 = [
      "@storybook/react-webpack5",
      "@storybook/vue-webpack5",
      "@storybook/vue3-webpack5",
      "@storybook/html-webpack5",
      "@storybook/web-components-webpack5"
    ];
    const needsBabelConfig = frameworkPackages5.find(
      (pkg2) => frameworksThatNeedBabelConfig2.includes(pkg2)
    );
    const hasNoBabelFile = !files.some(
      (fname) => fname.startsWith(".babel") || fname.startsWith("babel")
    );
    if (hasNoBabelFile && needsBabelConfig) {
      const isTypescript = language !== "javascript" /* JAVASCRIPT */;
      const isReact = rendererId === "react";
      depsToInstall.push(
        ...getBabelPresets({
          typescript: isTypescript,
          jsx: isReact
        })
      );
      await writeBabelConfigFile({
        typescript: isTypescript,
        jsx: isReact
      });
    }
  }
  try {
    if (process.env.CI !== "true") {
      const { hasEslint, isStorybookPluginInstalled, eslintConfigFile } = await extractEslintInfo(
        packageManager
      );
      if (hasEslint && !isStorybookPluginInstalled) {
        if (skipPrompts || await suggestESLintPlugin()) {
          depsToInstall.push("eslint-plugin-storybook");
          await configureEslintPlugin(eslintConfigFile, packageManager);
        }
      }
    }
  } catch (err) {
  }
  if (depsToInstall.length > 0) {
    const addDependenciesSpinner = ora({
      indent: 2,
      text: "Installing Storybook dependencies"
    }).start();
    await packageManager.addDependencies({ ...npmOptions, packageJson }, depsToInstall);
    addDependenciesSpinner.succeed();
  }
  await fse.ensureDir(`./${storybookConfigFolder}`);
  if (addMainFile) {
    const prefixes = shouldApplyRequireWrapperOnPackageNames ? [
      'import { join, dirname } from "path"',
      language === "javascript" /* JAVASCRIPT */ ? dedent`/**
            * This function is used to resolve the absolute path of a package.
            * It is needed in projects that use Yarn PnP or are set up within a monorepo.
            */ 
            function getAbsolutePath(value) {
              return dirname(require.resolve(join(value, 'package.json')))
            }` : dedent`/**
          * This function is used to resolve the absolute path of a package.
          * It is needed in projects that use Yarn PnP or are set up within a monorepo.
          */ 
          function getAbsolutePath(value: string): any {
            return dirname(require.resolve(join(value, 'package.json')))
          }`
    ] : [];
    await configureMain({
      framework: { name: frameworkInclude, options: options.framework || {} },
      prefixes,
      storybookConfigFolder,
      docs: { autodocs: "tag" },
      addons: shouldApplyRequireWrapperOnPackageNames ? addons.map((addon) => applyRequireWrapper(addon)) : addons,
      extensions,
      language,
      ...staticDir ? { staticDirs: [path9.join("..", staticDir)] } : null,
      ...extraMain,
      ...type !== "framework" ? {
        core: {
          builder: builderInclude
        }
      } : {}
    });
  }
  await configurePreview({
    frameworkPreviewParts,
    storybookConfigFolder,
    language,
    rendererId
  });
  if (addScripts) {
    await packageManager.addStorybookCommandInScripts({
      port: 6006
    });
  }
  if (addComponents) {
    const templateLocation = hasFrameworkTemplates(framework) ? framework : rendererId;
    await copyTemplateFiles({
      renderer: templateLocation,
      packageManager,
      language,
      destination: componentsDestinationPath
    });
  }
}
var ANGULAR_JSON_PATH = "angular.json";
var compoDocPreviewPrefix = dedent17`
  import { setCompodocJson } from "@storybook/addon-docs/angular";
  import docJson from "../documentation.json";
  setCompodocJson(docJson);
`.trimStart();
var promptForCompoDocs = async () => {
  const { useCompoDoc } = await prompts5({
    type: "confirm",
    name: "useCompoDoc",
    message: "Do you want to use Compodoc for documentation?"
  });
  return useCompoDoc;
};
var AngularJSON = class {
  constructor() {
    if (!fs.existsSync(ANGULAR_JSON_PATH)) {
      throw new Error(
        "An angular.json file was not found in the current working directory. Storybook needs it to work properly, so please rerun the command at the root of your project, where the angular.json file is located. More info: https://storybook.js.org/docs/angular/faq#error-no-angularjson-file-found"
      );
    }
    const jsonContent = fs.readFileSync(ANGULAR_JSON_PATH, "utf8");
    this.json = JSON.parse(jsonContent);
  }
  get projects() {
    return this.json.projects;
  }
  get projectsWithoutStorybook() {
    return Object.keys(this.projects).filter((projectName) => {
      const { architect } = this.projects[projectName];
      return !architect.storybook;
    });
  }
  get hasStorybookBuilder() {
    return Object.keys(this.projects).some((projectName) => {
      const { architect } = this.projects[projectName];
      return Object.keys(architect).some((key) => {
        return architect[key].builder === "@storybook/angular:start-storybook";
      });
    });
  }
  get rootProject() {
    const rootProjectName = Object.keys(this.projects).find((projectName) => {
      const { root } = this.projects[projectName];
      return root === "" || root === ".";
    });
    return rootProjectName ? this.projects[rootProjectName] : null;
  }
  getProjectSettingsByName(projectName) {
    return this.projects[projectName];
  }
  async getProjectName() {
    if (this.projectsWithoutStorybook.length > 1) {
      const { projectName } = await prompts5({
        type: "select",
        name: "projectName",
        message: "For which project do you want to generate Storybook configuration?",
        choices: this.projectsWithoutStorybook.map((name) => ({
          title: name,
          value: name
        }))
      });
      return projectName;
    }
    return this.projectsWithoutStorybook[0];
  }
  addStorybookEntries({
    angularProjectName,
    storybookFolder,
    useCompodoc,
    root
  }) {
    const { architect } = this.projects[angularProjectName];
    const baseOptions = {
      configDir: storybookFolder,
      browserTarget: `${angularProjectName}:build`,
      compodoc: useCompodoc,
      ...useCompodoc && { compodocArgs: ["-e", "json", "-d", root || "."] }
    };
    if (!architect.storybook) {
      architect.storybook = {
        builder: "@storybook/angular:start-storybook",
        options: {
          ...baseOptions,
          port: 6006
        }
      };
    }
    if (!architect["build-storybook"]) {
      architect["build-storybook"] = {
        builder: "@storybook/angular:build-storybook",
        options: {
          ...baseOptions,
          outputDir: Object.keys(this.projects).length === 1 ? `storybook-static` : `dist/storybook/${angularProjectName}`
        }
      };
    }
  }
  write() {
    fs.writeFileSync(ANGULAR_JSON_PATH, JSON.stringify(this.json, null, 2));
  }
};

// src/generators/ANGULAR/index.ts
var generator = async (packageManager, npmOptions, options, commandOptions) => {
  const angularVersion = await packageManager.getPackageVersion("@angular/core");
  const isWebpack5 = angularVersion && semver.gte(angularVersion, "12.0.0");
  const updatedOptions = isWebpack5 ? { ...options, builder: "webpack5" /* Webpack5 */ } : options;
  const angularJSON = new AngularJSON();
  if (!angularJSON.projects || angularJSON.projects && Object.keys(angularJSON.projects).length === 0) {
    throw new Error(
      "Storybook was not able to find any projects in your angular.json file. Are you sure this is an Angular CLI project?"
    );
  }
  if (angularJSON.projectsWithoutStorybook.length === 0) {
    throw new Error(
      "Every project in your workspace is already set up with Storybook. There is nothing to do!"
    );
  }
  const angularProjectName = await angularJSON.getProjectName();
  paddedLog(`Adding Storybook support to your "${angularProjectName}" project`);
  const angularProject = angularJSON.getProjectSettingsByName(angularProjectName);
  if (!angularProject) {
    throw new Error(
      `Somehow we were not able to retrieve the "${angularProjectName}" project in your angular.json file. This is likely a bug in Storybook, please file an issue.`
    );
  }
  const { root, projectType } = angularProject;
  const { projects } = angularJSON;
  const useCompodoc = commandOptions.yes ? true : await promptForCompoDocs();
  const storybookFolder = root ? `${root}/.storybook` : ".storybook";
  angularJSON.addStorybookEntries({
    angularProjectName,
    storybookFolder,
    useCompodoc,
    root
  });
  angularJSON.write();
  await baseGenerator(
    packageManager,
    npmOptions,
    {
      ...updatedOptions,
      ...useCompodoc && {
        frameworkPreviewParts: {
          prefix: compoDocPreviewPrefix
        }
      }
    },
    "angular",
    {
      ...useCompodoc && { extraPackages: ["@compodoc/compodoc", "@storybook/addon-docs"] },
      addScripts: false,
      componentsDestinationPath: root ? `${root}/src/stories` : void 0,
      storybookConfigFolder: storybookFolder
    },
    "angular"
  );
  if (Object.keys(projects).length === 1) {
    packageManager.addScripts({
      storybook: `ng run ${angularProjectName}:storybook`,
      "build-storybook": `ng run ${angularProjectName}:build-storybook`
    });
  }
  let projectTypeValue = projectType || "application";
  if (projectTypeValue !== "application" && projectTypeValue !== "library") {
    projectTypeValue = "application";
  }
  const templateDir = join(getCliDir(), "templates", "angular", projectTypeValue);
  if (templateDir) {
    copyTemplate(templateDir, root || void 0);
  }
  return {
    projectName: angularProjectName,
    configDir: storybookFolder
  };
};
var ANGULAR_default = generator;

// src/generators/EMBER/index.ts
var generator2 = async (packageManager, npmOptions, options) => {
  await baseGenerator(packageManager, npmOptions, options, "ember", {
    extraPackages: [
      // babel-plugin-ember-modules-api-polyfill is a peerDep of @storybook/ember
      "babel-plugin-ember-modules-api-polyfill",
      // babel-plugin-htmlbars-inline-precompile is a peerDep of @storybook/ember
      "babel-plugin-htmlbars-inline-precompile"
    ],
    staticDir: "dist"
  });
};
var EMBER_default = generator2;

// src/generators/REACT/index.ts
var generator3 = async (packageManager, npmOptions, options) => {
  const language = await detectLanguage(packageManager);
  const extraPackages = language === "javascript" /* JAVASCRIPT */ ? ["prop-types"] : [];
  await baseGenerator(packageManager, npmOptions, options, "react", {
    extraPackages,
    extraAddons: ["@storybook/addon-onboarding"]
  });
};
var REACT_default = generator3;

// src/generators/REACT_NATIVE/index.ts
var generator4 = async (packageManager, npmOptions) => {
  const packageJson = await packageManager.retrievePackageJson();
  const missingReactDom = !packageJson.dependencies["react-dom"] && !packageJson.devDependencies["react-dom"];
  const reactVersion = packageJson.dependencies.react;
  const packagesToResolve = [
    // addon-ondevice-controls peer deps
    "react-native-safe-area-context",
    "@react-native-async-storage/async-storage",
    "@react-native-community/datetimepicker",
    "@react-native-community/slider",
    "@storybook/addon-ondevice-controls",
    "@storybook/addon-ondevice-actions",
    "@storybook/react-native"
  ];
  const packagesWithFixedVersion = [
    "@storybook/addon-actions@^6.5.16",
    "@storybook/addon-controls@^6.5.16"
  ];
  const resolvedPackages = await packageManager.getVersionedPackages(packagesToResolve);
  const babelDependencies = await getBabelDependencies(packageManager, packageJson);
  const packages = [
    ...babelDependencies,
    ...packagesWithFixedVersion,
    ...resolvedPackages,
    missingReactDom && reactVersion && `react-dom@${reactVersion}`
  ].filter(Boolean);
  await packageManager.addDependencies({ ...npmOptions, packageJson }, packages);
  packageManager.addScripts({
    "storybook-generate": "sb-rn-get-stories",
    "storybook-watch": "sb-rn-watcher"
  });
  const storybookConfigFolder = ".storybook";
  await copyTemplateFiles({
    packageManager,
    renderer: "react-native",
    language: "javascript" /* JAVASCRIPT */,
    destination: storybookConfigFolder,
    includeCommonAssets: false
  });
};
var REACT_NATIVE_default = generator4;
var generator5 = async (packageManager, npmOptions, options) => {
  const monorepoRootPath = path9.join(__dirname, "..", "..", "..", "..", "..", "..");
  const extraMain = options.linkable ? {
    webpackFinal: `%%(config) => {
      // add monorepo root as a valid directory to import modules from
      config.resolve.plugins.forEach((p) => {
        if (Array.isArray(p.appSrcs)) {
          p.appSrcs.push('${monorepoRootPath}');
              }
            });
          return config;
          }
    %%`
  } : {};
  const craVersion = await packageManager.getPackageVersion("react-scripts");
  const isCra5OrHigher = craVersion && semver.gte(craVersion, "5.0.0");
  const updatedOptions = isCra5OrHigher ? { ...options, builder: "webpack5" /* Webpack5 */ } : options;
  const extraPackages = [];
  if (isCra5OrHigher) {
    extraPackages.push("webpack");
    extraPackages.push("babel-plugin-named-exports-order");
    extraPackages.push("prop-types");
  }
  const version = versions_default["@storybook/preset-create-react-app"];
  const extraAddons = [
    `@storybook/preset-create-react-app@${version}`,
    "@storybook/addon-onboarding"
  ];
  if (!isCra5OrHigher) {
    throw new Error(dedent17`
      Storybook 7.0+ doesn't support react-scripts@<5.0.0.

      https://github.com/storybookjs/storybook/blob/next/MIGRATION.md#create-react-app-dropped-cra4-support
    `);
  }
  await baseGenerator(packageManager, npmOptions, updatedOptions, "react", {
    extraAddons,
    extraPackages,
    staticDir: fs.existsSync(path9.resolve("./public")) ? "public" : void 0,
    skipBabel: true,
    extraMain
  });
};
var REACT_SCRIPTS_default = generator5;

// src/generators/NEXTJS/index.ts
var generator6 = async (packageManager, npmOptions, options) => {
  await baseGenerator(
    packageManager,
    npmOptions,
    options,
    "react",
    {
      extraAddons: ["@storybook/addon-onboarding"]
    },
    "nextjs"
  );
};
var NEXTJS_default = generator6;

// src/generators/SFC_VUE/index.ts
var generator7 = async (packageManager, npmOptions, options) => {
  await baseGenerator(packageManager, npmOptions, options, "vue");
};
var SFC_VUE_default = generator7;

// src/generators/VUE/index.ts
var generator8 = async (packageManager, npmOptions, options) => {
  const extraPackages = options.builder === "webpack5" /* Webpack5 */ ? ["vue-loader@^15.7.0"] : [];
  await baseGenerator(packageManager, npmOptions, options, "vue", {
    extraPackages
  });
};
var VUE_default = generator8;

// src/generators/VUE3/index.ts
var generator9 = async (packageManager, npmOptions, options) => {
  const extraPackages = options.builder === "webpack5" /* Webpack5 */ ? ["vue-loader@^17.0.0", "@vue/compiler-sfc@^3.2.0"] : [];
  await baseGenerator(packageManager, npmOptions, options, "vue3", {
    extraPackages
  });
};
var VUE3_default = generator9;

// src/generators/WEBPACK_REACT/index.ts
var generator10 = async (packageManager, npmOptions, options) => {
  await baseGenerator(packageManager, npmOptions, options, "react", {
    extraAddons: ["@storybook/addon-onboarding"]
  });
};
var WEBPACK_REACT_default = generator10;

// src/generators/HTML/index.ts
var generator11 = async (packageManager, npmOptions, options) => {
  await baseGenerator(packageManager, npmOptions, options, "html");
};
var HTML_default = generator11;

// src/generators/WEB-COMPONENTS/index.ts
var generator12 = async (packageManager, npmOptions, options) => {
  return baseGenerator(packageManager, npmOptions, options, "web-components", {
    extraPackages: ["lit"]
  });
};
var WEB_COMPONENTS_default = generator12;

// src/generators/PREACT/index.ts
var generator13 = async (packageManager, npmOptions, options) => {
  await baseGenerator(packageManager, npmOptions, options, "preact");
};
var PREACT_default = generator13;

// src/generators/SVELTE/index.ts
var generator14 = async (packageManager, npmOptions, options) => {
  await baseGenerator(packageManager, npmOptions, options, "svelte", {
    extensions: ["js", "jsx", "ts", "tsx", "svelte"]
  });
};
var SVELTE_default = generator14;

// src/generators/QWIK/index.ts
var generator15 = async (packageManager, npmOptions, options) => {
  await baseGenerator(packageManager, npmOptions, options, "qwik", {}, "qwik");
};
var QWIK_default = generator15;

// src/generators/SVELTEKIT/index.ts
var generator16 = async (packageManager, npmOptions, options) => {
  await baseGenerator(packageManager, npmOptions, options, "svelte", void 0, "sveltekit");
};
var SVELTEKIT_default = generator16;

// src/generators/SOLID/index.ts
var generator17 = async (packageManager, npmOptions, options) => {
  await baseGenerator(packageManager, npmOptions, options, "solid", {}, "solid");
};
var SOLID_default = generator17;

// src/generators/NUXT/index.ts
var generator18 = async (packageManager, npmOptions, options) => {
  await baseGenerator(packageManager, npmOptions, options, "vue3", {}, "nuxt");
};
var NUXT_default = generator18;

// src/generators/SERVER/index.ts
var generator19 = async (packageManager, npmOptions, options) => {
  await baseGenerator(packageManager, npmOptions, options, "server", {
    extensions: ["json"]
  });
};
var SERVER_default = generator19;

// src/initiate.ts
var logger5 = console;
var installStorybook = async (projectType, packageManager, options) => {
  const npmOptions = {
    installAsDevDependencies: true,
    skipInstall: options.skipInstall
  };
  const language = await detectLanguage(packageManager);
  const pnp = await detectPnp();
  const generatorOptions = {
    language,
    builder: options.builder || await detectBuilder(packageManager, projectType),
    linkable: !!options.linkable,
    pnp: pnp || options.usePnp,
    yes: options.yes
  };
  const runGenerator = async () => {
    switch (projectType) {
      case "REACT_SCRIPTS" /* REACT_SCRIPTS */:
        return REACT_SCRIPTS_default(packageManager, npmOptions, generatorOptions).then(
          commandLog('Adding Storybook support to your "Create React App" based project')
        );
      case "REACT" /* REACT */:
        return REACT_default(packageManager, npmOptions, generatorOptions).then(
          commandLog('Adding Storybook support to your "React" app')
        );
      case "REACT_NATIVE" /* REACT_NATIVE */: {
        return REACT_NATIVE_default(packageManager, npmOptions).then(
          commandLog('Adding Storybook support to your "React Native" app')
        );
      }
      case "QWIK" /* QWIK */: {
        return QWIK_default(packageManager, npmOptions, generatorOptions).then(
          commandLog('Adding Storybook support to your "Qwik" app')
        );
      }
      case "WEBPACK_REACT" /* WEBPACK_REACT */:
        return WEBPACK_REACT_default(packageManager, npmOptions, generatorOptions).then(
          commandLog('Adding Storybook support to your "Webpack React" app')
        );
      case "REACT_PROJECT" /* REACT_PROJECT */:
        return REACT_default(packageManager, npmOptions, generatorOptions).then(
          commandLog('Adding Storybook support to your "React" library')
        );
      case "NEXTJS" /* NEXTJS */:
        return NEXTJS_default(packageManager, npmOptions, generatorOptions).then(
          commandLog('Adding Storybook support to your "Next" app')
        );
      case "SFC_VUE" /* SFC_VUE */:
        return SFC_VUE_default(packageManager, npmOptions, generatorOptions).then(
          commandLog('Adding Storybook support to your "Single File Components Vue" app')
        );
      case "VUE" /* VUE */:
        return VUE_default(packageManager, npmOptions, generatorOptions).then(
          commandLog('Adding Storybook support to your "Vue" app')
        );
      case "VUE3" /* VUE3 */:
        return VUE3_default(packageManager, npmOptions, generatorOptions).then(
          commandLog('Adding Storybook support to your "Vue 3" app')
        );
      case "ANGULAR" /* ANGULAR */:
        commandLog('Adding Storybook support to your "Angular" app');
        return ANGULAR_default(packageManager, npmOptions, generatorOptions, options);
      case "EMBER" /* EMBER */:
        return EMBER_default(packageManager, npmOptions, generatorOptions).then(
          commandLog('Adding Storybook support to your "Ember" app')
        );
      case "HTML" /* HTML */:
        return HTML_default(packageManager, npmOptions, generatorOptions).then(
          commandLog('Adding Storybook support to your "HTML" app')
        );
      case "WEB_COMPONENTS" /* WEB_COMPONENTS */:
        return WEB_COMPONENTS_default(packageManager, npmOptions, generatorOptions).then(
          commandLog('Adding Storybook support to your "web components" app')
        );
      case "PREACT" /* PREACT */:
        return PREACT_default(packageManager, npmOptions, generatorOptions).then(
          commandLog('Adding Storybook support to your "Preact" app')
        );
      case "SVELTE" /* SVELTE */:
        return SVELTE_default(packageManager, npmOptions, generatorOptions).then(
          commandLog('Adding Storybook support to your "Svelte" app')
        );
      case "SVELTEKIT" /* SVELTEKIT */:
        return SVELTEKIT_default(packageManager, npmOptions, generatorOptions).then(
          commandLog('Adding Storybook support to your "SvelteKit" app')
        );
      case "NUXT" /* NUXT */:
        return NUXT_default(packageManager, npmOptions, generatorOptions).then(
          commandLog('Adding Storybook support to your "Nuxt" app')
        );
      case "SERVER" /* SERVER */:
        return SERVER_default(packageManager, npmOptions, generatorOptions).then(
          commandLog('Adding Storybook support to your "Server" app')
        );
      case "NX" /* NX */:
        throw new Error(dedent17`
          We have detected Nx in your project. Please use "nx g @nrwl/storybook:configuration" to add Storybook to your project.
          
          For more information, please see https://nx.dev/packages/storybook
        `);
      case "SOLID" /* SOLID */:
        return SOLID_default(packageManager, npmOptions, generatorOptions).then(
          commandLog('Adding Storybook support to your "SolidJS" app')
        );
      case "UNSUPPORTED" /* UNSUPPORTED */:
        paddedLog(`We detected a project type that we don't support yet.`);
        paddedLog(
          `If you'd like your framework to be supported, please let use know about it at https://github.com/storybookjs/storybook/issues`
        );
        logger5.log();
        return Promise.resolve();
      default:
        paddedLog(`We couldn't detect your project type. (code: ${projectType})`);
        paddedLog(
          "You can specify a project type explicitly via `storybook init --type <type>`, see our docs on how to configure Storybook for your framework: https://storybook.js.org/docs/react/get-started/install"
        );
        logger5.log();
        return projectTypeInquirer(options, packageManager);
    }
  };
  try {
    return await runGenerator();
  } catch (err) {
    if (err?.message !== "Canceled by the user" && err?.stack) {
      logger5.error(`
     ${chalk12.red(err.stack)}`);
    }
    throw new HandledError(err);
  }
};
var projectTypeInquirer = async (options, packageManager) => {
  const manualAnswer = options.yes ? true : await prompts5([
    {
      type: "confirm",
      name: "manual",
      message: "Do you want to manually choose a Storybook project type to install?"
    }
  ]);
  if (manualAnswer !== true && manualAnswer.manual) {
    const { manualFramework } = await prompts5([
      {
        type: "select",
        name: "manualFramework",
        message: "Please choose a project type from the following list:",
        choices: installableProjectTypes.map((type) => ({
          title: type,
          value: type.toUpperCase()
        }))
      }
    ]);
    if (manualFramework) {
      return installStorybook(manualFramework, packageManager, options);
    }
  }
  logger5.log();
  logger5.log("For more information about installing Storybook: https://storybook.js.org/docs");
  process.exit(0);
};
async function doInitiate(options, pkg2) {
  let { packageManager: pkgMgr } = options;
  if (options.useNpm) {
    useNpmWarning();
    pkgMgr = "npm";
  }
  const packageManager = JsPackageManagerFactory.getPackageManager({ force: pkgMgr });
  const welcomeMessage = "storybook init - the simplest way to add a Storybook to your project.";
  logger5.log(chalk12.inverse(`
 ${welcomeMessage} 
`));
  const { default: updateNotifier } = await import('simple-update-notifier');
  await updateNotifier({
    pkg: pkg2,
    updateCheckInterval: 1e3 * 60 * 60
    // every hour (we could increase this later on.)
  });
  let projectType;
  const projectTypeProvided = options.type;
  const infoText = projectTypeProvided ? `Installing Storybook for user specified project type: ${projectTypeProvided}` : "Detecting project type";
  const done = commandLog(infoText);
  if (projectTypeProvided) {
    if (installableProjectTypes.includes(projectTypeProvided)) {
      projectType = projectTypeProvided.toUpperCase();
    } else {
      done(`The provided project type was not recognized by Storybook: ${projectTypeProvided}`);
      logger5.log(`
The project types currently supported by Storybook are:
`);
      installableProjectTypes.sort().forEach((framework) => paddedLog(`- ${framework}`));
      logger5.log();
      throw new HandledError(`Unknown project type supplied: ${projectTypeProvided}`);
    }
  } else {
    try {
      projectType = await detect(packageManager, options);
    } catch (err) {
      done(err.message);
      throw new HandledError(err);
    }
  }
  done();
  const storybookInstantiated = isStorybookInstantiated();
  if (options.force === false && storybookInstantiated && projectType !== "ANGULAR" /* ANGULAR */) {
    logger5.log();
    const { force } = await prompts5([
      {
        type: "confirm",
        name: "force",
        message: "We found a .storybook config directory in your project. Therefore we assume that Storybook is already instantiated for your project. Do you still want to continue and force the initialization?"
      }
    ]);
    logger5.log();
    if (force) {
      options.force = true;
    } else {
      process.exit(0);
    }
  }
  if (!options.skipInstall) {
    await packageManager.installDependencies();
  }
  const installResult = await installStorybook(projectType, packageManager, options);
  if (!options.skipInstall) {
    await packageManager.installDependencies();
  }
  if (!options.disableTelemetry) {
    await telemetry("init", { projectType });
  }
  if (projectType === "REACT_NATIVE" /* REACT_NATIVE */) {
    logger5.log();
    logger5.log(chalk12.yellow("NOTE: installation is not 100% automated.\n"));
    logger5.log(`To quickly run Storybook, replace contents of your app entry with:
`);
    codeLog(["export {default} from './.storybook';"]);
    logger5.log("\n Then to run your Storybook, type:\n");
    codeLog([packageManager.getRunCommand("start")]);
    logger5.log("\n For more in information, see the github readme:\n");
    logger5.log(chalk12.cyan("https://github.com/storybookjs/react-native"));
    logger5.log();
    return { shouldRunDev: false };
  }
  const storybookCommand = projectType === "ANGULAR" /* ANGULAR */ ? `ng run ${installResult.projectName}:storybook` : packageManager.getRunStorybookCommand();
  logger5.log(
    (0, import_boxen.default)(
      dedent17`
          Storybook was successfully installed in your project! 🎉
          To run Storybook manually, run ${chalk12.yellow(
        chalk12.bold(storybookCommand)
      )}. CTRL+C to stop.
          
          Wanna know more about Storybook? Check out ${chalk12.cyan("https://storybook.js.org/")}
          Having trouble or want to chat? Join us at ${chalk12.cyan("https://discord.gg/storybook/")}
        `,
      { borderStyle: "round", padding: 1, borderColor: "#F1618C" }
    )
  );
  return {
    shouldRunDev: process.env.CI !== "true" && process.env.IN_STORYBOOK_SANDBOX !== "true",
    projectType,
    packageManager,
    storybookCommand
  };
}
async function initiate(options, pkg2) {
  const initiateResult = await withTelemetry(
    "init",
    {
      cliOptions: options,
      printError: (err) => !err.handled && logger5.error(err)
    },
    () => doInitiate(options, pkg2)
  );
  if (initiateResult.shouldRunDev) {
    const { projectType, packageManager, storybookCommand } = initiateResult;
    logger5.log("\nRunning Storybook");
    try {
      const isReactWebProject = projectType === "REACT_SCRIPTS" /* REACT_SCRIPTS */ || projectType === "REACT" /* REACT */ || projectType === "WEBPACK_REACT" /* WEBPACK_REACT */ || projectType === "REACT_PROJECT" /* REACT_PROJECT */ || projectType === "NEXTJS" /* NEXTJS */;
      const flags = [];
      if (packageManager.type === "npm") {
        flags.push("--");
      }
      if (isReactWebProject) {
        flags.push("--initial-path=/onboarding");
      }
      flags.push("--quiet");
      packageManager.runPackageCommandSync(
        storybookCommand.replace(/^yarn /, ""),
        flags,
        void 0,
        "inherit"
      );
    } catch (e) {
    }
  }
}
var logger6 = console;
var LEGACY_CONFIGS = ["addons", "config", "presets"];
var postinstallAddon = async (addonName, isOfficialAddon) => {
  let skipMsg = null;
  if (!isOfficialAddon) {
    skipMsg = "unofficial addon";
  } else if (!fs.existsSync(".storybook")) {
    skipMsg = "no .storybook config";
  } else {
    skipMsg = "no codmods found";
    LEGACY_CONFIGS.forEach((config) => {
      try {
        const codemod = __require.resolve(
          // @ts-expect-error (it is broken)
          `${getPackageName(addonName, isOfficialAddon)}/postinstall/${config}.js`
        );
        commandLog(`Running postinstall script for ${addonName}`)();
        let configFile = path9.join(".storybook", `${config}.ts`);
        if (!fs.existsSync(configFile)) {
          configFile = path9.join(".storybook", `${config}.js`);
          if (!fs.existsSync(configFile)) {
            fs.writeFileSync(configFile, "", "utf8");
          }
        }
        sync$1("npx", ["jscodeshift", "-t", codemod, configFile], {
          stdio: "inherit",
          shell: true
        });
        skipMsg = null;
      } catch (err) {
      }
    });
  }
  if (skipMsg) {
    commandLog(`Skipping postinstall for ${addonName}, ${skipMsg}`)();
  }
};
var getVersionSpecifier = (addon) => {
  const groups = /^(...*)@(.*)$/.exec(addon);
  return groups ? [groups[1], groups[2]] : [addon, void 0];
};
async function add(addon, options) {
  let { packageManager: pkgMgr } = options;
  if (options.useNpm) {
    useNpmWarning();
    pkgMgr = "npm";
  }
  const packageManager = JsPackageManagerFactory.getPackageManager({ force: pkgMgr });
  const packageJson = await packageManager.retrievePackageJson();
  const [addonName, versionSpecifier] = getVersionSpecifier(addon);
  const { mainConfig } = getStorybookInfo(packageJson);
  if (!mainConfig) {
    logger6.error("Unable to find storybook main.js config");
    return;
  }
  const main = await readConfig(mainConfig);
  logger6.log(`Verifying ${addonName}`);
  const latestVersion = await packageManager.latestVersion(addonName);
  if (!latestVersion) {
    logger6.error(`Unknown addon ${addonName}`);
  }
  const isStorybookAddon = addonName.startsWith("@storybook/");
  const storybookVersion = await getStorybookVersion(packageManager);
  const version = versionSpecifier || (isStorybookAddon ? storybookVersion : latestVersion);
  const addonWithVersion = `${addonName}@^${version}`;
  logger6.log(`Installing ${addonWithVersion}`);
  await packageManager.addDependencies({ installAsDevDependencies: true }, [addonWithVersion]);
  logger6.log(`Adding '${addon}' to main.js addons field.`);
  main.appendValueToArray(["addons"], addonName);
  await writeConfig(main);
  if (!options.skipPostinstall) {
    await postinstallAddon(addon, isStorybookAddon);
  }
}

// src/automigrate/index.ts
var import_boxen3 = __toESM(require_boxen());
var logger7 = console;
var getFrameworkPackageName = (mainConfig) => {
  const packageNameOrPath = typeof mainConfig?.framework === "string" ? mainConfig.framework : mainConfig?.framework?.name;
  if (!packageNameOrPath) {
    return null;
  }
  const normalizedPath = path9.normalize(packageNameOrPath).replace(new RegExp(/\\/, "g"), "/");
  return Object.keys(frameworkPackages).find((pkg2) => normalizedPath.endsWith(pkg2)) || packageNameOrPath;
};
var getBuilderPackageName = (mainConfig) => {
  const packageNameOrPath = typeof mainConfig?.core?.builder === "string" ? mainConfig.core.builder : mainConfig?.core?.builder?.name;
  if (!packageNameOrPath) {
    return null;
  }
  const normalizedPath = path9.normalize(packageNameOrPath).replace(new RegExp(/\\/, "g"), "/");
  return builderPackages.find((pkg2) => normalizedPath.endsWith(pkg2)) || packageNameOrPath;
};
var getRendererPackageNameFromFramework = (frameworkPackageName) => {
  if (frameworkPackageName) {
    if (Object.keys(rendererPackages).includes(frameworkPackageName)) {
      return frameworkPackageName;
    }
    if (Object.values(rendererPackages).includes(frameworkPackageName)) {
      return Object.keys(rendererPackages).find(
        (k) => rendererPackages[k] === frameworkPackageName
      );
    }
  }
  return null;
};
var getStorybookData = async ({
  packageManager,
  configDir: userDefinedConfigDir
}) => {
  const packageJson = await packageManager.retrievePackageJson();
  const {
    mainConfig: mainConfigPath,
    version: storybookVersionSpecifier,
    configDir: configDirFromScript,
    previewConfig: previewConfigPath
  } = getStorybookInfo(packageJson, userDefinedConfigDir);
  const storybookVersion = await getStorybookVersion(packageManager);
  const configDir = userDefinedConfigDir || configDirFromScript || ".storybook";
  let mainConfig;
  try {
    mainConfig = await loadMainConfig({ configDir, noCache: true });
  } catch (err) {
    throw new Error(
      dedent17`Unable to find or evaluate ${chalk12.blue(mainConfigPath)}: ${err.message}`
    );
  }
  return {
    configDir,
    mainConfig,
    storybookVersionSpecifier,
    storybookVersion,
    mainConfigPath,
    previewConfigPath
  };
};
var updateMainConfig = async ({ mainConfigPath, dryRun }, callback) => {
  try {
    const main = await readConfig(mainConfigPath);
    await callback(main);
    if (!dryRun) {
      await writeConfig(main);
    }
  } catch (e) {
    logger7.info(
      `\u274C The migration failed to update your ${chalk12.blue(
        mainConfigPath
      )} on your behalf because of the following error:
        ${e}
`
    );
    logger7.info(
      `\u26A0\uFE0F Storybook automigrations are based on AST parsing and it's possible that your ${chalk12.blue(
        mainConfigPath
      )} file contains a non-standard format (e.g. your export is not an object) or that there was an error when parsing dynamic values (e.g. "require" calls, or usage of environment variables). When your main config is non-standard, automigrations are unfortunately not possible. Please follow the instructions given previously and follow the documentation to make the updates manually.`
    );
  }
};
var getAddonNames = (mainConfig) => {
  const addons = mainConfig.addons || [];
  const addonList = addons.map((addon) => {
    let name = "";
    if (typeof addon === "string") {
      name = addon;
    } else if (typeof addon === "object") {
      name = addon.name;
    }
    if (name.startsWith(".")) {
      return void 0;
    }
    return name.replace(/\/dist\/.*/, "").replace(/\.[mc]?[tj]?s[x]?$/, "").replace(/\/register$/, "").replace(/\/manager$/, "").replace(/\/preset$/, "");
  });
  return addonList.filter(Boolean);
};

// src/automigrate/helpers/checkWebpack5Builder.ts
var logger8 = console;
var checkWebpack5Builder = async ({
  mainConfig,
  storybookVersion
}) => {
  if (semver.lt(storybookVersion, "6.3.0")) {
    logger8.warn(
      dedent17`
        Detected SB 6.3 or below, please upgrade storybook to use webpack5.

        To upgrade to the latest stable release, run this from your project directory:

        ${chalk12.cyan("npx storybook upgrade")}

        Add the ${chalk12.cyan("--prerelease")} flag to get the latest prerelease.
      `.trim()
    );
    return null;
  }
  if (semver.gte(storybookVersion, "7.0.0")) {
    return null;
  }
  if (!mainConfig) {
    logger8.warn("Unable to find storybook main.js config");
    return null;
  }
  const builderPackageName = getBuilderPackageName(mainConfig);
  if (builderPackageName && builderPackageName !== "webpack4") {
    logger8.info(`Found builder ${builderPackageName}, skipping`);
    return null;
  }
  return { storybookVersion };
};

// src/automigrate/fixes/webpack5.ts
var logger9 = console;
var webpack5 = {
  id: "webpack5",
  async check({ configDir, packageManager, mainConfig, storybookVersion }) {
    const webpackVersion = await packageManager.getPackageVersion("webpack");
    if (!webpackVersion || semver.lt(webpackVersion, "5.0.0") || semver.gte(webpackVersion, "6.0.0"))
      return null;
    const builderInfo = await checkWebpack5Builder({ mainConfig, storybookVersion });
    return builderInfo ? { webpackVersion, ...builderInfo } : null;
  },
  prompt({ webpackVersion }) {
    const webpackFormatted = chalk12.cyan(`webpack ${webpackVersion}`);
    return dedent`
      We've detected you're running ${webpackFormatted}.
      Your Storybook's main.js files specifies webpack4, which is incompatible.
      
      To run Storybook in webpack5-mode, we can install Storybook's ${chalk12.cyan(
      "@storybook/builder-webpack5"
    )} for you.

      More info: ${chalk12.yellow(
      "https://github.com/storybookjs/storybook/blob/next/MIGRATION.md#webpack-5-manager-build"
    )}
    `;
  },
  async run({
    result: { storybookVersion, webpackVersion },
    packageManager,
    dryRun,
    mainConfigPath
  }) {
    const deps = [`@storybook/builder-webpack5@${storybookVersion}`];
    if (!webpackVersion) {
      deps.push("webpack@5");
    }
    logger9.info(`\u2705 Adding dependencies: ${deps}`);
    if (!dryRun) {
      await packageManager.addDependencies({ installAsDevDependencies: true }, deps);
    }
    logger9.info("\u2705 Setting `core.builder` to `@storybook/builder-webpack5` in main.js");
    if (!dryRun) {
      await updateMainConfig({ mainConfigPath, dryRun }, async (main) => {
        main.setFieldValue(["core", "builder"], "@storybook/builder-webpack5");
      });
    }
  }
};

// src/automigrate/fixes/cra5.ts
var cra5 = {
  id: "cra5",
  async check({ packageManager, mainConfig, storybookVersion }) {
    const craVersion = await packageManager.getPackageVersion("react-scripts");
    if (!craVersion || semver.lt(craVersion, "5.0.0")) {
      return null;
    }
    const builderInfo = await checkWebpack5Builder({ mainConfig, storybookVersion });
    return builderInfo ? { craVersion, ...builderInfo } : null;
  },
  prompt({ craVersion }) {
    const craFormatted = chalk12.cyan(`Create React App (CRA) ${craVersion}`);
    return dedent`
      We've detected you are running ${craFormatted} which is powered by webpack5.
      Your Storybook's main.js files specifies webpack4, which is incompatible.

      In order to work with your version of CRA, we need to install Storybook's ${chalk12.cyan(
      "@storybook/builder-webpack5"
    )}.

      More info: ${chalk12.yellow(
      "https://github.com/storybookjs/storybook/blob/next/MIGRATION.md#cra5-upgrade"
    )}
    `;
  },
  async run(options) {
    return webpack5.run({
      ...options,
      result: { webpackVersion: null, ...options.result }
    });
  }
};
var vue3 = {
  id: "vue3",
  async check({ packageManager, mainConfig, storybookVersion }) {
    const vueVersion = await packageManager.getPackageVersion("vue");
    if (!vueVersion || semver.lt(vueVersion, "3.0.0")) {
      return null;
    }
    const builderInfo = await checkWebpack5Builder({ mainConfig, storybookVersion });
    return builderInfo ? { vueVersion, ...builderInfo } : null;
  },
  prompt({ vueVersion, storybookVersion }) {
    const vueFormatted = chalk12.cyan(`Vue ${vueVersion}`);
    const sbFormatted = chalk12.cyan(`Storybook ${storybookVersion}`);
    return dedent`
      We've detected you are running ${vueFormatted} with Storybook.
      ${sbFormatted} runs webpack4 by default, which is incompatible.

      In order to work with your version of Vue, we need to install Storybook's ${chalk12.cyan(
      "webpack5 builder"
    )}.

      More info: ${chalk12.yellow(
      "https://github.com/storybookjs/storybook/blob/next/MIGRATION.md#vue3-upgrade"
    )}
    `;
  },
  async run(options) {
    return webpack5.run({
      ...options,
      result: { webpackVersion: null, ...options.result }
    });
  }
};

// ../../node_modules/slash/index.js
function slash(path11) {
  const isExtendedLengthPath = path11.startsWith("\\\\?\\");
  if (isExtendedLengthPath) {
    return path11;
  }
  return path11.replace(/\\/g, "/");
}
var logger10 = console;
var mdxgfm = {
  id: "github-flavored-markdown-mdx",
  async check({ configDir, mainConfig, storybookVersion }) {
    if (!semver.gte(storybookVersion, "7.0.0")) {
      return null;
    }
    const hasMDXFiles = await mainConfig?.stories?.reduce(async (acc, item) => {
      const val = await acc;
      if (val === true) {
        return true;
      }
      const pattern = typeof item === "string" ? slash(join(configDir, item)) : slash(join(configDir, item.directory, item.files));
      const files = await glob(pattern, commonGlobOptions(pattern));
      return files.some((f) => f.endsWith(".mdx"));
    }, Promise.resolve(false));
    const usesMDX1 = mainConfig?.features?.legacyMdx1 === true || false;
    const skip = usesMDX1 || !hasMDXFiles || !!mainConfig.addons?.find((item) => {
      if (item === "@storybook/addon-mdx-gfm") {
        return true;
      }
      if (typeof item === "string") {
        return false;
      }
      if (item.name === "@storybook/addon-docs") {
        return item.options?.mdxPluginOptions?.mdxCompileOptions?.remarkPlugins?.length > 0;
      }
      return false;
    });
    if (skip) {
      return null;
    }
    return {};
  },
  prompt() {
    return dedent`
      In MDX1 you had the option of using GitHub flavored markdown.

      Storybook 7.0 uses MDX2 for compiling MDX, and thus no longer supports GFM out of the box.
      Because of this you need to explicitly add the GFM plugin in the addon-docs options:
      https://storybook.js.org/docs/react/writing-docs/mdx#lack-of-github-flavored-markdown-gfm

      We recommend you follow the guide on the link above, however we can add a temporary storybook addon that helps make this migration easier.
      We'll install the addon and add it to your storybook config.
    `;
  },
  async run({ packageManager, dryRun, mainConfigPath, skipInstall }) {
    if (!dryRun) {
      const packageJson = await packageManager.retrievePackageJson();
      const versionToInstall = getStorybookVersionSpecifier(
        await packageManager.retrievePackageJson()
      );
      await packageManager.addDependencies(
        { installAsDevDependencies: true, skipInstall, packageJson },
        [`@storybook/addon-mdx-gfm@${versionToInstall}`]
      );
      await updateMainConfig({ mainConfigPath, dryRun }, async (main) => {
        logger10.info(`\u2705 Adding "@storybook/addon-mdx-gfm" addon`);
        if (!dryRun) {
          main.appendValueToArray(["addons"], "@storybook/addon-mdx-gfm");
        }
      });
    }
  }
};
var logger11 = console;
var eslintPlugin = {
  id: "eslintPlugin",
  async check({ packageManager }) {
    const { hasEslint, isStorybookPluginInstalled } = await extractEslintInfo(packageManager);
    if (isStorybookPluginInstalled || !hasEslint) {
      return null;
    }
    let eslintFile;
    let unsupportedExtension;
    try {
      eslintFile = findEslintFile();
    } catch (err) {
      unsupportedExtension = err.message;
    }
    if (!eslintFile && !unsupportedExtension) {
      logger11.warn("Unable to find .eslintrc config file, skipping");
      return null;
    }
    return { eslintFile, unsupportedExtension };
  },
  prompt() {
    return dedent`
      We've detected you are not using our eslint-plugin.

      In order to have the best experience with Storybook and follow best practices, we advise you to install eslint-plugin-storybook.

      More info: ${chalk12.yellow("https://github.com/storybookjs/eslint-plugin-storybook#readme")}
    `;
  },
  async run({ result: { eslintFile, unsupportedExtension }, packageManager, dryRun, skipInstall }) {
    const deps = [`eslint-plugin-storybook`];
    logger11.info(`\u2705 Adding dependencies: ${deps}`);
    if (!dryRun) {
      await packageManager.addDependencies({ installAsDevDependencies: true, skipInstall }, deps);
    }
    if (!dryRun && unsupportedExtension) {
      logger11.info(dedent`
          ⚠️ The plugin was successfully installed but failed to configure.
          
          Found an eslint config file with an unsupported automigration format: .eslintrc.${unsupportedExtension}.
          The supported formats for this automigration are: ${SUPPORTED_ESLINT_EXTENSIONS.join(
        ", "
      )}.

          Please refer to https://github.com/storybookjs/eslint-plugin-storybook#usage to finish setting up the plugin manually.
      `);
      return;
    }
    if (!dryRun) {
      await configureEslintPlugin(eslintFile, packageManager);
    }
  }
};
var logger12 = console;
var builderVite = {
  id: "builder-vite",
  async check({ packageManager, mainConfig }) {
    const packageJson = await packageManager.retrievePackageJson();
    const builder = mainConfig.core?.builder;
    const builderName = typeof builder === "string" ? builder : builder?.name;
    if (builderName !== "storybook-builder-vite") {
      return null;
    }
    return { builder, packageJson };
  },
  prompt({ builder }) {
    const builderFormatted = chalk12.cyan(JSON.stringify(builder, null, 2));
    return dedent`
      We've detected you're using the community vite builder: ${builderFormatted}
      
      'storybook-builder-vite' is deprecated and now located at ${chalk12.cyan(
      "@storybook/builder-vite"
    )}.

      We can upgrade your project to use the new builder automatically.
      
      More info: ${chalk12.yellow(
      "https://github.com/storybookjs/storybook/blob/next/MIGRATION.md#vite-builder-renamed"
    )}
    `;
  },
  async run({ result: { builder, packageJson }, packageManager, dryRun, mainConfigPath }) {
    const { dependencies = {}, devDependencies = {} } = packageJson;
    logger12.info(`\u2705 Removing existing 'storybook-builder-vite' dependency`);
    if (!dryRun) {
      delete dependencies["storybook-builder-vite"];
      delete devDependencies["storybook-builder-vite"];
      await packageManager.writePackageJson(packageJson);
    }
    logger12.info(`\u2705 Adding '@storybook/builder-vite' as dev dependency`);
    if (!dryRun) {
      await packageManager.addDependencies({ installAsDevDependencies: true }, [
        "@storybook/builder-vite"
      ]);
    }
    logger12.info(`\u2705 Updating main.js to use vite builder`);
    if (!dryRun) {
      await updateMainConfig({ dryRun, mainConfigPath }, async (main) => {
        const updatedBuilder = typeof builder === "string" ? "@storybook/builder-vite" : { name: "@storybook/builder-vite", options: builder.options };
        main.setFieldValue(["core", "builder"], updatedBuilder);
        await writeConfig(main);
      });
    }
  }
};
var logger13 = console;
var getStorybookScripts = (allScripts) => {
  return Object.keys(allScripts).reduce((acc, key) => {
    let isStorybookScript = false;
    const allWordsFromScript = allScripts[key].split(" ");
    const newScript = allWordsFromScript.map((currentWord, index) => {
      const previousWord = allWordsFromScript[index - 1];
      const isSbBinary = currentWord === "build-storybook" || currentWord === "start-storybook" || currentWord === "sb";
      const isPrependedByPkgManager = previousWord && ["npx", "run", "yarn", "pnpx", "pnpm dlx"].some((cmd) => previousWord.includes(cmd));
      if (isSbBinary && !isPrependedByPkgManager) {
        isStorybookScript = true;
        return currentWord.replace("sb", "storybook").replace("start-storybook", "storybook dev").replace("build-storybook", "storybook build");
      }
      return currentWord;
    }).join(" ");
    if (isStorybookScript) {
      acc[key] = {
        before: allScripts[key],
        after: newScript
      };
    }
    return acc;
  }, {});
};
var sbScripts = {
  id: "sb-scripts",
  async check({ packageManager, storybookVersion }) {
    const packageJson = await packageManager.retrievePackageJson();
    const { scripts = {} } = packageJson;
    if (semver.lt(storybookVersion, "7.0.0")) {
      return null;
    }
    const storybookScripts = getStorybookScripts(scripts);
    if (Object.keys(storybookScripts).length === 0) {
      return null;
    }
    return { packageJson, storybookScripts, storybookVersion };
  },
  prompt({ storybookVersion, storybookScripts }) {
    const sbFormatted = chalk12.cyan(`Storybook ${storybookVersion}`);
    const newScriptsMessage = Object.keys(storybookScripts).reduce((acc, scriptKey) => {
      acc.push(
        [
          chalk12.bold(scriptKey),
          "from:",
          chalk12.cyan(storybookScripts[scriptKey].before),
          "to:",
          chalk12.cyan(storybookScripts[scriptKey].after)
        ].join("\n")
      );
      return acc;
    }, []);
    return dedent`
      We've detected you are using ${sbFormatted} with scripts from previous versions of Storybook.
      Starting in Storybook 7, the ${chalk12.yellow("start-storybook")} and ${chalk12.yellow(
      "build-storybook"
    )} binaries have changed to ${chalk12.magenta("storybook dev")} and ${chalk12.magenta(
      "storybook build"
    )} respectively.
      In order to work with ${sbFormatted}, your storybook scripts have to be adjusted to use the binary. We can adjust them for you:

      ${newScriptsMessage.join("\n\n")}

      In case this migration did not cover all of your scripts, or you'd like more info: ${chalk12.yellow(
      "https://github.com/storybookjs/storybook/blob/next/MIGRATION.md#start-storybook--build-storybook-binaries-removed"
    )}
      `;
  },
  async run({ result: { storybookScripts }, packageManager, dryRun }) {
    logger13.info(`\u2705 Updating scripts in package.json`);
    logger13.log();
    if (!dryRun) {
      const newScripts = Object.keys(storybookScripts).reduce((acc, scriptKey) => {
        acc[scriptKey] = storybookScripts[scriptKey].after;
        return acc;
      }, {});
      logger13.log();
      await packageManager.addScripts(newScripts);
    }
  }
};
var logger14 = console;
var sbBinary = {
  id: "storybook-binary",
  async check({ packageManager, storybookVersion }) {
    const packageJson = await packageManager.retrievePackageJson();
    const nrwlStorybookVersion = await packageManager.getPackageVersion("@nrwl/storybook");
    const sbBinaryVersion = await packageManager.getPackageVersion("sb");
    const storybookBinaryVersion = await packageManager.getPackageVersion("storybook");
    if (nrwlStorybookVersion || semver.lt(storybookVersion, "7.0.0")) {
      return null;
    }
    const hasSbBinary = !!sbBinaryVersion;
    const hasStorybookBinary = !!storybookBinaryVersion;
    if (!hasSbBinary && hasStorybookBinary) {
      return null;
    }
    return {
      hasSbBinary,
      hasStorybookBinary,
      storybookVersion,
      packageJson
    };
  },
  prompt({ storybookVersion, hasSbBinary, hasStorybookBinary }) {
    const sbFormatted = chalk12.cyan(`Storybook ${storybookVersion}`);
    const storybookBinaryMessage = !hasStorybookBinary ? `We've detected you are using ${sbFormatted} without Storybook's ${chalk12.magenta(
      "storybook"
    )} binary. Starting in Storybook 7.0, it has to be installed.` : "";
    const extraMessage = hasSbBinary ? "You're using the 'sb' binary and it should be replaced, as 'storybook' is the recommended way to run Storybook.\n" : "";
    return dedent`
      ${storybookBinaryMessage}
      ${extraMessage}

      More info: ${chalk12.yellow(
      "https://github.com/storybookjs/storybook/blob/next/MIGRATION.md#start-storybook--build-storybook-binaries-removed"
    )}
      `;
  },
  async run({
    result: { packageJson, hasSbBinary, hasStorybookBinary },
    packageManager,
    dryRun,
    skipInstall
  }) {
    if (hasSbBinary) {
      logger14.info(`\u2705 Removing 'sb' dependency`);
      if (!dryRun) {
        await packageManager.removeDependencies(
          { skipInstall: skipInstall || !hasStorybookBinary, packageJson },
          ["sb"]
        );
      }
    }
    if (!hasStorybookBinary) {
      logger14.log();
      logger14.info(`\u2705 Adding 'storybook' as dev dependency`);
      logger14.log();
      if (!dryRun) {
        const versionToInstall = getStorybookVersionSpecifier(packageJson);
        await packageManager.addDependencies(
          { installAsDevDependencies: true, packageJson, skipInstall },
          [`storybook@${versionToInstall}`]
        );
      }
    }
  }
};
var logger15 = console;
var packagesMap = {
  "@storybook/react": {
    webpack5: "@storybook/react-webpack5",
    vite: "@storybook/react-vite"
  },
  "@storybook/preact": {
    webpack5: "@storybook/preact-webpack5",
    vite: "@storybook/preact-vite"
  },
  "@storybook/server": {
    webpack5: "@storybook/server-webpack5"
  },
  "@storybook/ember": {
    webpack5: "@storybook/ember"
  },
  "@storybook/angular": {
    webpack5: "@storybook/angular"
  },
  "@storybook/vue": {
    webpack5: "@storybook/vue-webpack5",
    vite: "@storybook/vue-vite"
  },
  "@storybook/vue3": {
    webpack5: "@storybook/vue3-webpack5",
    vite: "@storybook/vue3-vite"
  },
  "@storybook/svelte": {
    webpack5: "@storybook/svelte-webpack5",
    vite: "@storybook/svelte-vite"
  },
  "@storybook/web-components": {
    webpack5: "@storybook/web-components-webpack5",
    vite: "@storybook/web-components-vite"
  },
  "@storybook/html": {
    webpack5: "@storybook/html-webpack5",
    vite: "@storybook/html-vite"
  }
};
var communityFrameworks = {
  vite: ["storybook-framework-qwik", "storybook-solidjs-vite", "@storybook-vue/nuxt"],
  webpack5: []
};
var viteConfigFiles2 = ["vite.config.js", "vite.config.cjs", "vite.config.mjs", "vite.config.ts"];
var webpackConfigFiles2 = [
  "webpack.config.js",
  "webpack.config.cjs",
  "webpack.config.mjs",
  "webpack.config.ts"
];
var detectBuilderInfo = async ({
  mainConfig,
  configDir,
  packageManager
}) => {
  let builderName;
  let builderOrFrameworkName;
  const { core = {}, framework } = mainConfig;
  const { builder } = core;
  const builderPackageName = getBuilderPackageName(mainConfig);
  const frameworkPackageName = getFrameworkPackageName(mainConfig);
  let builderOptions = typeof builder !== "string" ? builder?.options ?? {} : {};
  if (builderPackageName) {
    builderOrFrameworkName = builderPackageName;
  } else if (framework) {
    if (Object.keys(frameworkPackages).includes(frameworkPackageName)) {
      builderOrFrameworkName = frameworkPackageName;
      builderOptions = typeof framework === "object" ? framework.options?.builder ?? {} : {};
    }
  }
  if (!builderOrFrameworkName) {
    const viteConfigFile = await findUp(viteConfigFiles2, { cwd: configDir });
    if (viteConfigFile) {
      logger15.info(
        `No builder or framework field, detected Storybook builder via: ${viteConfigFile}`
      );
      builderOrFrameworkName = "vite";
    } else {
      const webpackConfigFile = await findUp(webpackConfigFiles2, { cwd: configDir });
      if (webpackConfigFile) {
        logger15.info(
          `No builder or framework field, detected Storybook builder via: ${webpackConfigFile}`
        );
        builderOrFrameworkName = "webpack5";
      }
    }
  }
  if (!builderOrFrameworkName) {
    const storybookBuilderViteVersion = await packageManager.getPackageVersion(
      "@storybook/builder-vite"
    );
    const storybookBuilderVite2Version = await packageManager.getPackageVersion(
      "storybook-builder-vite"
    );
    const storybookBuilderWebpack5Version = await packageManager.getPackageVersion(
      "@storybook/builder-webpack5"
    );
    const storybookBuilderManagerWebpack5Version = await packageManager.getPackageVersion(
      "@storybook/manager-webpack5"
    );
    if (storybookBuilderViteVersion || storybookBuilderVite2Version) {
      builderOrFrameworkName = "vite";
    } else if (storybookBuilderWebpack5Version || storybookBuilderManagerWebpack5Version) {
      builderOrFrameworkName = "webpack5";
    }
  }
  if (builderOrFrameworkName?.includes("vite") || communityFrameworks.vite.includes(builderOrFrameworkName)) {
    builderName = "vite";
  } else if (builderOrFrameworkName?.includes("webpack") || communityFrameworks.webpack5.includes(builderOrFrameworkName)) {
    builderName = "webpack5";
  } else {
    builderName = "webpack5";
  }
  return {
    name: builderName,
    options: builderOptions
  };
};
var getNextjsAddonOptions = (addons) => {
  const nextjsAddon = addons?.find(
    (addon) => typeof addon === "string" ? addon === "storybook-addon-next" : addon.name === "storybook-addon-next"
  );
  if (!nextjsAddon || typeof nextjsAddon === "string") {
    return {};
  }
  return nextjsAddon.options || {};
};
var detectRenderer = async (packageJson) => {
  const allDependencies = {
    ...packageJson.dependencies,
    ...packageJson.devDependencies,
    ...packageJson.peerDependencies
  };
  const matches = Object.keys(rendererPackages).filter((pkg2) => !!allDependencies[pkg2]);
  let [rendererPackage] = matches;
  if (matches.length > 1) {
    const response = await prompts5([
      {
        type: "select",
        name: "rendererPackage",
        hint: "> - Use arrow-keys. Return to submit. To automate this next time, you can pass the renderer to the CLI via the --renderer flag",
        message: "Tried to detect a Storybook renderer in your project but found multiple. This could happen in monorepos, when projects contain multiple Storybook packages in package.json. Please select the correct one:",
        choices: matches.map((type) => ({
          title: type,
          value: type
        }))
      }
    ]);
    rendererPackage = response.rendererPackage;
  }
  return rendererPackage;
};

// src/automigrate/fixes/new-frameworks.ts
var logger16 = console;
var newFrameworks = {
  id: "new-frameworks",
  async check({
    configDir,
    packageManager,
    storybookVersion,
    mainConfig,
    mainConfigPath,
    rendererPackage
  }) {
    if (!semver.gte(storybookVersion, "7.0.0")) {
      return null;
    }
    const packageJson = await packageManager.retrievePackageJson();
    const frameworkPackageName = getFrameworkPackageName(mainConfig);
    const rendererPackageName = rendererPackage ?? await getRendererPackageNameFromFramework(frameworkPackageName) ?? await detectRenderer(packageJson);
    let hasFrameworkInMainConfig = !!frameworkPackageName;
    if (frameworkPackageName && !!Object.values(rendererPackages).includes(frameworkPackageName)) {
      hasFrameworkInMainConfig = false;
    }
    const builderConfig = mainConfig.core?.builder;
    const supportedPackages = Object.keys(packagesMap);
    if (!supportedPackages.includes(rendererPackageName)) {
      return null;
    }
    const allDependencies = await packageManager.getAllDependencies();
    const builderInfo = await detectBuilderInfo({
      mainConfig,
      configDir,
      packageManager
    });
    let newFrameworkPackage = Object.keys(frameworkPackages).find(
      (pkg2) => pkg2 === frameworkPackageName
    );
    if (!newFrameworkPackage) {
      newFrameworkPackage = packagesMap[rendererPackageName]?.[builderInfo.name];
    }
    if (!newFrameworkPackage) {
      return null;
    }
    const renderer = rendererPackages[rendererPackageName];
    let rendererOptions = mainConfig[`${renderer}Options`] || {};
    const frameworkOptions = typeof mainConfig.framework === "string" ? {} : mainConfig.framework?.options;
    let dependenciesToRemove = [
      "@storybook/builder-webpack5",
      "@storybook/manager-webpack5",
      "@storybook/builder-webpack4",
      "@storybook/manager-webpack4",
      "@storybook/builder-vite",
      "storybook-builder-vite"
    ];
    let addonsToRemove = [];
    let addonOptions = {};
    let metaFramework;
    const nextVersion = await packageManager.getPackageVersion("next");
    const svelteKitVersion = await packageManager.getPackageVersion("@sveltejs/kit");
    const viteVersion = await packageManager.getPackageVersion("vite");
    if (rendererPackageName === "@storybook/react" && nextVersion) {
      const nextAddonOptions = getNextjsAddonOptions(mainConfig.addons);
      const isNextJsCandidate = nextVersion && semver.gte(nextVersion, "12.0.0") || Object.keys(nextAddonOptions).length > 0;
      if (isNextJsCandidate) {
        metaFramework = "nextjs";
        if (newFrameworkPackage === "@storybook/react-webpack5" || // when framework is already set up but maybe legacy addons are still installed
        newFrameworkPackage === "@storybook/nextjs") {
          newFrameworkPackage = "@storybook/nextjs";
          addonsToRemove = ["storybook-addon-next", "storybook-addon-next-router"].filter(
            (dep) => allDependencies[dep] || mainConfig.addons?.find(
              (addon) => typeof addon === "string" ? dep === addon : dep === addon.name
            )
          );
          addonOptions = nextAddonOptions;
          dependenciesToRemove.push(
            // in case users are coming from a properly set up @storybook/webpack5 project
            "@storybook/react-webpack5",
            "storybook-addon-next",
            "storybook-addon-next-router"
          );
        }
      }
    } else if (rendererPackageName === "@storybook/svelte" && svelteKitVersion && semver.gte(svelteKitVersion, "1.0.0")) {
      metaFramework = "sveltekit";
      if (newFrameworkPackage === "@storybook/svelte-vite") {
        newFrameworkPackage = "@storybook/sveltekit";
        rendererOptions = {};
        dependenciesToRemove.push(
          // in case users are coming from a properly set up @storybook/vite project
          "@storybook/svelte-vite"
        );
      }
    }
    const dependenciesToAdd = [newFrameworkPackage].filter((dep) => !allDependencies[dep]).filter(Boolean);
    dependenciesToRemove = dependenciesToRemove.filter((dep) => allDependencies[dep]).filter(Boolean);
    const isProjectAlreadyCorrect = hasFrameworkInMainConfig && !builderConfig && !Object.keys(rendererOptions).length && !Object.keys(addonOptions).length && !dependenciesToRemove.length && !dependenciesToAdd.length;
    if (isProjectAlreadyCorrect) {
      return null;
    }
    if (viteVersion && semver.lt(viteVersion, "3.0.0")) {
      throw new Error(dedent17`
        ❌ Your project should be upgraded to use the framework package ${chalk12.bold(
        newFrameworkPackage
      )}, but we detected that you are using Vite ${chalk12.bold(
        viteVersion
      )}, which is unsupported in ${chalk12.bold(
        "Storybook 7.0"
      )}. Please upgrade Vite to ${chalk12.bold("3.0.0 or higher")} and rerun this migration.
      `);
    }
    return {
      mainConfigPath,
      dependenciesToAdd,
      dependenciesToRemove,
      frameworkPackage: newFrameworkPackage,
      hasFrameworkInMainConfig,
      frameworkOptions: {
        ...frameworkOptions,
        ...rendererOptions,
        ...addonOptions
      },
      rendererOptions,
      addonOptions,
      addonsToRemove,
      builderInfo,
      renderer,
      builderConfig,
      metaFramework
    };
  },
  prompt({
    dependenciesToRemove,
    dependenciesToAdd,
    hasFrameworkInMainConfig,
    mainConfigPath,
    frameworkPackage,
    addonOptions,
    renderer,
    rendererOptions,
    builderConfig,
    addonsToRemove,
    metaFramework
  }) {
    let disclaimer = "";
    let migrationSteps = "";
    if (dependenciesToRemove.length > 0) {
      migrationSteps += `- Remove the following dependencies:
      ${dependenciesToRemove.map((dep) => `- * ${chalk12.cyan(dep)}`).join("\n")}
`;
    }
    if (dependenciesToAdd.length > 0) {
      migrationSteps += `- Add the following dependencies: 
      ${dependenciesToAdd.map((dep) => `- * ${chalk12.cyan(dep)}`).join("\n")}
`;
    }
    if (!hasFrameworkInMainConfig) {
      migrationSteps += `- Update or specify the ${chalk12.yellow("framework")} field in ${chalk12.blue(
        mainConfigPath
      )} with the value of "${chalk12.cyan(frameworkPackage)}".
`;
    }
    if (Object.keys(rendererOptions).length > 0) {
      migrationSteps += `- Move the ${chalk12.yellow(`${renderer}Options`)} field in ${chalk12.blue(
        mainConfigPath
      )} to ${chalk12.yellow("framework.options")}, and remove that field entirely.
      More info: ${chalk12.yellow(
        "https://github.com/storybookjs/storybook/blob/next/MIGRATION.md#frameworkoptions-renamed"
      )}
`;
    }
    if (addonsToRemove.length > 0) {
      migrationSteps += `- Remove the following addons from your ${chalk12.blue(
        mainConfigPath
      )}, as the new framework also supports features provided by them:
      ${addonsToRemove.map((dep) => `- * ${chalk12.cyan(dep)}`).join("\n")}
      `;
    }
    if (Object.keys(addonOptions).length > 0) {
      migrationSteps += `- Move the addon options "${chalk12.yellow(
        Object.keys(addonOptions).join(", ")
      )}" in ${chalk12.blue(mainConfigPath)} to the ${chalk12.yellow("framework.options")} field.
`;
    }
    if (builderConfig) {
      if (typeof builderConfig === "object" && Object.keys(builderConfig.options || {}).length > 0) {
        migrationSteps += `- Move the ${chalk12.yellow("core.builder.options")} field in ${chalk12.blue(
          mainConfigPath
        )} to ${chalk12.yellow("framework.options.builder")}
`;
      }
      migrationSteps += `- Remove the ${chalk12.yellow("core.builder")} field in ${chalk12.blue(
        mainConfigPath
      )}.
`;
    }
    if (dependenciesToRemove.includes("@storybook/builder-webpack4") || dependenciesToRemove.includes("@storybook/manager-webpack4")) {
      disclaimer = dedent17`\n\n
      ${chalk12.underline(chalk12.bold(chalk12.cyan("Webpack 4 users")))}

      Unless you're using Storybook's Vite builder, this automigration will install a Webpack 5 based framework.
      
      Given you were using Storybook's Webpack 4 builder (default in 6.x, discontinued in 7.0), this could be a breaking change -- especially if your project has a custom webpack configuration.
      
      To learn more about migrating from Webpack4, see: ${chalk12.yellow(
        "https://github.com/storybookjs/storybook/blob/next/MIGRATION.md#webpack4-support-discontinued"
      )}`;
    }
    if (metaFramework === "nextjs") {
      if (dependenciesToRemove.includes("storybook-addon-next-router")) {
        migrationSteps += `- Migrate the usage of the ${chalk12.cyan(
          "storybook-addon-next-router"
        )} addon to use the APIs from the ${chalk12.magenta(
          "@storybook/nextjs"
        )} framework package instead. Follow the instructions below.`;
      }
      if (frameworkPackage === "@storybook/react-vite") {
        disclaimer = dedent17`\n\n
          ${chalk12.bold("Important")}: We've detected you are using Storybook in a Next.js project.
  
          This migration is set to update your project to use the ${chalk12.magenta(
          "@storybook/react-vite"
        )} framework, but Storybook provides a framework package specifically for Next.js projects: ${chalk12.magenta(
          "@storybook/nextjs"
        )}.
  
          This package provides a better, out of the box experience for Next.js users, however it is only compatible with the Webpack 5 builder, so we can't automigrate for you, as you are using the Vite builder. If you switch this project to use Webpack 5 and rerun this migration, we can update your project.
          
          If you are interested in using this package, see: ${chalk12.yellow(
          "https://github.com/storybookjs/storybook/blob/next/code/frameworks/nextjs/README.md"
        )}
        `;
      } else if (frameworkPackage === "@storybook/nextjs") {
        disclaimer = dedent17`\n\n
        The ${chalk12.magenta(
          "@storybook/nextjs"
        )} package provides great user experience for Next.js users, and we highly recommend you to read more about it at ${chalk12.yellow(
          "https://github.com/storybookjs/storybook/blob/next/code/frameworks/nextjs/README.md"
        )}
        `;
      }
    }
    if (metaFramework === "sveltekit") {
      if (frameworkPackage === "@storybook/svelte-webpack5") {
        disclaimer = dedent17`\n\n
          ${chalk12.bold("Important")}: We've detected you are using Storybook in a SvelteKit project.
  
          This migration is set to update your project to use the ${chalk12.magenta(
          "@storybook/svelte-webpack5"
        )} framework, but Storybook provides a framework package specifically for SvelteKit projects: ${chalk12.magenta(
          "@storybook/sveltekit"
        )}.
  
          This package provides a better experience for SvelteKit users, however it is only compatible with the Vite builder, so we can't automigrate for you, as you are using the Webpack builder.
          
          If you are interested in using this package, see: ${chalk12.yellow(
          "https://github.com/storybookjs/storybook/blob/next/code/frameworks/sveltekit/README.md"
        )}
        `;
      } else {
        migrationSteps += `- Remove the ${chalk12.yellow(
          `${renderer}Options`
        )} field from ${chalk12.blue(mainConfigPath)}.
        More info: ${chalk12.yellow(
          "https://github.com/storybookjs/storybook/blob/next/MIGRATION.md#vite-builder-uses-vite-config-automatically"
        )}
`;
        disclaimer = dedent17`\n\n
        The ${chalk12.magenta(
          "@storybook/sveltekit"
        )} package provides great user experience for SvelteKit users, and we highly recommend you to read more about it at ${chalk12.yellow(
          "https://github.com/storybookjs/storybook/blob/next/code/frameworks/sveltekit/README.md"
        )}
        `;
      }
    }
    return dedent17`
      We've detected your project is not fully setup with Storybook's 7 new framework format.

      Storybook 7 introduced the concept of frameworks, which abstracts configuration for renderers (e.g. React, Vue), builders (e.g. Webpack, Vite) and defaults to make integrations easier.

      Your project should be updated to use Storybook's framework: ${chalk12.magenta(
      frameworkPackage
    )}. We can attempt to do this for you automatically.

      Here are the steps this migration will do to migrate your project:
      ${migrationSteps}

      To learn more about the new framework format, see: ${chalk12.yellow(
      "https://github.com/storybookjs/storybook/blob/next/MIGRATION.md#new-framework-api"
    )}${disclaimer}
    `;
  },
  async run({
    result: {
      dependenciesToAdd,
      dependenciesToRemove,
      frameworkPackage,
      frameworkOptions,
      builderInfo,
      renderer,
      addonsToRemove
    },
    packageManager,
    dryRun,
    mainConfigPath,
    skipInstall
  }) {
    const packageJson = await packageManager.retrievePackageJson();
    if (dependenciesToRemove.length > 0) {
      logger16.info(`\u2705 Removing dependencies: ${dependenciesToRemove.join(", ")}`);
      if (!dryRun) {
        await packageManager.removeDependencies(
          { skipInstall: skipInstall || dependenciesToAdd.length > 0, packageJson },
          dependenciesToRemove
        );
      }
    }
    if (dependenciesToAdd.length > 0) {
      logger16.info(`\u2705 Installing new dependencies: ${dependenciesToAdd.join(", ")}`);
      if (!dryRun) {
        const versionToInstall = getStorybookVersionSpecifier(packageJson);
        const depsToAdd = dependenciesToAdd.map((dep) => `${dep}@${versionToInstall}`);
        await packageManager.addDependencies(
          { installAsDevDependencies: true, skipInstall, packageJson },
          depsToAdd
        );
      }
    }
    await updateMainConfig({ mainConfigPath, dryRun }, async (main) => {
      logger16.info(`\u2705 Updating main.js`);
      logger16.info(`\u2705 Updating "framework" field`);
      if (!dryRun) {
        main.setFieldValue(["framework", "name"], frameworkPackage);
      }
      if (!dryRun) {
        main.setFieldValue(["framework", "options"], frameworkOptions);
      }
      if (main.getFieldNode([`${renderer}Options`])) {
        logger16.info(`\u2705 Moving "${renderer}Options" to "framework.options"`);
        if (!dryRun) {
          main.removeField([`${renderer}Options`]);
        }
      }
      const builder = main.getFieldNode(["core", "builder"]);
      if (builder) {
        logger16.info(`\u2705 Removing "core.builder" field`);
        if (!dryRun) {
          main.removeField(["core", "builder"]);
        }
      }
      if (Object.keys(builderInfo.options).length > 0) {
        logger16.info(`\u2705 Moving "core.builder.options" into "framework.options.builder"`);
        if (!dryRun) {
          main.setFieldValue(["framework", "options", "builder"], builderInfo.options);
        }
      }
      const currentCore = main.getFieldValue(["core"]);
      if (currentCore) {
        if (Object.keys(currentCore).length === 0) {
          logger16.info(`\u2705 Removing "core" field`);
          if (!dryRun) {
            main.removeField(["core"]);
          }
        }
      }
      if (addonsToRemove.length > 0) {
        const existingAddons = main.getFieldValue(["addons"]);
        const updatedAddons = existingAddons.filter((addon) => {
          if (typeof addon === "string") {
            return !addonsToRemove.includes(addon);
          }
          if (addon.name) {
            return !addonsToRemove.includes(addon.name);
          }
          return false;
        });
        logger16.info(`\u2705 Removing unnecessary addons`);
        if (!dryRun) {
          main.setFieldValue(["addons"], updatedAddons);
        }
      }
    });
  }
};
var RemovedAPIs = /* @__PURE__ */ ((RemovedAPIs2) => {
  RemovedAPIs2["addDecorator"] = "addDecorator";
  RemovedAPIs2["addParameters"] = "addParameters";
  RemovedAPIs2["addLoader"] = "addLoader";
  RemovedAPIs2["getStorybook"] = "getStorybook";
  RemovedAPIs2["setAddon"] = "setAddon";
  RemovedAPIs2["clearDecorators"] = "clearDecorators";
  return RemovedAPIs2;
})(RemovedAPIs || {});
var removedGlobalClientAPIs = {
  id: "removedglobalclientapis",
  promptOnly: true,
  async check({ previewConfigPath }) {
    if (previewConfigPath) {
      const contents = await readFile(previewConfigPath, "utf8");
      const usedAPIs = Object.values(RemovedAPIs).reduce((acc, item) => {
        if (contents.includes(item)) {
          acc.push(item);
        }
        return acc;
      }, []);
      if (usedAPIs.length) {
        return {
          usedAPIs,
          previewPath: previewConfigPath
        };
      }
    }
    return null;
  },
  prompt({ usedAPIs, previewPath }) {
    return dedent17`
      ${chalk12.bold(
      chalk12.red("Attention")
    )}: We could not automatically make this change. You'll need to do it manually.

      The following APIs (used in "${chalk12.yellow(previewPath)}") have been removed from Storybook:
      
      ${usedAPIs.map((api) => `- ${chalk12.cyan(api)}`).join("\n")}

      Please see the migration guide for more information:
      ${chalk12.yellow(
      "https://github.com/storybookjs/storybook/blob/next/MIGRATION.md#removed-global-client-apis"
    )}
    `;
  }
};
var MDX1_STYLE_START = /<style>{`/g;
var MDX1_STYLE_END = /`}<\/style>/g;
var MDX1_COMMENT = /<!--(.+)-->/g;
var MDX1_CODEBLOCK = /(?:\n~~~(?:\n|.)*?\n~~~)|(?:\n```(?:\n|.)*?\n```)/g;
var fixMdxStyleTags = (mdx) => {
  return mdx.replace(MDX1_STYLE_START, "<style>\n  {`").replace(MDX1_STYLE_END, "  `}\n</style>");
};
var fixMdxComments = (mdx) => {
  const codeblocks = mdx.matchAll(MDX1_CODEBLOCK);
  const sections = mdx.split(MDX1_CODEBLOCK).map((v) => v.replace(MDX1_COMMENT, (original, group) => `{/*${group}*/}`));
  return sections.reduce((acc, item, i) => {
    const next = codeblocks.next();
    return next.done ? acc + item : acc + item + next.value[0];
  }, "");
};
var logger17 = console;
var mdx1to2 = {
  id: "mdx1to2",
  async check() {
    const storiesMdxFiles = await glob("./!(node_modules)**/*.(story|stories).mdx");
    return storiesMdxFiles.length ? { storiesMdxFiles } : void 0;
  },
  prompt({ storiesMdxFiles }) {
    return dedent`
      We've found ${chalk12.yellow(storiesMdxFiles.length)} '.stories.mdx' files in your project.
      
      Storybook has upgraded to MDX2 (https://mdxjs.com/blog/v2/), which contains breaking changes from MDX1.
      We can try to automatically upgrade your MDX files to MDX2 format using some common patterns.
      
      After this install completes, and before you start Storybook, we strongly recommend reading the MDX2 section
      of the 7.0 migration guide. It contains useful tools for detecting and fixing any remaining issues.
      
      ${chalk12.cyan("https://storybook.js.org/migration-guides/7.0")}
    `;
  },
  async run({ result: { storiesMdxFiles }, dryRun }) {
    await Promise.all([
      ...storiesMdxFiles.map(async (fname) => {
        const contents = await fse.readFile(fname, "utf-8");
        const updated = fixMdxComments(fixMdxStyleTags(contents));
        if (updated === contents) {
          logger17.info(`\u{1F197} Unmodified ${basename(fname)}`);
        } else {
          logger17.info(`\u2705 Modified ${basename(fname)}`);
          if (!dryRun) {
            await fse.writeFile(fname, updated);
          }
        }
      })
    ]);
  }
};
var logger18 = console;
var autodocsTrue = {
  id: "autodocsTrue",
  async check({ mainConfig }) {
    const { docs } = mainConfig;
    const docsPageToAutodocsMapping = {
      true: "tag",
      automatic: true,
      false: false
    };
    if (docs?.docsPage) {
      const oldValue = docs?.docsPage.toString();
      if (!(oldValue in docsPageToAutodocsMapping)) {
        throw new Error(`Unexpected value for docs.docsPage: ${oldValue}`);
      }
      return {
        value: docsPageToAutodocsMapping[oldValue]
      };
    }
    return docs?.autodocs === void 0 ? { value: true } : null;
  },
  prompt({ value }) {
    const autodocsFormatted = chalk12.cyan(`docs: { autodocs: ${JSON.stringify(value ?? true)} }`);
    const tagWarning = dedent`
      NOTE: if you're upgrading from an older 7.0-beta using the 'docsPage' tag,
      please update your story files to use the 'autodocs' tag instead.
    `;
    if (value) {
      return dedent`
      We've changed the configuration of autodocs (previous docsPage), so now the value:
        - docs.autodocs: true -- means automatically create docs for every CSF file
        - docs.autodocs: 'tag' -- means only create autodocs for CSF files with the 'autodocs' tag
        - docs.autodocs: false -- means never create autodocs

      Based on your prior configuration,  we can set the \`docs.autodocs\` to keep your old behaviour:

      ${autodocsFormatted}
      ${value === "tag" ? tagWarning : ""}
      More info: ${chalk12.yellow(
        "https://github.com/storybookjs/storybook/blob/next/MIGRATION.md#autodocs-changes"
      )}
    `;
    }
    return dedent`
      We've detected that your main.js configuration file has not configured autodocs. In 6.x we
      we defaulted to having a autodocs for every story, in 7.x you need to opt in per-component.
      However, we can set the \`docs.autodocs\` to true to approximate the old behaviour:

      ${autodocsFormatted}

      More info: ${chalk12.yellow(
      "https://github.com/storybookjs/storybook/blob/next/MIGRATION.md#autodocs-changes"
    )}
    `;
  },
  async run({ result: { value }, dryRun, mainConfigPath }) {
    logger18.info(`\u2705 Setting 'docs.autodocs' to true in main.js`);
    if (!dryRun) {
      await updateMainConfig({ mainConfigPath, dryRun }, async (main) => {
        main.removeField(["docs", "docsPage"]);
        main.setFieldValue(["docs", "autodocs"], value ?? true);
      });
    }
  }
};
var addReact = {
  id: "addReact",
  async check({ packageManager }) {
    const packageJson = await packageManager.retrievePackageJson();
    const installedDependencies = new Set(
      Object.keys({ ...packageJson.dependencies, ...packageJson.devDependencies })
    );
    const dependents = ["@storybook/addon-essentials", "@storybook/addon-docs"].filter(
      (pkg2) => installedDependencies.has(pkg2)
    );
    const additionalDependencies = [];
    if (dependents.length > 0) {
      if (!installedDependencies.has("react")) {
        additionalDependencies.push("react");
      }
      if (!installedDependencies.has("react-dom")) {
        additionalDependencies.push("react-dom");
      }
    }
    if (additionalDependencies.length > 0) {
      return { dependents, additionalDependencies };
    }
    return null;
  },
  prompt({ dependents, additionalDependencies }) {
    const dependentsFormatted = dependents.map((pkg2) => chalk12.cyan(pkg2)).join(" & ");
    const additionalDependenciesFormatted = additionalDependencies.map((pkg2) => `- ${chalk12.cyan(pkg2)}`).join("\n");
    return dedent`
      We've detected that you're using ${dependentsFormatted}.
      
      Starting in Storybook 7, we now require these peer dependencies to render docs:

      ${additionalDependenciesFormatted}

      We can add these for you automatically as dev dependencies.
      
      More info: ${chalk12.yellow(
      "https://github.com/storybookjs/storybook/blob/next/MIGRATION.md#react-peer-dependencies-required"
    )}
    `;
  },
  async run({ packageManager, result: { additionalDependencies }, dryRun }) {
    if (!dryRun) {
      await packageManager.addDependencies(
        { installAsDevDependencies: true },
        additionalDependencies
      );
    }
  }
};
var nodeJsRequirement = {
  id: "nodejs-requirement",
  promptOnly: true,
  async check({ storybookVersion }) {
    if (!semver.gte(storybookVersion, "7.0.0")) {
      return null;
    }
    const nodeVersion = process.version;
    if (semver.lt(nodeVersion, "16.0.0")) {
      return { nodeVersion };
    }
    return null;
  },
  prompt({ nodeVersion }) {
    return dedent17`
      ${chalk12.bold(
      chalk12.red("Attention")
    )}: We could not automatically make this change. You'll need to do it manually.

      We've detected that you're using Node ${chalk12.bold(
      nodeVersion
    )} but Storybook 7 only supports Node ${chalk12.bold(
      "v16.0.0"
    )} and higher. You will either need to upgrade your Node version or keep using an older version of Storybook.

      Please see the migration guide for more information:
      ${chalk12.yellow(
      "https://github.com/storybookjs/storybook/blob/next/MIGRATION.md#dropped-support-for-node-15-and-below"
    )}
    `;
  }
};
var logger19 = console;
var frameworksThatNeedBabelConfig = [
  "@storybook/react-webpack5",
  "@storybook/vue-webpack5",
  "@storybook/vue3-webpack5",
  "@storybook/html-webpack5",
  "@storybook/web-components-webpack5"
];
var missingBabelRc = {
  id: "missing-babelrc",
  async check({ packageManager, mainConfig, storybookVersion }) {
    const packageJson = await packageManager.retrievePackageJson();
    if (!semver.gte(storybookVersion, "7.0.0")) {
      return null;
    }
    const { addons } = mainConfig;
    const hasCraPreset = addons && addons.find(
      (addon) => typeof addon === "string" ? addon.endsWith("@storybook/preset-create-react-app") : addon.name.endsWith("@storybook/preset-create-react-app")
    );
    const frameworkPackageName = getFrameworkPackageName(mainConfig);
    if (frameworkPackageName && frameworksThatNeedBabelConfig.includes(frameworkPackageName) && !hasCraPreset) {
      const config = await loadPartialConfigAsync({
        babelrc: true,
        filename: "__fake__.js"
        // somehow needed to detect .babelrc.* files
      });
      if (!config.config && !config.babelrc && !packageJson.babel) {
        return { needsBabelRc: true };
      }
    }
    return null;
  },
  prompt() {
    return dedent17`
      We detected that your project does not have a babel configuration (.babelrc, babel.config.js, etc.).

      In version 6.x, Storybook provided its own babel settings out of the box. Now, Storybook re-uses ${chalk12.bold(
      "your project's babel configuration"
    )}, with small, incremental updates from Storybook addons.

      If your project does not have a babel configuration file, we can generate one that's equivalent to the 6.x defaults for you. Keep in mind that this can affect your project if it uses babel, and you may need to make additional changes based on your projects needs.

      We can create a ${chalk12.blue(
      ".babelrc.json"
    )} file with some basic configuration and add any necessary package devDependencies.

      ${chalk12.bold(
      "Note:"
    )} After installing the necessary presets, if it does not work in a monorepo, see the babel documentation for reference:
      ${chalk12.yellow("https://babeljs.io/docs")}

      Please see the migration guide for more information:
      ${chalk12.yellow(
      "https://github.com/storybookjs/storybook/blob/next/MIGRATION.md#babel-mode-v7-exclusively"
    )}
    `;
  },
  async run() {
    logger19.info();
    await generateStorybookBabelConfigInCWD();
  }
};
var angularBuilders = {
  id: "angular-builders",
  async check({ packageManager, mainConfig }) {
    const angularVersion = await packageManager.getPackageVersion("@angular/core");
    const framewworkPackageName = getFrameworkPackageName(mainConfig);
    if (!angularVersion || await isNxProject() || framewworkPackageName !== "@storybook/angular") {
      return null;
    }
    if (semver.lt(angularVersion, "14.0.0")) {
      throw new Error(dedent`
      ❌ Your project uses Angular < 14.0.0. Storybook 7.0 for Angular requires Angular 14.0.0 or higher. 
      Please upgrade your Angular version to at least version 14.0.0 to use Storybook 7.0 in your project.
      `);
    }
    const angularJSON = new AngularJSON();
    const { hasStorybookBuilder } = angularJSON;
    if (hasStorybookBuilder) {
      return null;
    }
    if (!angularJSON.rootProject && Object.keys(angularJSON.projects).length > 1) {
      return null;
    }
    return {
      mainConfig,
      packageManager
    };
  },
  prompt() {
    return dedent`
      We have detected that your project does not use the Storybook Angular builder yet. In Storybook 6.4 we have deprecated calling Storybook directly (npm run storybook) for Angular. In Storybook 7.0, we've removed it entirely.
      
      In order to use the Storybook Angular builder, we need to add a few entries to your angular.json file. Additionally, we will add the @compodoc/compodoc package to your devDependencies if you want and we will add a few scripts to your package.json file.

      Also feel free to remove the Compodoc script from your package.json file if you don't use it apart from Storybook anymore. Storybook uses Compodoc internally and you don't have to call in separately anymore.

      Read more about the Angular builder here: ${chalk12.yellow(
      "https://storybook.js.org/docs/angular/configure/storybook-builders"
    )}
    `;
  },
  async run({ result }) {
    const angularJSON = new AngularJSON();
    const { packageManager } = result;
    const { useCompoDoc } = await prompts5({
      type: "confirm",
      name: "useCompoDoc",
      message: "Have you set up compodoc in Storybook previously?"
    });
    const angularProjectName = await angularJSON.getProjectName();
    angularJSON.addStorybookEntries({
      angularProjectName,
      storybookFolder: ".storybook",
      useCompodoc: useCompoDoc,
      root: "."
    });
    angularJSON.write();
    await packageManager.addScripts({
      storybook: `ng run ${angularProjectName}:storybook`,
      "build-storybook": `ng run ${angularProjectName}:build-storybook`
    });
  }
};
var getIncompatibleAddons = async (mainConfig, packageManager = JsPackageManagerFactory.getPackageManager()) => {
  const incompatibleList = {
    "@storybook/addon-knobs": "6.4.0",
    "@storybook/addon-postcss": "2.0.0",
    "storybook-addon-next-router": "4.0.2",
    "storybook-addon-outline": "1.4.2",
    // (deprecated)
    "@storybook/addon-info": "5.3.21",
    "storybook-addon-designs": "6.3.1",
    "storybook-addon-next": "1.7.0",
    // (deprecated)'
    "storybook-docs-toc": "1.7.0",
    "@storybook/addon-google-analytics": "6.2.9",
    "storybook-addon-pseudo-states": "1.15.5",
    "storybook-dark-mode": "2.1.1",
    "storybook-addon-gatsby": "0.0.5",
    "@etchteam/storybook-addon-css-variables-theme": "1.4.0",
    "@storybook/addon-cssresources": "6.2.9",
    "storybook-addon-grid": "0.3.1",
    "storybook-multilevel-sort": "1.2.0",
    "storybook-addon-i18next": "1.3.0",
    "storybook-source-link": "2.0.8",
    "babel-plugin-storybook-csf-title": "2.1.0",
    "@urql/storybook-addon": "2.0.1",
    "storybook-addon-intl": "2.4.1",
    "storybook-addon-mock": "3.2.0",
    "@chakra-ui/storybook-addon": "4.0.16",
    "storybook-mobile-addon": "1.0.2",
    "@storybook/addon-queryparams": "6.2.9"
  };
  const addons = getAddonNames(mainConfig).filter((addon) => addon in incompatibleList);
  if (addons.length === 0) {
    return [];
  }
  const addonVersions = await Promise.all(
    addons.map(
      async (addon) => ({
        name: addon,
        version: await packageManager.getPackageVersion(addon)
      })
    )
  );
  const incompatibleAddons2 = [];
  addonVersions.forEach(({ name, version: installedVersion }) => {
    if (installedVersion === null)
      return;
    const addonVersion = incompatibleList[name];
    try {
      if (semver.lte(semver.coerce(installedVersion), semver.coerce(addonVersion))) {
        incompatibleAddons2.push({ name, version: installedVersion });
      }
    } catch (err) {
    }
  });
  return incompatibleAddons2;
};

// src/automigrate/fixes/incompatible-addons.ts
var incompatibleAddons = {
  id: "incompatible-addons",
  promptOnly: true,
  async check({ mainConfig, packageManager }) {
    const incompatibleAddonList = await getIncompatibleAddons(mainConfig, packageManager);
    return incompatibleAddonList.length > 0 ? { incompatibleAddonList } : null;
  },
  prompt({ incompatibleAddonList }) {
    return dedent17`
      ${chalk12.bold(
      chalk12.red("Attention")
    )}: We've detected that you're using the following addons in versions which are known to be incompatible with Storybook 7:

      ${incompatibleAddonList.map(({ name, version }) => `- ${chalk12.cyan(`${name}@${version}`)}`).join("\n")}

      Please be aware they might not work in Storybook 7. Reach out to their maintainers for updates and check the following Github issue for more information:
      ${chalk12.yellow("https://github.com/storybookjs/storybook/issues/20529")}
    `;
  }
};
var angularBuildersMultiproject = {
  id: "angular-builders-multiproject",
  promptOnly: true,
  async check({ packageManager, mainConfig }) {
    const angularVersion = await packageManager.getPackageVersion("@angular/core");
    const frameworkPackageName = getFrameworkPackageName(mainConfig);
    if (await isNxProject() || frameworkPackageName !== "@storybook/angular" || !angularVersion || semver.lt(angularVersion, "14.0.0")) {
      return null;
    }
    const angularJSON = new AngularJSON();
    const { hasStorybookBuilder } = angularJSON;
    if (hasStorybookBuilder) {
      return null;
    }
    if (angularJSON.rootProject || Object.keys(angularJSON.projects).length === 1) {
      return null;
    }
    return {};
  },
  prompt() {
    return dedent`
    In Storybook 6.4 we have deprecated calling Storybook directly (npm run storybook) for Angular. In Storybook 7.0, we've removed it entirely. Instead you have to set up the Storybook builder in your ${chalk12.yellow(
      "angular.json"
    )} and execute ${chalk12.yellow("ng run <your-project>:storybook")} to start Storybook. 
    
    ❌ Your Angular workspace uses multiple projects defined in the ${chalk12.yellow(
      "angular.json"
    )} file and we were not able to detect a root project. Therefore we are not able to automigrate to use Angular Storybook builder. Instead, please visit ${chalk12.yellow(
      "https://github.com/storybookjs/storybook/tree/next/code/frameworks/angular"
    )} to do the migration manually.
    `;
  }
};
var defaultRequireWrapperName = "getAbsolutePath";
function doesVariableOrFunctionDeclarationExist(node, name) {
  return t.isVariableDeclaration(node) && node.declarations.length === 1 && t.isVariableDeclarator(node.declarations[0]) && t.isIdentifier(node.declarations[0].id) && node.declarations[0].id?.name === name || t.isFunctionDeclaration(node) && t.isIdentifier(node.id) && node.id.name === name;
}
function getReferenceToRequireWrapper(config, value) {
  return t.callExpression(
    t.identifier(getRequireWrapperName(config) ?? defaultRequireWrapperName),
    [t.stringLiteral(value)]
  );
}
function getRequireWrapperName(config) {
  const declarationName = config.getBodyDeclarations().flatMap(
    (node) => (
      // eslint-disable-next-line no-nested-ternary
      doesVariableOrFunctionDeclarationExist(node, "wrapForPnp") ? ["wrapForPnp"] : doesVariableOrFunctionDeclarationExist(node, defaultRequireWrapperName) ? [defaultRequireWrapperName] : []
    )
  );
  if (declarationName.length) {
    return declarationName[0];
  }
  return null;
}
function isRequireWrapperNecessary(node, cb = () => {
}) {
  if (t.isStringLiteral(node)) {
    cb(node);
    return true;
  }
  if (t.isObjectExpression(node)) {
    const nameProperty = node.properties.find(
      (property) => t.isObjectProperty(property) && t.isIdentifier(property.key) && property.key.name === "name"
    );
    if (nameProperty && t.isStringLiteral(nameProperty.value)) {
      cb(nameProperty);
      return true;
    }
  }
  if (t.isArrayExpression(node) && node.elements.some((element) => isRequireWrapperNecessary(element))) {
    cb(node);
    return true;
  }
  return false;
}
function getFieldsForRequireWrapper(config) {
  const frameworkNode = config.getFieldNode(["framework"]);
  const builderNode = config.getFieldNode(["core", "builder"]);
  const rendererNode = config.getFieldNode(["core", "renderer"]);
  const addons = config.getFieldNode(["addons"]);
  const fieldsWithRequireWrapper = [
    ...frameworkNode ? [frameworkNode] : [],
    ...builderNode ? [builderNode] : [],
    ...rendererNode ? [rendererNode] : [],
    ...addons && t.isArrayExpression(addons) ? [addons] : []
  ];
  return fieldsWithRequireWrapper;
}
function getRequireWrapperAsCallExpression(isConfigTypescript) {
  const functionDeclaration2 = {
    ...t.functionDeclaration(
      t.identifier(defaultRequireWrapperName),
      [
        {
          ...t.identifier("value"),
          ...isConfigTypescript ? { typeAnnotation: t.tsTypeAnnotation(t.tSStringKeyword()) } : {}
        }
      ],
      t.blockStatement([
        t.returnStatement(
          t.callExpression(t.identifier("dirname"), [
            t.callExpression(t.memberExpression(t.identifier("require"), t.identifier("resolve")), [
              t.callExpression(t.identifier("join"), [
                t.identifier("value"),
                t.stringLiteral("package.json")
              ])
            ])
          ])
        )
      ])
    ),
    ...isConfigTypescript ? { returnType: t.tSTypeAnnotation(t.tsAnyKeyword()) } : {}
  };
  t.addComment(
    functionDeclaration2,
    "leading",
    "*\n * This function is used to resolve the absolute path of a package.\n * It is needed in projects that use Yarn PnP or are set up within a monorepo.\n"
  );
  return functionDeclaration2;
}
function wrapValueWithRequireWrapper(config, node) {
  isRequireWrapperNecessary(node, (n) => {
    if (t.isStringLiteral(n)) {
      const wrapperNode = getReferenceToRequireWrapper(config, n.value);
      Object.keys(n).forEach((k) => {
        delete n[k];
      });
      Object.keys(wrapperNode).forEach((k) => {
        n[k] = wrapperNode[k];
      });
    }
    if (t.isObjectProperty(n) && t.isStringLiteral(n.value)) {
      n.value = getReferenceToRequireWrapper(config, n.value.value);
    }
    if (t.isArrayExpression(n)) {
      n.elements.forEach((element, index, elements) => {
        if (t.isStringLiteral(element)) {
          elements[index] = getReferenceToRequireWrapper(config, element.value);
        }
      });
    }
  });
}

// src/automigrate/fixes/wrap-require.ts
var wrapRequire = {
  id: "wrap-require",
  async check({ packageManager, storybookVersion, mainConfigPath }) {
    const isStorybookInMonorepo = await packageManager.isStorybookInMonorepo();
    const isPnp = await detectPnp();
    const config = await readConfig(mainConfigPath);
    if (!isStorybookInMonorepo && !isPnp) {
      return null;
    }
    if (!getFieldsForRequireWrapper(config).some((node) => isRequireWrapperNecessary(node))) {
      return null;
    }
    const isConfigTypescript = mainConfigPath.endsWith(".ts") || mainConfigPath.endsWith(".tsx");
    return { storybookVersion, isStorybookInMonorepo, isPnp, isConfigTypescript };
  },
  prompt({ storybookVersion, isStorybookInMonorepo }) {
    const sbFormatted = chalk12.cyan(`Storybook ${storybookVersion}`);
    return dedent`We have detected that you're using ${sbFormatted} in a ${isStorybookInMonorepo ? "monorepo" : "PnP"} project. 
    For Storybook to work correctly, some fields in your main config must be updated. We can do this for you automatically.
    
    More info: https://storybook.js.org/docs/react/faq#how-do-i-fix-module-resolution-in-special-environments`;
  },
  async run({ dryRun, mainConfigPath, result }) {
    return new Promise((resolve2, reject) => {
      updateMainConfig({ dryRun, mainConfigPath }, (mainConfig) => {
        try {
          getFieldsForRequireWrapper(mainConfig).forEach((node) => {
            wrapValueWithRequireWrapper(mainConfig, node);
          });
          if (getRequireWrapperName(mainConfig) === null) {
            if (mainConfig.fileName.endsWith(".cjs") || mainConfig.fileName.endsWith(".cts") || mainConfig.fileName.endsWith(".cjsx") || mainConfig.fileName.endsWith(".ctsx")) {
              mainConfig.setRequireImport(["dirname", "join"], "path");
            } else {
              mainConfig.setImport(["dirname", "join"], "path");
            }
            mainConfig.setBodyDeclaration(
              getRequireWrapperAsCallExpression(result.isConfigTypescript)
            );
          }
          resolve2();
        } catch (e) {
          reject(e);
        }
      });
    });
  }
};

// src/automigrate/fixes/index.ts
var allFixes = [
  nodeJsRequirement,
  newFrameworks,
  cra5,
  webpack5,
  vue3,
  eslintPlugin,
  builderVite,
  sbBinary,
  sbScripts,
  incompatibleAddons,
  removedGlobalClientAPIs,
  mdx1to2,
  mdxgfm,
  autodocsTrue,
  addReact,
  missingBabelRc,
  angularBuildersMultiproject,
  angularBuilders,
  wrapRequire
];
var ansiRegex = ({ onlyFirst = false } = {}) => {
  const pattern = [
    "[\\u001B\\u009B][[\\]()#;?]*(?:(?:(?:(?:;[-a-zA-Z\\d\\/#&.:=?%@~_]+)*|[a-zA-Z\\d]+(?:;[-a-zA-Z\\d\\/#&.:=?%@~_]*)*)?\\u0007)",
    "(?:(?:\\d{1,4}(?:;\\d{0,4})*)?[\\dA-PR-TZcf-ntqry=><~]))"
  ].join("|");
  return new RegExp(pattern, onlyFirst ? void 0 : "g");
};
var cleanLog = (str) => str.replace(ansiRegex(), "").replace(/╮│/g, "\u256E\n\u2502").replace(/││/g, "\u2502\n\u2502").replace(/│╰/g, "\u2502\n\u2570").replace(/⚠️ {2}failed to check/g, `${EOL}\u26A0\uFE0F  failed to check`);

// src/automigrate/helpers/getMigrationSummary.ts
var import_boxen2 = __toESM(require_boxen());
function hasMultipleVersions(versions) {
  return versions.find((v) => {
    const major = semver.major(v);
    if (major === 0) {
      const minor = semver.minor(v);
      if (minor === 0) {
        const patch = semver.patch(v);
        return versions.some((v2) => {
          return semver.patch(v2) !== patch;
        });
      }
      return versions.some((v2) => {
        return semver.minor(v2) !== minor;
      });
    }
    return versions.some((v2) => {
      return semver.major(v2) !== major;
    });
  });
}

// src/automigrate/helpers/getMigrationSummary.ts
var messageDivider = "\n\n";
var segmentDivider = "\n\n\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\u2500\n\n";
function getGlossaryMessages(fixSummary, fixResults, logFile) {
  const messages = [];
  if (fixSummary.succeeded.length > 0) {
    messages.push(chalk12.bold("Successful migrations:"));
    messages.push(fixSummary.succeeded.map((m) => chalk12.green(m)).join(", "));
  }
  if (Object.keys(fixSummary.failed).length > 0) {
    messages.push(chalk12.bold("Failed migrations:"));
    messages.push(
      Object.entries(fixSummary.failed).map(([id, error]) => {
        return `${chalk12.redBright(id)}:
${error}`;
      }).join("\n")
    );
    messages.push(`You can find the full logs in ${chalk12.cyan(logFile)}`);
  }
  if (fixSummary.manual.length > 0) {
    messages.push(chalk12.bold("Manual migrations:"));
    messages.push(
      fixSummary.manual.map((m) => fixResults[m] === "manual_succeeded" /* MANUAL_SUCCEEDED */ ? chalk12.green(m) : chalk12.blue(m)).join(", ")
    );
  }
  if (fixSummary.skipped.length > 0) {
    messages.push(chalk12.bold("Skipped migrations:"));
    messages.push(fixSummary.skipped.map((m) => chalk12.cyan(m)).join(", "));
  }
  return messages;
}
function getMigrationSummary({
  fixResults,
  fixSummary,
  logFile,
  installationMetadata
}) {
  const messages = [];
  messages.push(getGlossaryMessages(fixSummary, fixResults, logFile).join(messageDivider));
  messages.push(dedent17`If you'd like to run the migrations again, you can do so by running '${chalk12.cyan(
    "npx storybook@next automigrate"
  )}'
    
    The automigrations try to migrate common patterns in your project, but might not contain everything needed to migrate to the latest version of Storybook.
    
    Please check the changelog and migration guide for manual migrations and more information: ${chalk12.yellow(
    "https://storybook.js.org/migration-guides/7.0"
  )}
    And reach out on Discord if you need help: ${chalk12.yellow("https://discord.gg/storybook")}
  `);
  if (installationMetadata?.duplicatedDependencies && Object.keys(installationMetadata.duplicatedDependencies).length > 0) {
    messages.push(getWarnings(installationMetadata).join(messageDivider));
  }
  const hasNoFixes = Object.values(fixResults).every((r) => r === "unnecessary" /* UNNECESSARY */);
  const hasFailures = Object.values(fixResults).some(
    (r) => r === "failed" /* FAILED */ || r === "check_failed" /* CHECK_FAILED */
  );
  const title = hasNoFixes ? "No migrations were applicable to your project" : hasFailures ? "Migration check ran with failures" : "Migration check ran successfully";
  return (0, import_boxen2.default)(messages.filter(Boolean).join(segmentDivider), {
    borderStyle: "round",
    padding: 1,
    title,
    borderColor: hasFailures ? "red" : "green"
  });
}
var allowList = [
  "@storybook/csf",
  // see this file for more info: code/lib/preview/src/globals/types.ts
  "@storybook/addons",
  "@storybook/channel-postmessage",
  // @deprecated: remove in 8.0
  "@storybook/channel-websocket",
  // @deprecated: remove in 8.0
  "@storybook/channels",
  "@storybook/client-api",
  "@storybook/client-logger",
  "@storybook/core-client",
  "@storybook/core-events",
  "@storybook/preview-web",
  "@storybook/preview-api",
  "@storybook/store",
  // see this file for more info: code/ui/manager/src/globals/types.ts
  "@storybook/components",
  "@storybook/router",
  "@storybook/theming",
  "@storybook/api",
  // @deprecated: remove in 8.0
  "@storybook/manager-api"
];
var disallowList = [
  Object.keys(rendererPackages),
  Object.keys(frameworkPackages),
  "@storybook/instrumenter"
];
function getWarnings(installationMetadata) {
  const messages = [];
  const { critical, trivial } = Object.entries(
    installationMetadata?.duplicatedDependencies
  ).reduce(
    (acc, [dep, versions]) => {
      if (allowList.includes(dep)) {
        return acc;
      }
      const hasMultipleMajorVersions = hasMultipleVersions(versions);
      if (disallowList.includes(dep) && hasMultipleMajorVersions) {
        acc.critical.push(`${chalk12.redBright(dep)}:
${versions.join(", ")}`);
      } else {
        acc.trivial.push(`${chalk12.hex("#ff9800")(dep)}:
${versions.join(", ")}`);
      }
      return acc;
    },
    { critical: [], trivial: [] }
  );
  if (critical.length > 0) {
    messages.push(
      `${chalk12.bold(
        "Critical:"
      )} The following dependencies are duplicated and WILL cause unexpected behavior:`
    );
    messages.push(critical.join(messageDivider));
  }
  if (trivial.length > 0) {
    messages.push(
      `${chalk12.bold(
        "Attention:"
      )} The following dependencies are duplicated which might cause unexpected behavior:`
    );
    messages.push(trivial.join(messageDivider));
  }
  messages.push(
    `You can find more information for a given dependency by running ${chalk12.cyan(
      `${installationMetadata.infoCommand} <package-name>`
    )}`
  );
  return messages;
}

// src/automigrate/index.ts
var logger20 = console;
var LOG_FILE_NAME = "migration-storybook.log";
var LOG_FILE_PATH = join(process.cwd(), LOG_FILE_NAME);
var TEMP_LOG_FILE_PATH = "";
var originalStdOutWrite = process.stdout.write.bind(process.stdout);
var originalStdErrWrite = process.stderr.write.bind(process.stdout);
var augmentLogsToFile = () => {
  TEMP_LOG_FILE_PATH = tempy.file({ name: LOG_FILE_NAME });
  const logStream = createWriteStream(TEMP_LOG_FILE_PATH);
  process.stdout.write = (d) => {
    originalStdOutWrite(d);
    return logStream.write(cleanLog(d));
  };
  process.stderr.write = (d) => {
    return logStream.write(cleanLog(d));
  };
};
var cleanup = () => {
  process.stdout.write = originalStdOutWrite;
  process.stderr.write = originalStdErrWrite;
};
var logAvailableMigrations = () => {
  const availableFixes = allFixes.map((f) => chalk12.yellow(f.id)).join(", ");
  logger20.info(`
The following migrations are available: ${availableFixes}`);
};
var automigrate = async ({
  fixId,
  fixes: inputFixes,
  dryRun,
  yes,
  useNpm,
  packageManager: pkgMgr,
  list,
  configDir: userSpecifiedConfigDir,
  renderer: rendererPackage,
  skipInstall,
  hideMigrationSummary = false
} = {}) => {
  if (list) {
    logAvailableMigrations();
    return null;
  }
  const selectedFixes = inputFixes || allFixes;
  const fixes = fixId ? selectedFixes.filter((f) => f.id === fixId) : selectedFixes;
  if (fixId && fixes.length === 0) {
    logger20.info(`\u{1F4ED} No migrations found for ${chalk12.magenta(fixId)}.`);
    logAvailableMigrations();
    return null;
  }
  augmentLogsToFile();
  logger20.info("\u{1F50E} checking possible migrations..");
  const { fixResults, fixSummary, preCheckFailure } = await runFixes({
    fixes,
    useNpm,
    pkgMgr,
    userSpecifiedConfigDir,
    rendererPackage,
    skipInstall,
    dryRun,
    yes
  });
  const hasFailures = Object.values(fixResults).some(
    (r) => r === "failed" /* FAILED */ || r === "check_failed" /* CHECK_FAILED */
  );
  if (hasFailures) {
    await move(TEMP_LOG_FILE_PATH, join(process.cwd(), LOG_FILE_NAME), { overwrite: true });
  } else {
    await remove(TEMP_LOG_FILE_PATH);
  }
  if (!hideMigrationSummary) {
    const packageManager = JsPackageManagerFactory.getPackageManager({ force: pkgMgr });
    const installationMetadata = await packageManager.findInstallations([
      "@storybook/*",
      "storybook"
    ]);
    logger20.info();
    logger20.info(
      getMigrationSummary({ fixResults, fixSummary, logFile: LOG_FILE_PATH, installationMetadata })
    );
    logger20.info();
  }
  cleanup();
  return { fixResults, preCheckFailure };
};
async function runFixes({
  fixes,
  dryRun,
  yes,
  useNpm,
  pkgMgr,
  userSpecifiedConfigDir,
  rendererPackage,
  skipInstall
}) {
  if (useNpm) {
    useNpmWarning();
    pkgMgr = "npm";
  }
  const packageManager = JsPackageManagerFactory.getPackageManager({ force: pkgMgr });
  const fixResults = {};
  const fixSummary = { succeeded: [], failed: {}, manual: [], skipped: [] };
  const { configDir: inferredConfigDir, mainConfig: mainConfigPath } = getStorybookInfo(
    await packageManager.retrievePackageJson(),
    userSpecifiedConfigDir
  );
  const storybookVersion = await getStorybookVersion(packageManager);
  if (!storybookVersion) {
    logger20.info(dedent17`
      [Storybook automigrate] ❌ Unable to determine storybook version so the automigrations will be skipped.
        🤔 Are you running automigrate from your project directory? Please specify your Storybook config directory with the --config-dir flag.
      `);
    return {
      fixResults,
      fixSummary,
      preCheckFailure: "undetected_sb_version" /* UNDETECTED_SB_VERSION */
    };
  }
  const configDir = userSpecifiedConfigDir || inferredConfigDir || ".storybook";
  try {
    await loadMainConfig({ configDir });
  } catch (err) {
    if (err.message.includes("No configuration files have been found")) {
      logger20.info(
        dedent17`[Storybook automigrate] Could not find or evaluate your Storybook main.js config directory at ${chalk12.blue(
          configDir
        )} so the automigrations will be skipped. You might be running this command in a monorepo or a non-standard project structure. If that is the case, please rerun this command by specifying the path to your Storybook config directory via the --config-dir option.`
      );
      return {
        fixResults,
        fixSummary,
        preCheckFailure: "mainjs_not_found" /* MAINJS_NOT_FOUND */
      };
    }
    logger20.info(
      dedent17`[Storybook automigrate] ❌ Failed trying to evaluate ${chalk12.blue(
        mainConfigPath
      )} with the following error: ${err.message}`
    );
    logger20.info("Please fix the error and try again.");
    return {
      fixResults,
      fixSummary,
      preCheckFailure: "mainjs_evaluation_error" /* MAINJS_EVALUATION */
    };
  }
  for (let i = 0; i < fixes.length; i += 1) {
    const f = fixes[i];
    let result;
    try {
      const { mainConfig, previewConfigPath } = await getStorybookData({
        configDir,
        packageManager
      });
      result = await f.check({
        packageManager,
        configDir,
        rendererPackage,
        mainConfig,
        storybookVersion,
        previewConfigPath,
        mainConfigPath
      });
    } catch (error) {
      logger20.info(`\u26A0\uFE0F  failed to check fix ${chalk12.bold(f.id)}`);
      logger20.error(`
${error.stack}`);
      fixSummary.failed[f.id] = error.message;
      fixResults[f.id] = "check_failed" /* CHECK_FAILED */;
    }
    if (result) {
      logger20.info(`
\u{1F50E} found a '${chalk12.cyan(f.id)}' migration:`);
      const message = f.prompt(result);
      logger20.info(
        (0, import_boxen3.default)(message, {
          borderStyle: "round",
          padding: 1,
          borderColor: "#F1618C",
          title: f.promptOnly ? "Manual migration detected" : "Automigration detected"
        })
      );
      let runAnswer;
      try {
        if (dryRun) {
          runAnswer = { fix: false };
        } else if (yes) {
          runAnswer = { fix: true };
          if (f.promptOnly) {
            fixResults[f.id] = "manual_succeeded" /* MANUAL_SUCCEEDED */;
            fixSummary.manual.push(f.id);
          }
        } else if (f.promptOnly) {
          fixResults[f.id] = "manual_succeeded" /* MANUAL_SUCCEEDED */;
          fixSummary.manual.push(f.id);
          logger20.info();
          const { shouldContinue } = await prompts5(
            {
              type: "toggle",
              name: "shouldContinue",
              message: "Select continue once you have made the required changes, or quit to exit the migration process",
              initial: true,
              active: "continue",
              inactive: "quit"
            },
            {
              onCancel: () => {
                throw new Error();
              }
            }
          );
          if (!shouldContinue) {
            fixResults[f.id] = "manual_skipped" /* MANUAL_SKIPPED */;
            break;
          }
        } else {
          runAnswer = await prompts5(
            {
              type: "confirm",
              name: "fix",
              message: `Do you want to run the '${chalk12.cyan(f.id)}' migration on your project?`,
              initial: true
            },
            {
              onCancel: () => {
                throw new Error();
              }
            }
          );
        }
      } catch (err) {
        break;
      }
      if (!f.promptOnly) {
        if (runAnswer.fix) {
          try {
            await f.run({
              result,
              packageManager,
              dryRun,
              mainConfigPath,
              skipInstall
            });
            logger20.info(`\u2705 ran ${chalk12.cyan(f.id)} migration`);
            fixResults[f.id] = "succeeded" /* SUCCEEDED */;
            fixSummary.succeeded.push(f.id);
          } catch (error) {
            fixResults[f.id] = "failed" /* FAILED */;
            fixSummary.failed[f.id] = error.message;
            logger20.info(`\u274C error when running ${chalk12.cyan(f.id)} migration`);
            logger20.info(error);
            logger20.info();
          }
        } else {
          fixResults[f.id] = "skipped" /* SKIPPED */;
          fixSummary.skipped.push(f.id);
        }
      }
    } else {
      fixResults[f.id] = fixResults[f.id] || "unnecessary" /* UNNECESSARY */;
    }
  }
  return { fixResults, fixSummary };
}
var logger21 = console;
var getNextGlob = (glob2) => {
  const extGlobsRegex = new RegExp(/(.*\.)(stories\.@.*)(\|mdx|mdx\|)(.*)$/i);
  if (glob2.match(extGlobsRegex)) {
    return glob2.replace(extGlobsRegex, "$1@(mdx|$2$4)");
  }
  const allStoriesExtensionsRegex = new RegExp(/(.*\.)(stories\.\*)$/i);
  if (glob2.match(allStoriesExtensionsRegex)) {
    return glob2.replace(allStoriesExtensionsRegex, "$1@(mdx|$2)");
  }
  return glob2.replaceAll(".stories.mdx", ".mdx");
};
var bareMdxStoriesGlob = {
  id: "bare-mdx-stories-glob",
  async check({ storybookVersion, mainConfig }) {
    if (!semver.gte(storybookVersion, "7.0.0")) {
      return null;
    }
    const existingStoriesEntries = mainConfig.stories;
    if (!existingStoriesEntries) {
      throw new Error(dedent17`
      ❌ Unable to determine Storybook stories globs in ${chalk12.blue(
        mainConfig
      )}, skipping ${chalk12.cyan(this.id)} fix.
      
      In Storybook 7, we have deprecated defining stories in MDX files, and consequently have changed the suffix to simply .mdx.

      We were unable to automatically migrate your 'stories' config to include any .mdx file instead of just .stories.mdx.
      We suggest you make this change manually.

      To learn more about this change, see: ${chalk12.yellow(
        "https://github.com/storybookjs/storybook/blob/next/MIGRATION.md#mdx-docs-files"
      )}
      `);
    }
    const nextStoriesEntries = existingStoriesEntries.map((entry) => {
      const isSpecifier = typeof entry !== "string";
      const glob2 = isSpecifier ? entry.files : entry;
      if (!glob2) {
        return entry;
      }
      const nextGlob = getNextGlob(glob2);
      return isSpecifier ? { ...entry, files: nextGlob } : nextGlob;
    });
    if (existingStoriesEntries.length === nextStoriesEntries.length && existingStoriesEntries.every((entry, index) => {
      const nextEntry = nextStoriesEntries[index];
      if (typeof entry === "string") {
        return entry === nextEntry;
      }
      if (typeof nextEntry === "string") {
        return false;
      }
      return entry.files === nextEntry.files;
    })) {
      return null;
    }
    return { existingStoriesEntries, nextStoriesEntries };
  },
  prompt({ existingStoriesEntries, nextStoriesEntries }) {
    const prettyExistingStoriesEntries = existingStoriesEntries.map((entry) => JSON.stringify(entry, null, 2)).join("\n");
    const prettyNextStoriesEntries = nextStoriesEntries.map((entry) => JSON.stringify(entry, null, 2)).join("\n");
    return dedent17`
    We've detected your project has one or more globs in your 'stories' config that matches .stories.mdx files:
      ${chalk12.cyan(prettyExistingStoriesEntries)}
    
    In Storybook 7, we have deprecated defining stories in MDX files, and consequently have changed the suffix to simply .mdx.

    We can automatically migrate your 'stories' config to include any .mdx file instead of just .stories.mdx.
    That would result in the following 'stories' config:
      ${chalk12.cyan(prettyNextStoriesEntries)}

    To learn more about this change, see: ${chalk12.yellow(
      "https://github.com/storybookjs/storybook/blob/next/MIGRATION.md#mdx-docs-files"
    )}
  `;
  },
  async run({ dryRun, mainConfigPath, result: { nextStoriesEntries } }) {
    logger21.info(dedent17`✅ Setting 'stories' config:
      ${JSON.stringify(nextStoriesEntries, null, 2)}`);
    if (!dryRun) {
      await updateMainConfig({ mainConfigPath, dryRun }, async (main) => {
        main.setFieldValue(["stories"], nextStoriesEntries);
      });
    }
  }
};

// src/migrate.ts
var logger22 = console;
async function migrate(migration, { glob: glob2, dryRun, list, rename, parser }) {
  if (list) {
    listCodemods().forEach((key) => logger22.log(key));
  } else if (migration) {
    if (migration === "mdx-to-csf" && !dryRun) {
      await runFixes({ fixes: [bareMdxStoriesGlob] });
      await addStorybookBlocksPackage();
    }
    await runCodemod(migration, { glob: glob2, dryRun, logger: logger22, rename, parser });
  } else {
    throw new Error("Migrate: please specify a migration name or --list");
  }
}
async function addStorybookBlocksPackage() {
  const packageManager = JsPackageManagerFactory.getPackageManager();
  const packageJson = await packageManager.retrievePackageJson();
  const versionToInstall = getStorybookVersionSpecifier(await packageManager.retrievePackageJson());
  logger22.info(`\u2705 Adding "@storybook/blocks" package`);
  await packageManager.addDependencies({ installAsDevDependencies: true, packageJson }, [
    `@storybook/blocks@${versionToInstall}`
  ]);
}
var read = async (url) => {
  const browser = await usePuppeteerBrowser();
  const page = await browser.newPage();
  await page.goto(url);
  await page.waitForFunction(`
    (window.__STORYBOOK_PREVIEW__ && window.__STORYBOOK_PREVIEW__.extract && window.__STORYBOOK_PREVIEW__.extract()) ||
    (window.__STORYBOOK_STORY_STORE__ && window.__STORYBOOK_STORY_STORE__.extract && window.__STORYBOOK_STORY_STORE__.extract())
  `);
  const data = JSON.parse(
    await page.evaluate(async () => {
      return JSON.stringify(window.__STORYBOOK_STORY_STORE__.getStoriesJsonData(), null, 2);
    })
  );
  setImmediate(() => {
    browser.close();
  });
  return data;
};
var useLocation = async (input) => {
  await stat(path9.resolve(input));
  if (input.match(/^http/)) {
    return [input, async () => {
    }];
  }
  const app = express();
  app.use(express.static(input));
  const port = await getPort();
  return new Promise((resolve2) => {
    const server = app.listen(port, () => {
      const result = `http://localhost:${port}/iframe.html`;
      logger.info(`connecting to: ${result}`);
      resolve2([result, server.close.bind(server)]);
    });
  });
};
var usePuppeteerBrowser = async () => {
  const args = ["--no-sandbox ", "--disable-setuid-sandbox"];
  try {
    return await puppeteerCore.launch({ args, executablePath: process.env.SB_CHROMIUM_PATH });
  } catch (e) {
    logger.info("installing puppeteer...");
    return new Promise((resolve2, reject) => {
      __require("child_process").exec(
        `node ${__require.resolve(path9.join("puppeteer-core", "install.js"))}`,
        (error) => error ? reject(error) : resolve2(puppeteerCore.launch({ args }))
      );
    });
  }
};
async function extract(input, targetPath) {
  if (input && targetPath) {
    const [location, exit] = await useLocation(input);
    const data = await read(location);
    await writeFile(targetPath, JSON.stringify(data, null, 2));
    await exit();
  } else {
    throw new Error(
      "Extract: please specify a path where your built-storybook is (can be a public url) and a target directory"
    );
  }
}
var versionRegex = /(@storybook\/[^@]+)@(\S+)/;
var getStorybookVersion2 = (line) => {
  if (line.startsWith("npm "))
    return null;
  const match = versionRegex.exec(line);
  if (!match || !semver.clean(match[2]))
    return null;
  return {
    package: match[1],
    version: match[2]
  };
};
var excludeList = [
  "@storybook/addon-bench",
  "@storybook/addon-console",
  "@storybook/addon-postcss",
  "@storybook/babel-plugin-require-context-hook",
  "@storybook/bench",
  "@storybook/builder-vite",
  "@storybook/csf",
  "@storybook/design-system",
  "@storybook/ember-cli-storybook",
  "@storybook/eslint-config-storybook",
  "@storybook/expect",
  "@storybook/jest",
  "@storybook/linter-config",
  "@storybook/mdx1-csf",
  "@storybook/mdx2-csf",
  "@storybook/react-docgen-typescript-plugin",
  "@storybook/storybook-deployer",
  "@storybook/test-runner",
  "@storybook/testing-library",
  "@storybook/testing-react",
  "@nrwl/storybook",
  "@nx/storybook"
];
var isCorePackage = (pkg2) => pkg2.startsWith("@storybook/") && !pkg2.startsWith("@storybook/preset-") && !excludeList.includes(pkg2);
var deprecatedPackages = [
  {
    minVersion: "6.0.0-alpha.0",
    url: "https://github.com/storybookjs/storybook/blob/next/MIGRATION.md#60-deprecations",
    deprecations: [
      "@storybook/addon-notes",
      "@storybook/addon-info",
      "@storybook/addon-contexts",
      "@storybook/addon-options",
      "@storybook/addon-centered"
    ]
  }
];
var formatPackage = (pkg2) => `${pkg2.package}@${pkg2.version}`;
var warnPackages = (pkgs) => pkgs.forEach((pkg2) => logger.warn(`- ${formatPackage(pkg2)}`));
var checkVersionConsistency = () => {
  const lines = sync$1("npm", ["ls"], { stdio: "pipe", shell: true }).output.toString().split("\n");
  const storybookPackages = lines.map(getStorybookVersion2).filter(Boolean).filter((pkg2) => isCorePackage(pkg2.package));
  if (!storybookPackages.length) {
    logger.warn("No storybook core packages found.");
    logger.warn(`'npm ls | grep storybook' can show if multiple versions are installed.`);
    return;
  }
  storybookPackages.sort((a, b) => semver.rcompare(a.version, b.version));
  const latestVersion = storybookPackages[0].version;
  const outdated = storybookPackages.filter((pkg2) => pkg2.version !== latestVersion);
  if (outdated.length > 0) {
    logger.warn(
      `Found ${outdated.length} outdated packages (relative to '${formatPackage(
        storybookPackages[0]
      )}')`
    );
    logger.warn("Please make sure your packages are updated to ensure a consistent experience.");
    warnPackages(outdated);
  }
  deprecatedPackages.forEach(({ minVersion, url, deprecations }) => {
    if (semver.gte(latestVersion, minVersion)) {
      const deprecated = storybookPackages.filter((pkg2) => deprecations.includes(pkg2.package));
      if (deprecated.length > 0) {
        logger.warn(`Found ${deprecated.length} deprecated packages since ${minVersion}`);
        logger.warn(`See ${url}`);
        warnPackages(deprecated);
      }
    }
  });
};
var EXTRA_FLAGS = {
  "react-scripts@<5": ["--reject", "/preset-create-react-app/"]
};
var addExtraFlags = (extraFlags, flags, { dependencies, devDependencies }) => {
  return Object.entries(extraFlags).reduce(
    (acc, entry) => {
      const [pattern, extra] = entry;
      const [pkg2, specifier] = getPackageDetails(pattern);
      const pkgVersion = dependencies[pkg2] || devDependencies[pkg2];
      if (pkgVersion && semver.satisfies(semver.coerce(pkgVersion), specifier)) {
        return [...acc, ...extra];
      }
      return acc;
    },
    [...flags]
  );
};
var addNxPackagesToReject = (flags) => {
  const newFlags = [...flags];
  const index = flags.indexOf("--reject");
  if (index > -1) {
    if (newFlags[index + 1].endsWith("/") && newFlags[index + 1].startsWith("/")) {
      newFlags[index + 1] = newFlags[index + 1].substring(1, newFlags[index + 1].length - 1);
      newFlags[index + 1] = `/(${newFlags[index + 1]}|@nrwl/storybook|@nx/storybook)/`;
    } else {
      newFlags[index + 1] = `${newFlags[index + 1]},@nrwl/storybook,@nx/storybook`;
    }
  } else {
    newFlags.push("--reject");
    newFlags.push("@nrwl/storybook,@nx/storybook");
  }
  return newFlags;
};
var doUpgrade = async ({
  tag,
  prerelease,
  skipCheck,
  useNpm,
  packageManager: pkgMgr,
  dryRun,
  configDir,
  yes,
  ...options
}) => {
  if (useNpm) {
    useNpmWarning();
    pkgMgr = "npm";
  }
  const packageManager = JsPackageManagerFactory.getPackageManager({ force: pkgMgr });
  const beforeVersion = await getStorybookCoreVersion();
  commandLog(`Checking for latest versions of '@storybook/*' packages`);
  if (tag && prerelease) {
    throw new Error(
      `Cannot set both --tag and --prerelease. Use --tag next to get the latest prereleae`
    );
  }
  let target = "latest";
  if (prerelease) {
    target = "@next";
  } else if (tag) {
    target = `@${tag}`;
  }
  let flags = [];
  if (!dryRun)
    flags.push("--upgrade");
  flags.push("--target");
  flags.push(target);
  flags = addExtraFlags(EXTRA_FLAGS, flags, await packageManager.retrievePackageJson());
  flags = addNxPackagesToReject(flags);
  const check = sync$1("npx", ["npm-check-updates@latest", "/storybook/", ...flags], {
    stdio: "pipe",
    shell: true
  }).output.toString();
  logger.info(check);
  const checkSb = sync$1("npx", ["npm-check-updates@latest", "sb", ...flags], {
    stdio: "pipe",
    shell: true
  }).output.toString();
  logger.info(checkSb);
  if (!dryRun) {
    commandLog(`Installing upgrades`);
    await packageManager.installDependencies();
  }
  let automigrationResults;
  if (!skipCheck) {
    checkVersionConsistency();
    automigrationResults = await automigrate({ dryRun, yes, packageManager: pkgMgr, configDir });
  }
  if (!options.disableTelemetry) {
    const afterVersion = await getStorybookCoreVersion();
    const { preCheckFailure, fixResults } = automigrationResults || {};
    const automigrationTelemetry = {
      automigrationResults: preCheckFailure ? null : fixResults,
      automigrationPreCheckFailure: preCheckFailure || null
    };
    telemetry("upgrade", {
      prerelease,
      tag,
      beforeVersion,
      afterVersion,
      ...automigrationTelemetry
    });
  }
};
async function upgrade(options) {
  await withTelemetry("upgrade", { cliOptions: options }, () => doUpgrade(options));
}

// src/sandbox.ts
var import_boxen4 = __toESM(require_boxen());

// src/sandbox-templates.ts
var baseTemplates = {
  "cra/default-js": {
    name: "Create React App (Javascript)",
    script: "npx create-react-app {{beforeDir}}",
    expected: {
      // TODO: change this to @storybook/cra once that package is created
      framework: "@storybook/react-webpack5",
      renderer: "@storybook/react",
      builder: "@storybook/builder-webpack5"
    },
    skipTasks: ["e2e-tests-dev", "bench"]
  },
  "cra/default-ts": {
    name: "Create React App (Typescript)",
    script: "npx create-react-app {{beforeDir}} --template typescript",
    // Re-enable once https://github.com/storybookjs/storybook/issues/19351 is fixed.
    skipTasks: ["smoke-test", "bench"],
    expected: {
      // TODO: change this to @storybook/cra once that package is created
      framework: "@storybook/react-webpack5",
      renderer: "@storybook/react",
      builder: "@storybook/builder-webpack5"
    }
  },
  "nextjs/12-js": {
    name: "Next.js v12 (JavaScript)",
    script: 'yarn create next-app {{beforeDir}} -e https://github.com/vercel/next.js/tree/next-12-3-2/examples/hello-world && cd {{beforeDir}} && npm pkg set "dependencies.next"="^12.2.0" && yarn && git add . && git commit --amend --no-edit && cd ..',
    expected: {
      framework: "@storybook/nextjs",
      renderer: "@storybook/react",
      builder: "@storybook/builder-webpack5"
    },
    skipTasks: ["e2e-tests-dev", "bench"]
  },
  "nextjs/default-js": {
    name: "Next.js (JavaScript)",
    script: "yarn create next-app {{beforeDir}} --javascript --eslint",
    expected: {
      framework: "@storybook/nextjs",
      renderer: "@storybook/react",
      builder: "@storybook/builder-webpack5"
    },
    skipTasks: ["e2e-tests-dev", "bench"]
  },
  "nextjs/default-ts": {
    name: "Next.js (TypeScript)",
    script: "yarn create next-app {{beforeDir}} --typescript --eslint",
    expected: {
      framework: "@storybook/nextjs",
      renderer: "@storybook/react",
      builder: "@storybook/builder-webpack5"
    },
    skipTasks: ["e2e-tests-dev", "bench"]
  },
  "react-vite/default-js": {
    name: "React Vite (JS)",
    script: "npm create vite@latest --yes {{beforeDir}} -- --template react",
    expected: {
      framework: "@storybook/react-vite",
      renderer: "@storybook/react",
      builder: "@storybook/builder-vite"
    },
    skipTasks: ["e2e-tests-dev", "bench"]
  },
  "react-vite/default-ts": {
    name: "React Vite (TS)",
    script: "npm create vite@latest --yes {{beforeDir}} -- --template react-ts",
    expected: {
      framework: "@storybook/react-vite",
      renderer: "@storybook/react",
      builder: "@storybook/builder-vite"
    },
    skipTasks: ["bench"]
  },
  "react-webpack/18-ts": {
    name: "React 18 Webpack5 (TS)",
    script: "yarn create webpack5-react {{beforeDir}}",
    expected: {
      framework: "@storybook/react-webpack5",
      renderer: "@storybook/react",
      builder: "@storybook/builder-webpack5"
    },
    skipTasks: ["e2e-tests-dev", "bench"]
  },
  "react-webpack/17-ts": {
    name: "React 17 Webpack5 (TS)",
    script: 'yarn create webpack5-react {{beforeDir}} --version-react="17" --version-react-dom="17"',
    expected: {
      framework: "@storybook/react-webpack5",
      renderer: "@storybook/react",
      builder: "@storybook/builder-webpack5"
    },
    skipTasks: ["e2e-tests-dev", "bench"]
  },
  "solid-vite/default-js": {
    name: "SolidJS Vite (JS)",
    script: "npx degit solidjs/templates/js {{beforeDir}}",
    expected: {
      framework: "storybook-solidjs-vite",
      renderer: "storybook-solidjs",
      builder: "@storybook/builder-vite"
    },
    // TODO: remove this once solid-vite framework is released
    inDevelopment: true,
    skipTasks: ["e2e-tests-dev", "bench"]
  },
  "solid-vite/default-ts": {
    name: "SolidJS Vite (TS)",
    script: "npx degit solidjs/templates/ts {{beforeDir}}",
    expected: {
      framework: "storybook-solidjs-vite",
      renderer: "storybook-solidjs",
      builder: "@storybook/builder-vite"
    },
    // TODO: remove this once solid-vite framework is released
    inDevelopment: true,
    skipTasks: ["e2e-tests-dev", "bench"]
  },
  "vue3-vite/default-js": {
    name: "Vue3 Vite (JS)",
    script: "npm create vite@latest --yes {{beforeDir}} -- --template vue",
    expected: {
      framework: "@storybook/vue3-vite",
      renderer: "@storybook/vue3",
      builder: "@storybook/builder-vite"
    },
    skipTasks: ["e2e-tests-dev", "bench"]
  },
  "vue3-vite/default-ts": {
    name: "Vue3 Vite (TS)",
    script: "npm create vite@latest --yes {{beforeDir}} -- --template vue-ts",
    expected: {
      framework: "@storybook/vue3-vite",
      renderer: "@storybook/vue3",
      builder: "@storybook/builder-vite"
    },
    skipTasks: ["e2e-tests-dev", "bench"]
  },
  "vue2-vite/2.7-js": {
    name: "Vue2 Vite (vue 2.7 JS)",
    script: "npx create-vue@2 {{beforeDir}} --default",
    expected: {
      framework: "@storybook/vue-vite",
      renderer: "@storybook/vue",
      builder: "@storybook/builder-vite"
    },
    // Remove smoke-test from the list once https://github.com/storybookjs/storybook/issues/19351 is fixed.
    skipTasks: ["smoke-test", "e2e-tests-dev", "bench"]
  },
  "nuxt-vite/default-ts": {
    name: "Nuxt Vite (TS)",
    script: "npx nuxi@latest init {{beforeDir}} --typescript --strict  --skip-install --no-playwright",
    // TODO: remove this once nuxt-vite framework is released
    inDevelopment: true,
    expected: {
      framework: "@storybook-vue/nuxt",
      renderer: "@storybook/vue3",
      builder: "@storybook/builder-vite"
    },
    skipTasks: ["e2e-tests", "e2e-tests-dev", "bench"]
  },
  "html-webpack/default": {
    name: "HTML Webpack5",
    script: "yarn create webpack5-html {{beforeDir}}",
    expected: {
      framework: "@storybook/html-webpack5",
      renderer: "@storybook/html",
      builder: "@storybook/builder-webpack5"
    },
    skipTasks: ["e2e-tests-dev", "bench"]
  },
  "html-vite/default-js": {
    name: "HTML Vite JS",
    script: 'npm create vite@latest --yes {{beforeDir}} -- --template vanilla && cd {{beforeDir}} && echo "export default {}" > vite.config.js',
    expected: {
      framework: "@storybook/html-vite",
      renderer: "@storybook/html",
      builder: "@storybook/builder-vite"
    },
    skipTasks: ["e2e-tests-dev", "bench"]
  },
  "html-vite/default-ts": {
    name: "HTML Vite TS",
    script: 'npm create vite@latest --yes {{beforeDir}} -- --template vanilla-ts && cd {{beforeDir}} && echo "export default {}" > vite.config.js',
    expected: {
      framework: "@storybook/html-vite",
      renderer: "@storybook/html",
      builder: "@storybook/builder-vite"
    },
    skipTasks: ["e2e-tests-dev", "bench"]
  },
  "svelte-vite/default-js": {
    name: "Svelte Vite (JS)",
    script: "npm create vite@latest --yes {{beforeDir}} -- --template svelte",
    expected: {
      framework: "@storybook/svelte-vite",
      renderer: "@storybook/svelte",
      builder: "@storybook/builder-vite"
    },
    skipTasks: ["e2e-tests-dev", "bench"]
  },
  "svelte-vite/default-ts": {
    name: "Svelte Vite (TS)",
    script: "npm create vite@latest --yes {{beforeDir}} -- --template svelte-ts",
    expected: {
      framework: "@storybook/svelte-vite",
      renderer: "@storybook/svelte",
      builder: "@storybook/builder-vite"
    },
    // Remove smoke-test from the list once https://github.com/storybookjs/storybook/issues/19351 is fixed.
    skipTasks: ["smoke-test", "e2e-tests-dev", "bench"]
  },
  "angular-cli/prerelease": {
    name: "Angular CLI (Prerelease)",
    script: "npx -p @angular/cli@next ng new angular-v16 --directory {{beforeDir}} --routing=true --minimal=true --style=scss --strict --skip-git --skip-install --package-manager=yarn",
    expected: {
      framework: "@storybook/angular",
      renderer: "@storybook/angular",
      builder: "@storybook/builder-webpack5"
    },
    skipTasks: ["e2e-tests-dev", "bench"]
  },
  "angular-cli/default-ts": {
    name: "Angular CLI (latest)",
    script: "npx -p @angular/cli ng new angular-latest --directory {{beforeDir}} --routing=true --minimal=true --style=scss --strict --skip-git --skip-install --package-manager=yarn",
    expected: {
      framework: "@storybook/angular",
      renderer: "@storybook/angular",
      builder: "@storybook/builder-webpack5"
    },
    skipTasks: ["e2e-tests-dev", "bench"]
  },
  "angular-cli/15-ts": {
    name: "Angular CLI (Version 15)",
    script: "npx -p @angular/cli@15 ng new angular-v15 --directory {{beforeDir}} --routing=true --minimal=true --style=scss --strict --skip-git --skip-install --package-manager=yarn",
    expected: {
      framework: "@storybook/angular",
      renderer: "@storybook/angular",
      builder: "@storybook/builder-webpack5"
    },
    skipTasks: ["e2e-tests-dev", "bench"]
  },
  "svelte-kit/skeleton-js": {
    name: "Svelte Kit (JS)",
    script: "yarn create svelte-with-args --name=svelte-kit/skeleton-js --directory={{beforeDir}} --template=skeleton --types=null --no-prettier --no-eslint --no-playwright --no-vitest",
    expected: {
      framework: "@storybook/sveltekit",
      renderer: "@storybook/svelte",
      builder: "@storybook/builder-vite"
    },
    skipTasks: ["e2e-tests-dev", "bench"]
  },
  "svelte-kit/skeleton-ts": {
    name: "Svelte Kit (TS)",
    script: "yarn create svelte-with-args --name=svelte-kit/skeleton-ts --directory={{beforeDir}} --template=skeleton --types=typescript --no-prettier --no-eslint --no-playwright --no-vitest",
    expected: {
      framework: "@storybook/sveltekit",
      renderer: "@storybook/svelte",
      builder: "@storybook/builder-vite"
    },
    skipTasks: ["e2e-tests-dev", "bench"]
  },
  "lit-vite/default-js": {
    name: "Lit Vite (JS)",
    script: 'npm create vite@latest --yes {{beforeDir}} -- --template lit && cd {{beforeDir}} && echo "export default {}" > vite.config.js',
    expected: {
      framework: "@storybook/web-components-vite",
      renderer: "@storybook/web-components",
      builder: "@storybook/builder-vite"
    },
    // Remove smoke-test from the list once https://github.com/storybookjs/storybook/issues/19351 is fixed.
    skipTasks: ["smoke-test", "e2e-tests-dev", "bench"]
  },
  "lit-vite/default-ts": {
    name: "Lit Vite (TS)",
    script: 'npm create vite@latest --yes {{beforeDir}} -- --template lit-ts && cd {{beforeDir}} && echo "export default {}" > vite.config.js',
    expected: {
      framework: "@storybook/web-components-vite",
      renderer: "@storybook/web-components",
      builder: "@storybook/builder-vite"
    },
    // Remove smoke-test from the list once https://github.com/storybookjs/storybook/issues/19351 is fixed.
    skipTasks: ["smoke-test", "e2e-tests-dev", "bench"]
  },
  "vue-cli/default-js": {
    name: "Vue-CLI (Default JS)",
    script: 'npx -p @vue/cli vue create {{beforeDir}} --default --packageManager=yarn --force --merge && cd {{beforeDir}} && echo "module.exports = {}" > webpack.config.js',
    expected: {
      framework: "@storybook/vue3-webpack5",
      renderer: "@storybook/vue3",
      builder: "@storybook/builder-webpack5"
    },
    // Remove smoke-test from the list once https://github.com/storybookjs/storybook/issues/19351 is fixed.
    skipTasks: ["smoke-test", "e2e-tests-dev", "bench"]
  },
  "vue-cli/vue2-default-js": {
    name: "Vue-CLI (Vue2 JS)",
    script: 'npx -p @vue/cli vue create {{beforeDir}} --default --packageManager=yarn --force --merge --preset="Default (Vue 2)" && cd {{beforeDir}} && echo "module.exports = {}" > webpack.config.js',
    expected: {
      framework: "@storybook/vue-webpack5",
      renderer: "@storybook/vue",
      builder: "@storybook/builder-webpack5"
    },
    // Remove smoke-test from the list once https://github.com/storybookjs/storybook/issues/19351 is fixed.
    skipTasks: ["smoke-test", "e2e-tests-dev", "bench"]
  },
  "preact-webpack5/default-js": {
    name: "Preact CLI (Default JS)",
    script: 'npx preact-cli create default {{beforeDir}} --name preact-app --yarn --no-install && cd {{beforeDir}} && echo "module.exports = {}" > webpack.config.js',
    expected: {
      framework: "@storybook/preact-webpack5",
      renderer: "@storybook/preact",
      builder: "@storybook/builder-webpack5"
    },
    skipTasks: ["e2e-tests-dev", "bench"]
  },
  "preact-webpack5/default-ts": {
    name: "Preact CLI (Default TS)",
    script: 'npx preact-cli create typescript {{beforeDir}} --name preact-app --yarn --no-install && cd {{beforeDir}} && echo "module.exports = {}" > webpack.config.js',
    expected: {
      framework: "@storybook/preact-webpack5",
      renderer: "@storybook/preact",
      builder: "@storybook/builder-webpack5"
    },
    skipTasks: ["e2e-tests-dev", "bench"]
  },
  "preact-vite/default-js": {
    name: "Preact Vite (JS)",
    script: "npm create vite@latest --yes {{beforeDir}} -- --template preact",
    expected: {
      framework: "@storybook/preact-vite",
      renderer: "@storybook/preact",
      builder: "@storybook/builder-vite"
    },
    skipTasks: ["e2e-tests-dev", "bench"]
  },
  "preact-vite/default-ts": {
    name: "Preact Vite (TS)",
    script: "npm create vite@latest --yes {{beforeDir}} -- --template preact-ts",
    expected: {
      framework: "@storybook/preact-vite",
      renderer: "@storybook/preact",
      builder: "@storybook/builder-vite"
    },
    skipTasks: ["e2e-tests-dev", "bench"]
  },
  "qwik-vite/default-ts": {
    name: "Qwik CLI (Default TS)",
    script: "yarn create qwik basic {{beforeDir}} --no-install",
    // TODO: The community template does not provide standard stories, which is required for e2e tests. Reenable once it does.
    inDevelopment: true,
    expected: {
      framework: "storybook-framework-qwik",
      renderer: "storybook-framework-qwik",
      builder: "storybook-framework-qwik"
    },
    // TODO: The community template does not provide standard stories, which is required for e2e tests.
    skipTasks: ["e2e-tests", "e2e-tests-dev", "bench"]
  }
};
var internalTemplates = {
  "internal/ssv6-vite": {
    ...baseTemplates["react-vite/default-ts"],
    name: "StoryStore v6 (react-vite/default-ts)",
    isInternal: true,
    modifications: {
      mainConfig: {
        features: {
          storyStoreV7: false,
          storyStoreV7MdxErrors: false
        }
      }
    },
    skipTasks: ["bench"]
  },
  "internal/ssv6-webpack": {
    ...baseTemplates["cra/default-ts"],
    name: "StoryStore v6 (cra/default-ts)",
    isInternal: true,
    modifications: {
      mainConfig: {
        features: {
          storyStoreV7: false,
          storyStoreV7MdxErrors: false
        }
      }
    },
    skipTasks: ["bench"]
  },
  "internal/swc-webpack": {
    ...baseTemplates["react-webpack/18-ts"],
    name: "SWC (react-webpack/18-ts)",
    isInternal: true,
    inDevelopment: true,
    modifications: {
      mainConfig: {
        framework: {
          name: "@storybook/react-webpack5",
          options: {
            builder: {
              useSWC: true
            }
          }
        }
      }
    },
    skipTasks: ["bench"]
  },
  "internal/server-webpack5": {
    name: "Server Webpack5",
    script: 'yarn init -y && echo "module.exports = {}" > webpack.config.js',
    expected: {
      framework: "@storybook/server-webpack5",
      renderer: "@storybook/server",
      builder: "@storybook/builder-webpack5"
    },
    isInternal: true,
    skipTasks: ["bench"]
  }
  // 'internal/pnp': {
  //   ...baseTemplates['cra/default-ts'],
  //   name: 'PNP (cra/default-ts)',
  //   script: 'yarn create react-app . --use-pnp',
  //   isInternal: true,
  //   inDevelopment: true,
  // },
};
var benchTemplates = {
  "bench/react-vite-default-ts": {
    ...baseTemplates["react-vite/default-ts"],
    name: "Bench (react-vite/default-ts)",
    isInternal: true,
    modifications: {
      skipTemplateStories: true
    },
    skipTasks: ["e2e-tests-dev", "test-runner", "test-runner-dev", "e2e-tests", "chromatic"]
  },
  "bench/react-webpack-18-ts": {
    ...baseTemplates["react-webpack/18-ts"],
    name: "Bench (react-webpack/18-ts)",
    isInternal: true,
    modifications: {
      skipTemplateStories: true
    },
    skipTasks: ["e2e-tests-dev", "test-runner", "test-runner-dev", "e2e-tests", "chromatic"]
  }
};
var allTemplates = {
  ...baseTemplates,
  ...internalTemplates,
  ...benchTemplates
};

// src/sandbox.ts
var logger25 = console;
var toChoices = (c) => ({ title: allTemplates[c].name, value: c });
var sandbox = async ({
  output: outputDirectory,
  filterValue,
  branch,
  init
}) => {
  let selectedConfig = allTemplates[filterValue];
  let selectedTemplate = selectedConfig ? filterValue : null;
  if (!selectedConfig) {
    const filterRegex = new RegExp(`^${filterValue || ""}`, "i");
    const keys = Object.keys(allTemplates);
    const choices = keys.reduce((acc, group) => {
      const current = allTemplates[group];
      if (!filterValue) {
        acc.push(group);
        return acc;
      }
      if (current.name.match(filterRegex) || group.match(filterRegex) || current.expected.builder.match(filterRegex) || current.expected.framework.match(filterRegex) || current.expected.renderer.match(filterRegex)) {
        acc.push(group);
        return acc;
      }
      return acc;
    }, []);
    if (choices.length === 0) {
      logger25.info(
        (0, import_boxen4.default)(
          dedent`
            🔎 You filtered out all templates. 🔍

            After filtering all the templates with "${chalk12.yellow(
            filterValue
          )}", we found no results. Please try again with a different filter.

            Available templates:
            ${keys.map((key) => chalk12.blue`- ${key}`).join("\n")}
            `.trim(),
          { borderStyle: "round", padding: 1, borderColor: "#F1618C" }
        )
      );
      process.exit(1);
    }
    if (choices.length === 1) {
      [selectedTemplate] = choices;
    } else {
      logger25.info(
        (0, import_boxen4.default)(
          dedent`
            🤗 Welcome to ${chalk12.yellow("sb sandbox")}! 🤗

            Create a ${chalk12.green("new project")} to minimally reproduce Storybook issues.

            1. select an environment that most closely matches your project setup.
            2. select a location for the reproduction, outside of your project.

            After the reproduction is ready, we'll guide you through the next steps.
            `.trim(),
          { borderStyle: "round", padding: 1, borderColor: "#F1618C" }
        )
      );
      selectedTemplate = await promptSelectedTemplate(choices);
    }
    const hasSelectedTemplate = !!(selectedTemplate ?? null);
    if (!hasSelectedTemplate) {
      logger25.error("Somehow we got no templates. Please rerun this command!");
      return;
    }
    selectedConfig = allTemplates[selectedTemplate];
    if (!selectedConfig) {
      throw new Error("\u{1F6A8} Sandbox: please specify a valid template type");
    }
  }
  let selectedDirectory = outputDirectory;
  const outputDirectoryName = outputDirectory || selectedTemplate;
  if (selectedDirectory && existsSync(`${selectedDirectory}`)) {
    logger25.info(`\u26A0\uFE0F  ${selectedDirectory} already exists! Overwriting...`);
  }
  if (!selectedDirectory) {
    const { directory } = await prompts5(
      {
        type: "text",
        message: "Enter the output directory",
        name: "directory",
        initial: outputDirectoryName,
        validate: async (directoryName) => existsSync(directoryName) ? `${directoryName} already exists. Please choose another name.` : true
      },
      {
        onCancel: () => {
          logger25.log("Command cancelled by the user. Exiting...");
          process.exit(1);
        }
      }
    );
    selectedDirectory = directory;
  }
  try {
    const templateDestination = path9.isAbsolute(selectedDirectory) ? selectedDirectory : path9.join(process.cwd(), selectedDirectory);
    logger25.info(`\u{1F3C3} Adding ${selectedConfig.name} into ${templateDestination}`);
    logger25.log("\u{1F4E6} Downloading sandbox template...");
    try {
      const templateType = init ? "after-storybook" : "before-storybook";
      const gitPath = `github:storybookjs/sandboxes/${selectedTemplate}/${templateType}#${branch}`;
      await downloadTemplate(gitPath, {
        force: true,
        dir: templateDestination
      });
      if ((await readdir(templateDestination)).length === 0) {
        throw new Error(
          dedent`Template downloaded from ${chalk12.blue(gitPath)} is empty.
          Are you use it exists? Or did you want to set ${chalk12.yellow(
            selectedTemplate
          )} to inDevelopment first?`
        );
      }
    } catch (err) {
      logger25.error(`\u{1F6A8} Failed to download sandbox template: ${err.message}`);
      throw err;
    }
    const initMessage = init ? chalk12.yellow(`yarn install
yarn storybook`) : `Recreate your setup, then ${chalk12.yellow(`npx storybook@latest init`)}`;
    logger25.info(
      (0, import_boxen4.default)(
        dedent`
        🎉 Your Storybook reproduction project is ready to use! 🎉

        ${chalk12.yellow(`cd ${selectedDirectory}`)}
        ${initMessage}

        Once you've recreated the problem you're experiencing, please:

        1. Document any additional steps in ${chalk12.cyan("README.md")}
        2. Publish the repository to github
        3. Link to the repro repository in your issue

        Having a clean repro helps us solve your issue faster! 🙏
      `.trim(),
        { borderStyle: "round", padding: 1, borderColor: "#F1618C" }
      )
    );
  } catch (error) {
    logger25.error("\u{1F6A8} Failed to create sandbox");
    throw error;
  }
};
async function promptSelectedTemplate(choices) {
  const { template } = await prompts5(
    {
      type: "select",
      message: "\u{1F308} Select the template",
      name: "template",
      choices: choices.map(toChoices)
    },
    {
      onCancel: () => {
        logger25.log("Command cancelled by the user. Exiting...");
        process.exit(1);
      }
    }
  );
  return template || null;
}
var exec = async (command2, options = {}, {
  startMessage,
  errorMessage,
  dryRun
} = {}) => {
  if (startMessage)
    logger.info(startMessage);
  if (dryRun) {
    logger.info(`
> ${command2}
`);
    return void 0;
  }
  logger.info(command2);
  return new Promise((resolve2, reject) => {
    const child = spawn(command2, {
      ...options,
      shell: true,
      stdio: "pipe"
    });
    child.stderr.pipe(process.stdout);
    child.stdout.pipe(process.stdout);
    child.on("exit", (code) => {
      if (code === 0) {
        resolve2(void 0);
      } else {
        logger.error(chalk12.red(`An error occurred while executing: \`${command2}\``));
        logger.info(errorMessage);
        reject(new Error(`command exited with code: ${code}: `));
      }
    });
  });
};
var link = async ({ target, local, start }) => {
  const storybookDir = process.cwd();
  try {
    const packageJson = await fse.readJSON("package.json");
    if (packageJson.name !== "@storybook/root") {
      throw new Error();
    }
  } catch {
    throw new Error("Expected to run link from the root of the storybook monorepo");
  }
  let reproDir = target;
  let reproName = path9.basename(target);
  if (!local) {
    const reprosDir = path9.join(storybookDir, "../storybook-repros");
    logger.info(`Ensuring directory ${reprosDir}`);
    await fse.ensureDir(reprosDir);
    logger.info(`Cloning ${target}`);
    await exec(`git clone ${target}`, { cwd: reprosDir });
    reproName = path9.basename(target, path9.extname(target));
    reproDir = path9.join(reprosDir, reproName);
  }
  const reproPackageJson = await fse.readJSON(path9.join(reproDir, "package.json"));
  const version = sync$1("yarn", ["--version"], {
    cwd: reproDir,
    stdio: "pipe",
    shell: true
  }).stdout.toString();
  if (!/^[23]\./.test(version)) {
    logger.warn(`\u{1F6A8} Expected yarn 2 or 3 in ${reproDir}!`);
    logger.warn("");
    logger.warn("Please set it up with `yarn set version berry`,");
    logger.warn(`then link '${reproDir}' with the '--local' flag.`);
    return;
  }
  logger.info(`Linking ${reproDir}`);
  await exec(`yarn link --all ${storybookDir}`, { cwd: reproDir });
  logger.info(`Installing ${reproName}`);
  await exec(`yarn install`, { cwd: reproDir });
  if (!reproPackageJson.devDependencies?.vite) {
    await exec(`yarn add -D webpack-hot-middleware`, { cwd: reproDir });
  }
  await exec(`yarn add @types/node@16`, { cwd: reproDir });
  if (start) {
    logger.info(`Running ${reproName} storybook`);
    await exec(`yarn run storybook`, { cwd: reproDir });
  }
};
function printError(error) {
  instance.heading = "";
  if (error instanceof Error) {
    if (error.error) {
      logger.error(error.error);
    } else if (error.stats && error.stats.compilation.errors) {
      error.stats.compilation.errors.forEach((e) => logger.plain(e));
    } else {
      logger.error(error);
    }
  } else if (error.compilation?.errors) {
    error.compilation.errors.forEach((e) => logger.plain(e));
  }
  logger.line();
  logger.warn(
    error.close ? dedent`
          FATAL broken build!, will close the process,
          Fix the error below and restart storybook.
        ` : dedent`
          Broken build, fix the error above.
          You may need to refresh the browser.
        `
  );
  logger.line();
}
var dev = async (cliOptions) => {
  process.env.NODE_ENV = process.env.NODE_ENV || "development";
  const options = {
    ...cliOptions,
    configDir: cliOptions.configDir || "./.storybook",
    configType: "DEVELOPMENT",
    ignorePreview: !!cliOptions.previewUrl && !cliOptions.forceBuildPreview,
    cache,
    packageJson: sync({ cwd: __dirname }).packageJson
  };
  await withTelemetry(
    "dev",
    {
      cliOptions,
      presetOptions: options,
      printError
    },
    () => buildDevStandalone(options)
  );
};
var build = async (cliOptions) => {
  const options = {
    ...cliOptions,
    configDir: cliOptions.configDir || "./.storybook",
    outputDir: cliOptions.outputDir || "./storybook-static",
    ignorePreview: !!cliOptions.previewUrl && !cliOptions.forceBuildPreview,
    configType: "PRODUCTION",
    cache: cache,
    packageJson: sync({ cwd: __dirname }).packageJson
  };
  await withTelemetry(
    "build",
    { cliOptions, presetOptions: options },
    () => buildStaticStandalone(options)
  );
};

// src/generate.ts
addToGlobalContext("cliVersion", versions_default.storybook);
var pkg = sync({ cwd: __dirname }).packageJson;
var consoleLogger = console;
var command = (name) => program.command(name).option(
  "--disable-telemetry",
  "disable sending telemetry data",
  // default value is false, but if the user sets STORYBOOK_DISABLE_TELEMETRY, it can be true
  process.env.STORYBOOK_DISABLE_TELEMETRY && process.env.STORYBOOK_DISABLE_TELEMETRY !== "false"
).option("--debug", "Get more logs in debug mode", false).option("--enable-crash-reports", "enable sending crash reports to telemetry data");
command("init").description("Initialize Storybook into your project.").option("-f --force", "Force add Storybook").option("-s --skip-install", "Skip installing deps").option("--package-manager <npm|pnpm|yarn1|yarn2>", "Force package manager for installing deps").option("-N --use-npm", "Use npm to install deps (deprecated)").option("--use-pnp", "Enable pnp mode for Yarn 2+").option("-p --parser <babel | babylon | flow | ts | tsx>", "jscodeshift parser").option("-t --type <type>", "Add Storybook for a specific project type").option("-y --yes", "Answer yes to all prompts").option("-b --builder <webpack5 | vite>", "Builder library").option("-l --linkable", "Prepare installation for link (contributor helper)").action((options) => {
  initiate(options, pkg).catch(() => process.exit(1));
});
command("add <addon>").description("Add an addon to your Storybook").option(
  "--package-manager <npm|pnpm|yarn1|yarn2>",
  "Force package manager for installing dependencies"
).option("-N --use-npm", "Use NPM to install dependencies (deprecated)").option("-s --skip-postinstall", "Skip package specific postinstall config modifications").action((addonName, options) => add(addonName, options));
command("babelrc").description("generate the default storybook babel config into your current working directory").action(() => generateStorybookBabelConfigInCWD());
command("upgrade").description("Upgrade your Storybook packages to the latest").option(
  "--package-manager <npm|pnpm|yarn1|yarn2>",
  "Force package manager for installing dependencies"
).option("-N --use-npm", "Use NPM to install dependencies (deprecated)").option("-y --yes", "Skip prompting the user").option("-n --dry-run", "Only check for upgrades, do not install").option("-t --tag <tag>", "Upgrade to a certain npm dist-tag (e.g. next, prerelease)").option("-p --prerelease", "Upgrade to the pre-release packages").option("-s --skip-check", "Skip postinstall version and automigration checks").option("-c, --config-dir <dir-name>", "Directory where to load Storybook configurations from").action(async (options) => upgrade(options).catch(() => process.exit(1)));
command("info").description("Prints debugging information about the local environment").action(() => {
  consoleLogger.log(chalk12.bold("\nEnvironment Info:"));
  envinfo.run({
    System: ["OS", "CPU"],
    Binaries: ["Node", "Yarn", "npm"],
    Browsers: ["Chrome", "Edge", "Firefox", "Safari"],
    npmPackages: "@storybook/*",
    npmGlobalPackages: "@storybook/*"
  }).then(consoleLogger.log);
});
command("migrate [migration]").description("Run a Storybook codemod migration on your source files").option("-l --list", "List available migrations").option("-g --glob <glob>", "Glob for files upon which to apply the migration", "**/*.js").option("-p --parser <babel | babylon | flow | ts | tsx>", "jscodeshift parser").option(
  "-n --dry-run",
  "Dry run: verify the migration exists and show the files to which it will be applied"
).option(
  "-r --rename <from-to>",
  'Rename suffix of matching files after codemod has been applied, e.g. ".js:.ts"'
).action((migration, { configDir, glob: glob2, dryRun, list, rename, parser }) => {
  migrate(migration, {
    configDir,
    glob: glob2,
    dryRun,
    list,
    rename,
    parser,
    logger: consoleLogger
  }).catch((err) => {
    logger.error(err);
    process.exit(1);
  });
});
command("extract [location] [output]").description("extract stories.json from a built version").action(
  (location = "storybook-static", output = path9.join(location, "stories.json")) => extract(location, output).catch((e) => {
    logger.error(e);
    process.exit(1);
  })
);
command("sandbox [filterValue]").alias("repro").description("Create a sandbox from a set of possible templates").option("-o --output <outDir>", "Define an output directory").option("-b --branch <branch>", "Define the branch to download from", "next").option("--no-init", "Whether to download a template without an initialized Storybook", false).action(
  (filterValue, options) => sandbox({ filterValue, ...options }).catch((e) => {
    logger.error(e);
    process.exit(1);
  })
);
command("link <repo-url-or-directory>").description("Pull down a repro from a URL (or a local directory), link it, and run storybook").option("--local", "Link a local directory already in your file system").option("--no-start", "Start the storybook", true).action(
  (target, { local, start }) => link({ target, local, start }).catch((e) => {
    logger.error(e);
    process.exit(1);
  })
);
command("automigrate [fixId]").description("Check storybook for known problems or migrations and apply fixes").option("-y --yes", "Skip prompting the user").option("-n --dry-run", "Only check for fixes, do not actually run them").option("--package-manager <npm|pnpm|yarn1|yarn2>", "Force package manager").option("-N --use-npm", "Use npm as package manager (deprecated)").option("-l --list", "List available migrations").option("-c, --config-dir <dir-name>", "Directory of Storybook configurations to migrate").option("-s --skip-install", "Skip installing deps").option(
  "--renderer <renderer-pkg-name>",
  "The renderer package for the framework Storybook is using."
).action(async (fixId, options) => {
  await automigrate({ fixId, ...options }).catch((e) => {
    logger.error(e);
    process.exit(1);
  });
});
command("dev").option("-p, --port <number>", "Port to run Storybook", (str) => parseInt(str, 10)).option("-h, --host <string>", "Host to run Storybook").option("-s, --static-dir <dir-names>", "Directory where to load static files from", parseList).option("-c, --config-dir <dir-name>", "Directory where to load Storybook configurations from").option(
  "--https",
  "Serve Storybook over HTTPS. Note: You must provide your own certificate information."
).option(
  "--ssl-ca <ca>",
  "Provide an SSL certificate authority. (Optional with --https, required if using a self-signed certificate)",
  parseList
).option("--ssl-cert <cert>", "Provide an SSL certificate. (Required with --https)").option("--ssl-key <key>", "Provide an SSL key. (Required with --https)").option("--smoke-test", "Exit after successful start").option("--ci", "CI mode (skip interactive prompts, don't open browser)").option("--no-open", "Do not open Storybook automatically in the browser").option("--loglevel <level>", "Control level of logging during build").option("--quiet", "Suppress verbose build output").option("--no-version-updates", "Suppress update check", true).option("--debug-webpack", "Display final webpack configurations for debugging purposes").option("--webpack-stats-json [directory]", "Write Webpack Stats JSON to disk").option(
  "--preview-url <string>",
  "Disables the default storybook preview and lets your use your own"
).option("--force-build-preview", "Build the preview iframe even if you are using --preview-url").option("--docs", "Build a documentation-only site using addon-docs").option(
  "--initial-path [path]",
  "URL path to be appended when visiting Storybook for the first time"
).action(async (options) => {
  logger.setLevel(program.loglevel);
  consoleLogger.log(chalk12.bold(`${pkg.name} v${pkg.version}`) + chalk12.reset("\n"));
  getEnvConfig(options, {
    port: "SBCONFIG_PORT",
    host: "SBCONFIG_HOSTNAME",
    staticDir: "SBCONFIG_STATIC_DIR",
    configDir: "SBCONFIG_CONFIG_DIR",
    ci: "CI"
  });
  if (parseInt(`${options.port}`, 10)) {
    options.port = parseInt(`${options.port}`, 10);
  }
  await dev({ ...options, packageJson: pkg }).catch(() => process.exit(1));
});
command("build").option("-s, --static-dir <dir-names>", "Directory where to load static files from", parseList).option("-o, --output-dir <dir-name>", "Directory where to store built files").option("-c, --config-dir <dir-name>", "Directory where to load Storybook configurations from").option("--quiet", "Suppress verbose build output").option("--loglevel <level>", "Control level of logging during build").option("--debug-webpack", "Display final webpack configurations for debugging purposes").option("--webpack-stats-json [directory]", "Write Webpack Stats JSON to disk").option(
  "--preview-url <string>",
  "Disables the default storybook preview and lets your use your own"
).option("--force-build-preview", "Build the preview iframe even if you are using --preview-url").option("--docs", "Build a documentation-only site using addon-docs").action(async (options) => {
  process.env.NODE_ENV = process.env.NODE_ENV || "production";
  logger.setLevel(program.loglevel);
  consoleLogger.log(chalk12.bold(`${pkg.name} v${pkg.version}
`));
  getEnvConfig(options, {
    staticDir: "SBCONFIG_STATIC_DIR",
    outputDir: "SBCONFIG_OUTPUT_DIR",
    configDir: "SBCONFIG_CONFIG_DIR"
  });
  await build({ ...options, packageJson: pkg }).catch(() => process.exit(1));
});
program.on("command:*", ([invalidCmd]) => {
  consoleLogger.error(
    " Invalid command: %s.\n See --help for a list of available commands.",
    invalidCmd
  );
  const availableCommands = program.commands.map((cmd) => cmd._name);
  const suggestion = availableCommands.find((cmd) => leven(cmd, invalidCmd) < 3);
  if (suggestion) {
    consoleLogger.info(`
 Did you mean ${suggestion}?`);
  }
  process.exit(1);
});
program.usage("<command> [options]").version(pkg.version).parse(process.argv);
